<?php namespace App\Yantrana\Base;

use Illuminate\Support\Facades\Route;
use YesSecurity;
use JavaScript;
use YesAuthority;
use Auth;
use Request;

use App\Yantrana\__Laraware\Core\CoreController;

abstract class BaseController extends CoreController
{
    /**
     * Prepare data for clideside.
     *
     * @return Response
     */
    public function prepareForBrowser()
    {
        // get all application routes.
        $routeCollection = Route::getRoutes();
        $routes = [];
        $index = 1;

        // if routes in application
        if (!empty($routeCollection)) {
            foreach ($routeCollection as $route) {
                if ($route->getName()) {
                    $routes[ $route->getName() ] = $route->uri();
                } else {
                    $routes[ 'unnamed_'.$index ] = $route->uri();
                }

                ++$index;
            }
        }
        
        $auth_token = !__isEmpty(config('app.yestoken.jti')) ? '&auth_token='.config('app.yestoken.jti') : '';

        return [
            '__appImmutables' => [
                'public_encryption_token' => YesSecurity::getPublicRsaKey(),
                'form_security_id' => YesSecurity::getFormSecurityID(),
                'routes' => $routes,
                'messages' => [
                    'validation' => trans('validation'),
                    'js_string' => trans('js-string'),
                ],
                'auth_info' =>  getUserAuthInfo(),
                'restrict_user_email_update' => getConfigurationSettings('restrict_user_email_update'),
                'config' => [],
                'publicApp' => Request::route()->getName() != 'manage.app' ? true : false,
                'availableRoutes' => YesAuthority::availableRoutes(),
                'getCurrency'  => getCurrency(),
                'getCurrencySymbol' =>  getCurrencySymbol(),
            ],
             '__appTemps' => [
                'stateViaRoute' => null
            ],
            'appConfig' => [
                'debug' => env('APP_DEBUG', false),
                'appBaseURL' => asset(''),
            ],
        ];
    }

    public function loadPublicView($viewName, $data = [])
    {
        $browserData = $this->prepareForBrowser();

        JavaScript::put($browserData);

        $output = view('public-master', $data)->nest('pageRequested', $viewName, $data)->render();

        if (!env('APP_DEBUG', false)) {
            $filters = array(
                '/<!--([^\[|(<!)].*)/' => '',  // Remove HTML Comments (breaks with HTML5 Boilerplate)
                '/(?<!\S)\/\/\s*[^\r\n]*/' => '',  // Remove comments in the form /* */
                '/\s{2,}/' => ' ', // Shorten multiple white spaces
                '/(\r?\n)/' => '',  // Collapse new lines
            );

            return preg_replace(
                    array_keys($filters),
                    array_values($filters),
                    $output
                );
        } else { // for clog

            $clogSessItemName = '__clog';

            if (isset($_SESSION[$clogSessItemName]) == true and !empty($_SESSION[$clogSessItemName])) {
                $responseData = [];

                $responseData['__dd'] = true;
                $responseData['__clogType'] = 'NonAjax';
                $responseData[$clogSessItemName] = $clogSessItemName;
                // set for response
                $responseData[$clogSessItemName] = $_SESSION[$clogSessItemName];
                //reset the __clog items in session
                $_SESSION[$clogSessItemName] = [];

                $output = $output.'<script>__globals.clog('.json_encode($responseData).')</script>';
            }
        }

        return $output;
    }

    public function loadMailView($viewName, $data = [])
    {
        return view('emails.index', $data)->nest('pageMailRequested', $viewName, $data);
    }
}
