<?php
/*
* BillEngine.php - Main component file
*
* This file is part of the Bill component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Bill;

use App\Yantrana\Base\BaseEngine;
 
use App\Yantrana\Components\Bill\Interfaces\BillEngineInterface;

use App\Yantrana\Components\Bill\Repositories\BillRepository;
use App\Yantrana\Components\Package\Models\PackageModel;
use App\Yantrana\Components\User\Repositories\UserRepository;
use App\Yantrana\Components\Transaction\Repositories\TransactionRepository;
use Carbon\Carbon;
use App\Yantrana\Components\Transaction\TransactionEngine;

class BillEngine extends BaseEngine implements BillEngineInterface 
{   
     
    /**
     * @var  BillRepository $billRepository - Bill Repository
     */
    protected $billRepository;

    /**
     * @var  UserRepository $userRepository - User Repository
     */
    protected $userRepository;

    /**
     * @var  TransactionEngine $transactionEngine - Transaction Engine
     */
    protected $transactionEngine;
    

    /**
      * Constructor
      *
      * @param  BillRepository $billRepository - Bill Repository
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    function __construct(
        BillRepository $billRepository,
        UserRepository $userRepository,
        TransactionEngine $transactionEngine)
    {
        $this->billRepository = $billRepository;
        $this->userRepository = $userRepository;
        $this->transactionEngine = $transactionEngine;
    }

    /**
     * Lastly generated bils uids 
     */
    private function lastBillGeneratedUids() 
    {
        $lastBills = $this->billRepository->fetchLastBills();
            
        $lastBillsUids = [];

        if (!__isEmpty($lastBills)) {

            foreach($lastBills as $lastBill) 
            {
                if (!__isEmpty($lastBill->__data) and array_has($lastBill->__data, 'last_bill_uid')) {
        
                        $lastBillsUids[] = $lastBill->__data['last_bill_uid']; 
                }
            }
        }

        return $lastBillsUids;
    }

    /**
     * Get bill total amount
     */
    private function calculateBillAmount($billAmount, $taxAmount, $discountAmount) 
    {
        return ($billAmount - $discountAmount) + $taxAmount;
    }
    
  /**
    * Bill datatable source 
    *
    * @param integer $status
    *
  	* @return  array
  	*---------------------------------------------------------------- */
 
	public function prepareBillDataTableSource($status)
	{   
        $status = (int) $status;

        /* if ($status === 2) { // Upcomming
            
            $billCollection = $this->billRepository->fetchBillDataTableSource($status, $this->lastBillGeneratedUids());

            $config = configItem('packages.frequencies');

            $requireColumns = [
                '_id',
                '_uid',
                'user_package_uid',
                'name',
                'package_title',
                'bill_uid' => function($key) {
                    return generateBillNo($key['_uid']);
                },
                'bill_period' => function($key) {
                    
                    $bill_period_from = formatDateTime($key['bill_period_from'], 'Y-m-d');
                    $bill_period_to = formatDateTime($key['bill_period_to'], 'Y-m-d');
                    
                    if ($bill_period_from === $bill_period_to) {
                        return formatDateTime($key['bill_period_from'], 'l jS F Y');
                    }

                    return formatDateTime($key['bill_period_from'], 'jS F Y').' <strong>To</strong> '.formatDateTime($key['bill_period_to'], 'jS F Y');

                },
                'formated_amount' => function($key) {
                    return moneyFormat($key['amount'], true, false);
                },
                'currentTab' => function($key) use($status) {
                    return $status;
                },
                'formated_due_date' => function($key) {
                    return formatDateTime($key['due_date'], 'jS F Y');
                },
                'formated_frequency' => function($key) use($config) {
                    return configItemString($config, $key['frequency']);
                },
            ];
    
            return $this->dataTableResponse($billCollection, $requireColumns);

        } else { */

            $billCollection = $this->billRepository->fetchBillsDatatable($status);

            $config = configItem('bills.status');

            $requireColumns = [
                '_id',
                '_uid',    
                'name',
                'status',
                'bill_uid' => function($key) {
                    return generateBillNo($key['bill_uid']);
                },
                'formatted_status' => function($key) use($config) {
                    return configItemString($config, $key['status']);
                },
                'formated_amount' => function($key) {
                    return moneyFormat($key['amount'], true, false);
                },
                'generated_at' => function($key) {
                    return formatDateTime($key['created_at']);
                },
                'bill_period' => function($key) {

                    $bill_period_from = formatDateTime($key['bill_period_from'], 'Y-m-d');
                    $bill_period_to = formatDateTime($key['bill_period_to'], 'Y-m-d');

                    if ($bill_period_from === $bill_period_to) {
                        return formatDateTime($key['bill_period_from'], 'jS F Y');
                    }

                    return formatDateTime($key['bill_period_from'], 'jS F Y').' <strong>TO</strong> '.formatDateTime($key['bill_period_to'], 'jS F Y');

                },
                'formated_due_date' => function($key) {
                    return formatDateTime($key['due_date'], 'jS F Y');
                },
                'formated_discount_amount' => function($key) {
                    return moneyFormat($key['discount_amount'], true, false);
                },
                'formated_tax_amount' => function($key) {
                    return moneyFormat($key['tax_amount'], true, false);
                },
                'printBillUrl' => function($key) {
                    if ($key['status'] == 3) {
                        return route('manage.bill.read.print_bill', $key['bill_uid']);
                    }
                    return '';
                },
                'currentTab' => function($key) use($status) {
                    return $status;
                },
                'isOverDue' => function($key) {
                    return $key['due_date'] < now()->format('Y-m-d');
                },
                'user_packages_uid' => function($key) {

                    if ($key['user_package']) {
                        return $key['user_package']['_uid'];
                    }

                    return '';
                },
                'view_bill_details' => function($key) {
                	return canAccess('view_bill_details');
                },
                'cancel_bill' => function($key) {
                	return canAccess('manage.bill.write.process_cancel');
                },
                'delete_bill' => function($key) {
                	return canAccess('manage.bill.write.process_delete');
                },
                'edit_bill' => function($key) {
                	return canAccess('edit_bill');
                },
                'pay_bill' => function($key) {
                	return canAccess('pay_bill');
                }
            ];

            return $this->dataTableResponse($billCollection, $requireColumns);
        /* } */
    }
    

    /**
	  * Bill Generate
	  *
	  * @param array $inputData
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function processGenerate($request)
	{
	    try{
	    $inputData = $request->all();
	    $inputData['client_id'] = $request->user()->client_id;

        $customer = $this->userRepository->fetchUser($inputData['user']);

        if (__isEmpty($customer)) {
            return $this->engineReaction(18, null, 'Customer data not found.');
        }

        $userpackage = $this->billRepository->fetchPurchasedPackage($inputData['user_packages_id']);
//        $userpackage = PackageModel::where('_id',$inputData['package'])->first();
        if (__isEmpty($userpackage)) {
            return $this->engineReaction(18, null, 'Generating bill for package not exists.');
        }

        $response = $this->billRepository->processTransaction(function() 
            use($inputData, $customer, $userpackage) {

            $billData = [
                'amount' => $inputData['total_bill_amount'],
                'user_authorities__id' => getUserAuthorityId(),
                'client_id'=> $inputData['client_id'],
                'member_id'=> $inputData['user'],
                'short_description' => $inputData['short_description'] ?? '',
                'customer_user_authorities__id' => $inputData['customer_user_authorities__id'],
                'due_date' => $inputData['due_date'],
                'discount_amount' => $inputData['discount_amount'],
                'tax_amount' => $inputData['tax_amount'],
                'bill_period_from' => $inputData['bill_period_from'],
                'bill_period_to' => $inputData['bill_period_to'],
                'status' => 1,
                '__data' => [
                    'customer_data' => $customer->toArray(),
                    'purchased_package' => $userpackage->toArray()
                ]
            ];

            if (!$bill = $this->billRepository->storeBill($billData)) {

                return $this->billRepository->transactionResponse(2, null, __tr('Bill not generated.'));
            }

            $billId = $bill->_id;

            activityLog(9, $billId, 11);

            
            $insertItems = [];

            foreach($inputData['items'] as $key => $item) {

                $insertItems[] = [
                	'bills__id' => $billId,
					'status' => 1,
					'short_description' => $item['short_description'],
					'amount' => $item['amount'],
					'user_packages__id' => $item['user_packages_id'] ?? $item['user_packages__id']
//					'user_packages__id' => $item['user_packages_id'] ?? $item['user_packages__id']
                ];
            }

            if (!$billItems = $this->billRepository->insert($insertItems)) {

                return $this->billRepository->transactionResponse(2, null, __tr('Bill not generated.'));
            }
    
            activityLog(10, $billId, 11);

            return $this->billRepository->transactionResponse(1, [
                'bill_data' => [
                    'bill_no'           => generateBillNo($bill->_uid),
                    'bill_uid'          => $bill->_uid,
                    'due_date'          => formatDateTime($bill->due_date, 'l jS F Y'),
                    'bill_period_from'  => formatDateTime($bill->bill_period_from, 'l jS F Y'),
                    'bill_period_to'    => formatDateTime($bill->bill_period_to, 'l jS F Y'),
                    'status'      => configItemString(configItem('bills.status'), $bill->status),
                    'amount'      => moneyFormat($bill->amount, true, true),
                ],
            ], 'Bill Generated Sucessfully.');

        });

        return $this->engineReaction($response);
        }catch (\Exception $exception){
            return $this->engineReaction(18, null, $exception->getMessage());
        }
    }
    
    /**
    * Read bill Details
    *
    * @param string $billUid
    *
    * @return array
    *---------------------------------------------------------------- */

	public function prepareDetails($billUid)
	{   
        $bill = $this->billRepository->fetchDetails($billUid);

        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists');
        }
        
        $config = configItem('bills.status');

        $data = [
            'bill_period_from' => formatDateTime($bill->bill_period_from, 'l jS F Y'),
            'bill_period_to' => formatDateTime($bill->bill_period_to, 'l jS F Y'),
            'bill_uid' => generateBillNo($bill->_uid),
            'formatted_status' => configItemString($config, $bill->status),
            'formated_bill_amount' =>moneyFormat($bill->amount, true, true),
            'generated_at' =>  formatDateTime($bill->created_at),
            'formated_due_date' => formatDateTime($bill->due_date, 'l jS F Y'),
            'formated_discount_amount' => moneyFormat($bill->discount_amount, true, false),
            'formated_tax_amount' =>moneyFormat($bill->tax_amount, true, false),
            'formated_tax_amount' =>moneyFormat($bill->tax_amount, true, false),
            'print_bill_url' => ($bill->status == 3) ?  route('manage.bill.read.print_bill', $bill['_uid']) : ''
        ];

        $userPackage = $itemsAmount = [];

        foreach($bill->billItems as $item) {

            $data['items'][] = [
                'short_description' => $item->short_description,
                'amount' => moneyFormat($item->amount, true, false),
            ];

            $itemsAmount[] = $item->amount;
            
            $userPackage = $item->userPackage;
        }

        $package = [];

        if ($userPackage) {

            $package = [
                'frequency' => $userPackage->frequency,
                'formated_frequency'  => configItemString('packages.frequencies', $userPackage->frequency),
                'formated_started_at' => formatDateTime($userPackage->starts_at, 'l jS F Y')
            ];
    
        }

        $data['package'] = $package;

        $transactions = [];

        if ($bill->transactions) {

            $tranConfig = configItem('transaction');

            foreach($bill->transactions as $transaction) {

                $transactions[] = [
                    'txn_id' => $transaction->txn_id,
                    'amount' => moneyFormat($transaction->amount, true, false),
                    'created_at' => formatDateTime($transaction->created_at),
                    'formatted_type' => configItemString($tranConfig['types'], $transaction->type),
                    'formatted_status' => configItemString($tranConfig['status'], $transaction->status),
                    'formatted_subtype' => configItemString($tranConfig['subtypes'], $transaction->subtype),
                    'payment_method' => configItemString($tranConfig['payment_methods'], $transaction->payment_method)
                ];
            }
        }

        $data['transactions'] = $transactions;
        $data['items_amount'] = moneyFormat(array_sum($itemsAmount), true, false);

        return $this->engineReaction(1, [
            'data' =>  $data
        ]);

    }

    /**
    * Read bill Details
    *
    * @param string $billUid
    *
    * @return array
    *---------------------------------------------------------------- */

	public function prepareBillEditDetails($billUid)
	{
        $bill = $this->billRepository->fetchDetails($billUid);

        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists');
        }
 
        $data = [
			'bill_period_from'	=> $bill->bill_period_from,
			'bill_period_to'	=> $bill->bill_period_to,
			'due_date'			=> $bill->due_date,
			'short_description'	=> $bill->short_description,
			'bill_no' 			=> generateBillNo($bill->_uid),
			'bill_uid' 			=> $bill->_uid,
			'bill_amount' 		=> $bill->amount,
			'tax_amount' 		=> $bill->tax_amount,
			'discount_amount' 	=> $bill->discount_amount,
            'total_item_amount' => $bill->amount
        ];

        $c = 1; 
        $userPackageId = '';

        foreach($bill->billItems as $item) {
            
            $userPackageId = $item->user_packages__id;
            $data['items'][] = [
                'rowId' => "row$c",
                'short_description' => $item->short_description,
                'amount' => $item->amount,
                'bill_item_id' => $item->_id,
                'bill_item_uid' => $item->_uid,
                'user_packages__id' => $userPackageId,
            ];

            $c++;
        }

        $data['user_packages__id'] = $userPackageId;

        return $this->engineReaction(1, [
            'data' =>  $data
        ]);

    }

	/**
	  * Prepare Print Bill Data
	  *
	  * @param string $billUid
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function prepareBillData($billUid)
	{      
     	$bill = $this->billRepository->preparePrintBillData($billUid);

		if (!__isEmpty($bill)) {
 			
 			$billItems = [];
 			$packageAmt = 0;
 			if (!__isEmpty($bill->billItems)) {
 				foreach ($bill->billItems as $key => $billitem) {
 					if (!__isEmpty($billitem->userPackage) && !__isEmpty($billitem->userPackage->package)) {
 						$packageAmt = $billitem->amount + $packageAmt;
						$billItems[] = [
							'package_title' 	=> $billitem->userPackage->package->title,
							'package_amount' 	=> moneyFormat($billitem->amount, true, false)
						];
 					}
 				}
 			}

 			$tax = isset($bill->tax_amount) ? $bill->tax_amount : 0;
 			$discount = isset($bill->discount_amount) ? $bill->discount_amount : 0;
 			$totalAmount = (($packageAmt + $tax) - $discount);
 
			$billData = [
				'amount'			=> $bill->amount,
				'bill_no'			=> $bill->_uid,
				'short_description'	=> $bill->short_description,
				'due_date'			=> formatDateTime($bill->due_date, 'l jS F Y'),
				'discount_amount'	=> moneyFormat($bill->discount_amount, true, false),
				'tax_amount'		=> moneyFormat($bill->tax_amount, true, false),
				'customer_name'		=> $bill->customer_name,
				'mobile_number'		=> $bill->mobile_number,
				'address_line_1'	=> $bill->address_line_1,
				'address_line_2'	=> $bill->address_line_2,
				'country_name'		=> $bill->country_name,
				'bill_items'		=> $billItems,
				'totalPayableAmount'=> moneyFormat($totalAmount, true, false),
				'totalAmount'		=> moneyFormat($packageAmt, true, false),
				'store_name'		=> getConfigurationSettings('name'),
				'store_address'		=> getConfigurationSettings('company_address')
	        ];
		}
     	
     	
        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists');
        }

        return $this->engineReaction(1, [
            'billData' =>  $billData
        ]);
    }


    /**
    * Process Cancel bill
    *
    * @param string $billUid
    *
    * @return array
    *---------------------------------------------------------------- */

	public function processCancel($billUid)
	{   
        $bill = $this->billRepository->fetch($billUid);

        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists.');
        }

        // Check if the bill status not unpaid then return response 
        if ($bill->status != 1) {
            return $this->engineReaction(2, null, "Transactions are present related to this bill that's why can't cancel bill.");
        }

        if ($this->billRepository->update($bill, ['status' => 4])) {

            activityLog(9, $bill->_id, 10);

            return $this->engineReaction(1, null, 'Bill cancelled sucessfully.');
        }

        return $this->engineReaction(18, null, 'Bill not cancelled');

    }

    /**
    * Process Cancel bill
    *
    * @param string $billUid
    *
    * @return array
    *---------------------------------------------------------------- */

	public function processDelete($billUid)
	{   
        $bill = $this->billRepository->fetch($billUid);

        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists.');
        }

        // Check if the bill status not unpaid then return response 
        if ($bill->status != 1) {
            return $this->engineReaction(2, null, "Transactions are present related to this bill that's why can't delete bill.");
        }

        $billId = $bill->_id;

        if ($this->billRepository->delete($bill)) {

            activityLog(9, $billId, 3);

            return $this->engineReaction(1, null, 'Bill deleted sucessfully.');
        }

        return $this->engineReaction(18, null, 'Bill not deleted');

    }

    /**
    * Read bill Details
    *
    * @param string $billUid
    *
    * @return array
    *---------------------------------------------------------------- */

	public function readBillPaymentDetails($billUid)
	{   
        $bill = $this->billRepository->fetchDetails($billUid);

        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists');
        }

        $transaction_amount = 0;

        if (!__isEmpty($bill->transactions)) {

            foreach($bill->transactions as $transaction) {
            	if ($transaction->status == 2) {
            		$transaction_amount = $transaction->amount + $transaction_amount;
            	}
            }
        }

        $paymentData = $this->transactionEngine->preparBillPaymentInfo($bill->_id, $bill->amount);

        $paymentData['bill_uid'] = $bill->_uid;
        $paymentData['bill_no']  = generateBillNo($bill->_uid);
        $paymentData['f_bill_amount'] = moneyFormat($paymentData['bill_amount'], true, false);
        $paymentData['f_paid_amount'] = moneyFormat($paymentData['paid_amount'], true, false);
        $paymentData['f_balance_amount'] = moneyFormat($paymentData['balance_amount'], true, false);
        $paymentData['paying_amount'] = $paymentData['balance_amount'];

        return $this->engineReaction(1, [
            'paymentData' =>  $paymentData,
            'payment_methods' => configItem('transaction.payment_methods')
        ]);

    }

    /**
	  * Bill Generate
	  *
	  * @param array $inputData
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function processBillEdit($inputData, $billUid)
	{
        $bill = $this->billRepository->fetchDetails($billUid);

        if (__isEmpty($bill)) {
            return $this->engineReaction(18, null, 'Bill not exists');
        }

        if ($bill->status !== 1) {
            return $this->engineReaction(2, null, "You can't update bill");
        }

        if (__isEmpty($inputData['items'])) {
            return $this->engineReaction(2, null, 'Bill items empty');
        }

        $existingAndNewInputItems = $newItems = $removedItems = $existingItem = $billItemsAmount = [];
        
        foreach($inputData['items'] as $key => $item) 
        {
            $inputBillItemUid = $item['bill_item_uid'] ?? '';
            unset($item['rowId']);

            if (__isEmpty($inputBillItemUid)) { 

                $item['status'] = 1;
                $item['bills__id'] = $bill->_id;
                $newItems[] = $item;
            }

            if (!__isEmpty($inputBillItemUid)) {

                $existingAndNewInputItems[$inputBillItemUid] = $item; 
            }

            $billItemsAmount[] = $item['amount'];
        }

        $c = 0;

        foreach($bill->billItems as $bItem) 
        {
            if (!array_key_exists($bItem['_uid'], $existingAndNewInputItems)) {
                $removedItems[] = $bItem['_id'];
            }

            if (array_key_exists($bItem['_uid'], $existingAndNewInputItems)) {

                $row = $existingAndNewInputItems[$bItem['_uid']];

                $existingItem[] = [
                    '_id'               => $bItem['_id'],
                    '_uid'              => $bItem['_uid'],
                    'short_description' => $row['short_description'],
                    'amount'            => $row['amount']
                ];
            }
        }

        $response = $this->billRepository->processTransaction(function() 
            use($inputData, $newItems, $removedItems, $bill, $billItemsAmount, $existingItem) {

            $billData = [
                'amount' => $this->calculateBillAmount(array_sum($billItemsAmount), $inputData['tax_amount'], $inputData['discount_amount']),
                'discount_amount' => $inputData['discount_amount'],
                'tax_amount' => $inputData['tax_amount'],
                'bill_period_from' => $inputData['bill_period_from'],
                'bill_period_to' => $inputData['bill_period_to'],
                'short_description' => $inputData['short_description'] ?? '',
                'due_date' => $inputData['due_date'],
            ];

            $billId = $bill->_id;

            if (!__isEmpty($newItems)) {

	            if (!$billItems = $this->billRepository->insert($newItems)) {
	                return $this->billRepository->transactionResponse(2, null, __tr('Bill not updated.'));
	            }
            }

            if (!__isEmpty($removedItems)) {

	            if (!$billItemsDeleted = $this->billRepository->deleteBillItems($removedItems)) {

	                return $this->billRepository->transactionResponse(2, null, __tr('Bill not updated.'));
	            }
            }

            $isUpdate = true;

            if ($billEdited = $this->billRepository->update($bill, $billData)) {
                $isUpdate = true;
            }


            if ($this->billRepository->bunchUpdateBillItems($existingItem)) {
                $isUpdate = true;
            }

            if ($isUpdate) {
    
                return $this->billRepository->transactionResponse(1, [
                    'bill' => [
                        '_uid'    => $bill->_uid,
                        'bill_number' => generateBillNo($bill->_uid),
                        'amount' => $bill->amount
                    ]
                ], 'Bill updated sucessfully.');
            }

            return $this->billRepository->transactionResponse(2, null, __tr('Bill not updated.'));

        });

        return $this->engineReaction($response);

    }

    /**
    * prepare Upcomming Bills
    *
    * @return  array
    *---------------------------------------------------------------- */

	public function prepareUpcommingBills()
	{   
        $upcommingBills = $this->billRepository->fetchUpcommingBills($this->lastBillGeneratedUids());
        
        $billData = [];

        if (!__isEmpty($upcommingBills)) {

            $config = configItem('bills.status');

            foreach($upcommingBills as $key => $data) 
            {   
                $frequency = $data->frequency;
                $bill_period_to = $data->bill_period_to;
                $dueDate = formatDateTime(now(), 'l jS F Y');

                if ($frequency !== 1) { // Daily
                    $dueDate = formatDateTime(Carbon::parse($data->bill_period_to)->addDays(getConfigurationSettings('generate_bill_before_day'), 'l jS F Y'));
                }

                $billData[$key] = [
                    'amount' => moneyFormat($data->price, true, true),
                    'customer' => $data->name,
                    'user_authorities__id' => $data->user_authorities__id,
                    'short_description' => '',
                    'customer_user_authorities__id' => $data->customer_user_authorities__id,
                    'discount_amount' => moneyFormat(0, true, false),
                    'tax_amount' =>  moneyFormat(0, true, false),
                    'bill_period_from' => formatDateTime($bill_period_to, 'l jS F Y'),
                    'bill_period_to' => formatDateTime(calcFrequencyEndDate($bill_period_to, $frequency), 'l jS F Y'),
                    'due_date' => $dueDate,
                    'status' => configItemString($config, 1),
                    'formated_frequency' => configItemString('packages.frequencies', $frequency),
                    'items' => [
                        'short_description' => $data->package_title."<br>".$data->package_short_description ?? '',
                        'amount' =>  moneyFormat($data->price, true, false),
                        'user_packages__id' => $data->user_package_id
                    ]
                ];
            }
    
        }

        return $this->engineReaction(1, [
            'data' => $billData
        ]);

    }

    /**
    * Process Generate Upcomming Bills
    *
    * @return  array
    *---------------------------------------------------------------- */

	public function processGenerateUpcommingBills()
	{   
        $upcommingBills = $this->billRepository->fetchUpcommingBills($this->lastBillGeneratedUids());
        
        if (__isEmpty($upcommingBills)) {
            return $this->engineReaction(18, null, 'No upcomming bills found.');
        }

        $billData = [];

        foreach($upcommingBills as $key => $data) 
        {   
            $frequency = $data->frequency;

            $dueDate = now()->format('Y-m-d');

            if ($frequency !== 1) { // Daily
                $dueDate = Carbon::parse($data->bill_period_to)->addDays(getConfigurationSettings('generate_bill_before_day'));
            }

            $billData[$key] = [
                'amount' => $data->price,
                'user_authorities__id' => $data->user_authorities__id,
                'short_description' => '',
                'customer_user_authorities__id' => $data->customer_user_authorities__id,
                'due_date' => $dueDate,
                'discount_amount' => 0,
                'tax_amount' => 0,
                'bill_period_from' => $data->bill_period_to,
                'bill_period_to' => calcFrequencyEndDate($data->bill_period_to, $data->frequency),
                'status' => 1,
                '__data' => [
                    'customer_data'     => $this->userRepository->fetchUserWithProfile($data->user_id)->toArray(),
                    'purchased_package' => $this->billRepository->fetchPurchasedPackage($data->user_package_id)->toArray(),
                    'last_bill_uid' => $data->bill_uid
                ],
                'items' => [
                    'short_description' => $data->package_title."\r\n".$data->package_short_description ?? '',
                    'amount' => $data->price,
                    'user_packages__id' => $data->user_package_id
                ]
            ];
        }

        $response = $this->billRepository->processTransaction(function() use($billData) {

            if (!$this->billRepository->insertBillWithItems($billData)) {
                return $this->billRepository->transactionResponse(18, null, 'Upcomming bills not generated');
            }

            return $this->billRepository->transactionResponse(1, null, 'Upcomming bills generated sucessfully.');

        });

        return $this->engineReaction($response);

    }
}