<?php
/*
* BillController.php - Controller file
*
* This file is part of the Bill component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Bill\Controllers;

use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Bill\BillEngine;
use App\Yantrana\Components\Bill\Requests\BillAddRequest;
use App\Yantrana\Components\Bill\Requests\BillEditRequest;


class BillController extends BaseController 
{    
    /**
     * @var  BillEngine $billEngine - Bill Engine
     */
    protected $billEngine;

    /**
      * Constructor
      *
      * @param  BillEngine $billEngine - Bill Engine
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    function __construct(BillEngine $billEngine)
    {
        $this->billEngine = $billEngine;
    }

     
    /**
      * list of Bill
      *
      * @return  json object
      *---------------------------------------------------------------- */

  	public function prepareBillList($status) 
  	{
      	return $this->billEngine->prepareBillDataTableSource($status);
      }
      

    /**
    * Process generate bill
    * 
    * @param  mix $itemIdOrUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function processGenerate(BillAddRequest $request)
	{
	    $processReaction = $this->billEngine->processGenerate($request);

    	return __processResponse($processReaction, [], [], true);
    }
    
    /**
    * Process generate bill
    * 
    * @param  mix $itemIdOrUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function processBillEdit(BillEditRequest $request, $billUid)
	{   
    	$processReaction = $this->billEngine
                            	->processBillEdit($request->all(), $billUid);

    	return __processResponse($processReaction, [], [], true);
    }

    /**
    * Prepare Bill details
    * 
    * @param  mix $itemIdOrUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function readDetails($billUid)
	{
    	$processReaction = $this->billEngine->prepareDetails($billUid);

    	return __processResponse($processReaction, [], [], true);
	}

	 /**
    * Prepare Bill details
    * 
    * @param  mix $itemIdOrUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function readBillEditDetails($billUid)
	{
    	$processReaction = $this->billEngine->prepareBillEditDetails($billUid);

    	return __processResponse($processReaction, [], [], true);
	}

	/**
    * Prepare Bill details
    * 
    * @param  mix $itemIdOrUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function readBillPaymentDetails($billUid)
	{   
    	$processReaction = $this->billEngine->readBillPaymentDetails($billUid);

    	return __processResponse($processReaction, [], [], true);
	}

	/**
      * Print Bill
      *
      * @param string $billUid
      *
      * @return  array
      *---------------------------------------------------------------- */
    public function printBill($billUid)
    {
        $processResponse = $this->billEngine->prepareBillData($billUid);

        return view('bill.print-bill', ['bill' => $processResponse['data']['billData'] ]);
    }

    /**
    * Process Cancell
    * 
    * @param  mix $billUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function processCancel($billUid)
	{   
    	$processReaction = $this->billEngine->processCancel($billUid);

    	return __processResponse($processReaction, [], [], true);
    }
    
    /**
    * Process Delete
    * 
    * @param  mix $billUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function processDelete($billUid)
	{   
    	$processReaction = $this->billEngine->processDelete($billUid);

    	return __processResponse($processReaction, [], [], true);
    }
    
    /**
    * Process Read upcomming bills
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function readUpcommingBills()
	{   
    	$processReaction = $this->billEngine->prepareUpcommingBills();

    	return __processResponse($processReaction, [], [], true);
	}
}