<?php
/*
* Bill.php - Model file
*
* This file is part of the Bill component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Bill\Models;

use App\scopes\ClientScopeFilter;
use App\Yantrana\Base\BaseModel;
use App\Yantrana\Components\Bill\Models\BillItemsModel;
use App\Yantrana\Components\Transaction\Models\TransactionModel;
use App\Yantrana\Components\Package\Models\UserPackageModel;

class BillModel extends BaseModel
{
    use ClientScopeFilter;

    /**
     * @var  string $table - The database table used by the model.
     */
    protected $table = "bills";

    /**
     * @var  array $casts - The attributes that should be casted to native types.
     */
    protected $casts = [
        'amount' => 'float',
        'user_authorities__id' => 'integer',
        'customer_user_authorities__id' => 'integer',
        'discount_amount' => 'float',
        'tax_amount' => 'float',
        '__data' => 'array'
    ];

    /**
     * @var  array $fillable - The attributes that are mass assignable.
     */
    protected $fillable = [
        'client_id',
        'member_id'
    ];


    /**
     * Does it has has Entity Ownership ID
     *
     * @var bool
     *----------------------------------------------------------------------- */
    protected $hasEoId = true;


    /**
     * Let the system knows Text columns treated as JSON
     *
     * @var array
     *----------------------------------------------------------------------- */
    protected $jsonColumns = [
        '__data' => [
            'customer_data' => 'array',
            'purchased_package' => 'array',
            'last_bill_uid' => 'string'
        ]
    ];

    /**
     * Get the comments for the blog post.
     */
    public function billItems()
    {
        return $this->hasMany(BillItemsModel::class, 'bills__id', '_id')->with('userPackage');
    }

    /**
     * Get the comments for the blog post.
     */
    public function transactions()
    {
        return $this->hasMany(TransactionModel::class, 'bills__id', '_id');
    }

    /**
     * Get the comments for the blog post.
     */
    public function userPackage()
    {
        return $this->hasOne(UserPackageModel::class, 'user_authorities__id', 'customer_user_authorities__id');
    }
}