<?php
/*
* ItemController.php - Controller file
*
* This file is part of the Item component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Item\Controllers;

use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Item\ItemEngine;
use App\Yantrana\Components\Item\Requests\{
                        ItemAddRequest, ItemEditRequest, ItemSlotAddRequest
                    };

class ItemController extends BaseController 
{    
    /**
     * @var  ItemEngine $itemEngine - Item Engine
     */
    protected $itemEngine;

    /**
      * Constructor
      *
      * @param  ItemEngine $itemEngine - Item Engine
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    function __construct(ItemEngine $itemEngine)
    {
        $this->itemEngine = $itemEngine;
    }
    
    
     
    /**
      * list of Item
      *
      * @return  json object
      *---------------------------------------------------------------- */

  	public function prepareItemList() 
  	{ 
      	return $this->itemEngine
                  	->prepareItemDataTableSource();
  	}
     
	/**
	  * Item process delete 
	  *
	  * @param  mix $itemIdOrUid
	  *
	  * @return  json object
	  *---------------------------------------------------------------- */

	public function processItemDelete($itemIdOrUid)
	{   
	    $processReaction = $this->itemEngine
	                            ->processItemDelete($itemIdOrUid);

	    return __processResponse($processReaction, [], [], true);
	}
 
    /**
      * Item Add Support Data 
      *
      * @return  json object
      *---------------------------------------------------------------- */

    public function prepareItemSupportData()
    {

        $processReaction = $this->itemEngine->prepareItemSupportData();

        return __processResponse($processReaction, [], [], true);
    }
 
	/**
	  * Item create process 
	  *
	  * @param  object ItemListRequest $request
	  *
	  * @return  json object
	  *---------------------------------------------------------------- */

	public function processItemCreate(ItemAddRequest $request)
	{   
	    $processReaction = $this->itemEngine
	                            ->processItemCreate($request);
        return __processResponse($processReaction, [], [], true);
	}

 	/**
	  * Item create process 
	  *
	  * @param  object ItemListRequest $request
	  *
	  * @return  json object
	  *---------------------------------------------------------------- */

	public function processItemSlotCreate(ItemSlotAddRequest $request, $itemIdOrUid)
	{   
	    $processReaction = $this->itemEngine->processItemSlotCreate($request, $itemIdOrUid);
	    
        return __processResponse($processReaction, [], [], true);
	}
    /**
  	  * Item get update data 
  	  *
  	  * @param  mix $itemIdOrUid
  	  *
  	  * @return  json object
  	  *---------------------------------------------------------------- */

	public function updateItemData($itemIdOrUid)
	{

    	$processReaction = $this->itemEngine
                            ->prepareItemUpdateData($itemIdOrUid);
    	return __processResponse($processReaction, [], [], true);
	}

	/**
  	  * Item process update 
  	  * 
  	  * @param  mix @param  mix $itemIdOrUid
  	  * @param  object ItemListRequest $request
  	  *
  	  * @return  json object
  	  *---------------------------------------------------------------- */

	public function processItemUpdate($itemIdOrUid, ItemEditRequest $request)
	{   
    	$processReaction = $this->itemEngine
                            	->processItemUpdate($itemIdOrUid, $request);

    	return __processResponse($processReaction, [], [], true);
    }
    

    /**
    * Item Details
    *
    * @param  mix $itemIdOrUid
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function details($itemIdOrUid)
	{
    	$processReaction = $this->itemEngine
                            ->prepareItemDetails($itemIdOrUid);

    	return __processResponse($processReaction, [], [], true);
	}
    

}