<?php
/*
* Item.php - Model file
*
* This file is part of the Item component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Item\Models;

use App\scopes\ClientScopeFilter;
use App\Yantrana\Base\BaseModel;
use App\Yantrana\Components\Slots\Models\SlotsModel;

class ItemModel extends BaseModel
{
    use ClientScopeFilter;

    /**
     * @var  string $table - The database table used by the model.
     */
    protected $table = "items";

    /**
     * Does it has has Entity Ownership ID
     *
     * @var bool
     *----------------------------------------------------------------------- */
    protected $hasEoId = true;


    /**
     * @var  array $casts - The attributes that should be casted to native types.
     */
    protected $casts = [
        'type' => 'integer',
        'user_authorities__id' => 'integer',
        'slot_type' => 'integer'
    ];

    /**
     * @var  array $fillable - The attributes that are mass assignable.
     */
    protected $fillable = [
        'client_id'
    ];

    /**
     * get package items associated with package
     */
    public function itemSlots()
    {
        return $this->hasMany(SlotsModel::class, 'items__id', '_id');
    }

    /**
     * get package items associated with package
     */
    public function itemSlot()
    {
        return $this->hasOne(SlotsModel::class, 'items__id', '_id');
    }

}