<?php
/*
* ItemRepository.php - Repository file
*
* This file is part of the Item component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Item\Repositories;

use App\Yantrana\Base\BaseRepository;
 
use App\Yantrana\Components\Item\Models\ItemModel;
use App\Yantrana\Components\Item\Interfaces\ItemRepositoryInterface;
use App\Yantrana\Components\Package\Models\PriceModel;

class ItemRepository extends BaseRepository
                          implements ItemRepositoryInterface 
{ 
    
    /**
      * Fetch the record of Item
      *
      * @param    int || string $idOrUid
      *
      * @return    eloquent collection object
      *---------------------------------------------------------------- */

    public function fetch($idOrUid)
    {   
        if (is_numeric($idOrUid)) {

            return ItemModel::where('_id', $idOrUid)->first();
        }

        return ItemModel::where('_uid', $idOrUid)->first();
    }
    
    
    
    /**
      * Fetch item datatable source
      *
      * @return  mixed
      *---------------------------------------------------------------- */
 
	public function fetchItemDataTableSource()
	{   
    	$dataTableConfig = [
        	'searchable' => [
                'title',            
                'short_description'
            ],            
            'fieldAlias' => [
                'formated_created'    => 'created_at',
                'formated_updated_at' => 'updated_at'
            ],
    	];

        return ItemModel::dataTables($dataTableConfig)->toArray();
	}

    /**
	  * Delete $item record and return response
	  *
	  * @param  array $inputData
	  *
	  * @return  mixed
	  *---------------------------------------------------------------- */

	public function deleteItem($item)
	{   
	    // Check if $item deleted
	    if ($item->delete()) {

	        return true;
	    }

	    return false;
	}

	/**
	  * Store new item record and return response
	  *
	  * @param  array $inputData
	  *
	  * @return  mixed
	  *---------------------------------------------------------------- */

	public function storeItem($inputData)
	{   
        $keyValues = [
            'title',         
            'short_description',
            'type',
            'user_authorities__id' => getUserAuthorityId(),
            'status'            
        ];

	    $newItem = new ItemModel(['client_id'=>$inputData['client_id']]);
	    
	    // Check if task testing record added then return positive response
	    if ($newItem->assignInputsAndSave($inputData, $keyValues)) {

	        return $newItem;
	    }

	    return false;
	}

    /**
  	  * Update item record and return response
  	  *
  	  * @param  object $item
  	  * @param  array $inputData
  	  *
  	  * @return  mixed
  	  *---------------------------------------------------------------- */

	public function update($item, $inputData)
	{       
    	// Check if item updated then return positive response
    	if ($item->modelUpdate($inputData)) {

        	return true;
    	}

    	return false;
	}

	/**
	  * Store new item record and return response
	  *
	  * @param  array $inputData
	  *
	  * @return  mixed
	  *---------------------------------------------------------------- */
	public function getAllItems()
	{   
		return ItemModel::get();
    }
    
	/**
	  * fetch all services
	  *
	  * @param  array $inputData
	  *
	  * @return  mixed
	  *---------------------------------------------------------------- */

	public function fetchAllServices()
	{   
		return ItemModel::leftjoin('prices', 'items._id', '=', 'prices.items__id')
					->with('itemSlots')
					->where('items.type', '=', 1)
    				->select(__nestedKeyValues([
    					'items.*',
						'prices' => [
							'items__id',
							'price',
							'_id as price_id'
						],
    				]))
    				->get();
    }
    
    /**
    * Fetch Item by type
    *
    * @param  int $type
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function fetchItemByType($type)
	{   
	    return ItemModel::where('type', $type)->select('_id', 'title')->get();
    }
    

    /**
    * Fetch Item by type
    *
    * @param  int $type
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function fetchItems()
	{   
	    return ItemModel::get();
	}
	
}