<?php
/*
* PackageController.php - Controller file
*
* This file is part of the Package component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Package\Controllers;

use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Package\Requests\PackageAddRequest; 
use App\Yantrana\Components\Package\Requests\PackageEditRequest;
use App\Yantrana\Components\Package\Requests\PackageAssignRequest; 
use App\Yantrana\Components\Package\PackageEngine;
use App\Yantrana\Support\CommonPostRequest;

class PackageController extends BaseController 
{    
    /**
     * @var  PackageEngine $packageEngine - Package Engine
     */
    protected $packageEngine;

    /**
      * Constructor
      *
      * @param  PackageEngine $packageEngine - Package Engine
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    function __construct(PackageEngine $packageEngine)
    {
        $this->packageEngine = $packageEngine;
    }
    
     
    /**
      * list of Package
      *
      * @return  json object
      *---------------------------------------------------------------- */

  	public function preparePackageList() 
  	{

      	return $this->packageEngine
                  	->preparePackageDataTableSource();
  	}
     
	/**
	  * Package process delete 
	  *
	  * @param  mix $packageIdOrUid
	  *
	  * @return  json object
	  *---------------------------------------------------------------- */

	public function processPackageDelete($packageIdOrUid)
	{   
	    $processReaction = $this->packageEngine
	                            ->processPackageDelete($packageIdOrUid);

	    return __processResponse($processReaction, [], [], true);
	}
 
    /**
      * Package Add Support Data 
      *
      * @return  json object
      *---------------------------------------------------------------- */

    public function preparePackageSupportData()
    {   
        $processReaction = $this->packageEngine
                                ->preparePackageSupportData();

        return __processResponse($processReaction, [], [], true);
    }
   
	/**
	  * Package Process Create 
	  *
	  * @param  object PackageListRequest $request
	  *
	  * @return  json object
	  *---------------------------------------------------------------- */

	public function processPackageCreate(PackageAddRequest $request)
	{   
	    $processReaction = $this->packageEngine
	                            ->processPackageCreate($request);

	    return __processResponse($processReaction);
	}
 
    /**
      * Package get update data 
      *
      * @param  mix $packageIdOrUid
      *
      * @return  json object
      *---------------------------------------------------------------- */

    public function updatePackageData($packageIdOrUid)
    {   
        $processReaction = $this->packageEngine
                                ->preparePackageUpdateData($packageIdOrUid);

        return __processResponse($processReaction, [], [], true);
    }

    /**
      * Package process update 
      * 
      * @param  mix @param  mix $packageIdOrUid
      * @param  object PackageListRequest $request
      *
      * @return  json object
      *---------------------------------------------------------------- */

    public function processPackageUpdate($packageIdOrUid, PackageEditRequest $request)
    {   
        $processReaction = $this->packageEngine
                                ->processPackageUpdate($packageIdOrUid, $request->all());

        return __processResponse($processReaction, [], [], true);
    }
    
    /**
  	* Package get details data 
  	*
  	* @param  mix $packageIdOrUid
  	*
  	* @return  json object
  	*---------------------------------------------------------------- */

	public function preparePackageDetails($packageIdOrUid)
	{   
    	$processReaction = $this->packageEngine
                            ->preparePackageDetails($packageIdOrUid);

    	return __processResponse($processReaction, [], [], true);
	}

    /**
    * Prepare User Purchases
    *
    * @param  object PackageListRequest $request
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function prepareUsersPurchases()
	{

	    $processReaction = $this->packageEngine->prepareUsersPurchases();

	    return __processResponse($processReaction, [], [], true);
    }
    
    /**
    * Prepare User Purchases
    *
    * @param  object PackageListRequest $request
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function prepareUserPurchase($requestFor, $userPurcheUid)
	{   
        $processReaction = $this->packageEngine
                                ->prepareUserPurchase(
                                    $requestFor, $userPurcheUid
                                );

	    return __processResponse($processReaction, [], [], true);
	}

	 /**
    * Prepare User Purchases
    *
    * @param  object PackageListRequest $request
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function processUserPackageUpdate(CommonPostRequest $request)
	{ 
        $processReaction = $this->packageEngine->updateUserPackage($request->all());

	    return __processResponse($processReaction, [], [], true);
	}

}