<?php
/*
* Package.php - Model file
*
* This file is part of the Package component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Package\Models;

use App\scopes\ClientScopeFilter;
use App\Yantrana\Base\BaseModel;
use App\Yantrana\Components\Slots\Models\SlotsModel;
use App\Yantrana\Components\Slots\Models\UserSlotModel;
use App\Yantrana\Components\User\Models\UserAuthorityModel;
use App\Yantrana\Components\Bill\Models\BillItemsModel;
use App\Yantrana\Components\Bill\Models\BillModel;

class UserPackageModel extends BaseModel
{
    use ClientScopeFilter;
    /**
     * @var  string $table - The database table used by the model.
     */
    protected $table = "user_packages";

    /**
     * has eouid.
     *
     * @var string
     *----------------------------------------------------------------------- */
    protected $hasEoId = false;

    /**
     * @var  array $casts - The attributes that should be casted to native types.
     */
    protected $casts = [
    ];

    /**
     * @var  array $fillable - The attributes that are mass assignable.
     */
    protected $fillable = [
        "client_id",
        "member_id",
        "memberName",
        "memberPhone",
        "memberEmail",
    ];

    /**
     * get package items associated with package
     */
    public function packageItems()
    {
        return $this->hasMany(PackageItem::class, 'packages__id', '_id')->with('item');
    }

    /**
     * get package items associated with package
     */
    public function itemSlots()
    {
        return $this->hasMany(SlotsModel::class, 'items__id', '_id');
    }

    /**
     * get user slots
     */
    public function userSlots()
    {
        return $this->hasMany(UserSlotModel::class, 'user_packages__id', '_id')->with('item');
    }

    /**
     * get user slots
     */
    public function userAssignedSlot()
    {
        return $this->hasMany(UserSlotModel::class, 'user_packages__id', '_id')->with('itemWithSlot');
    }

    /**
     * get user slots
     */
    public function package()
    {
        return $this->hasOne(PackageModel::class, '_id', 'packages__id')->with('packageItems');
    }

    /**
     * get user slots
     */
    public function packages()
    {
        return $this->hasMany(PackageModel::class, '_id', 'packages__id');
    }

    /**
     * get user slots
     */
    public function customer()
    {
        return $this->hasOne(UserAuthorityModel::class, '_id', 'user_authorities__id')
            ->with('user');
    }

    /**
     * get user slots
     */
    public function billItems()
    {
        return $this->hasMany(BillItemsModel::class, 'user_packages__id', '_id')->with('transactions');
    }

    /**
     * get user slots
     */
    public function bills()
    {
        return $this->hasMany(BillModel::class, 'customer_user_authorities__id', 'user_authorities__id')->with('transactions');
    }

}