<?php            
/*
* PackageListRequest.php - Request file
*
* This file is part of the Package component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Package\Requests;

use App\Yantrana\Base\BaseRequest;

class PackageAddRequest extends BaseRequest 
{   
    /**
      * Set if you need form request secured.
      *------------------------------------------------------------------------ */
    protected $securedForm = true;

    /**
     * Unsecured/Unencrypted form fields.
     *------------------------------------------------------------------------ */
    protected $unsecuredFields = [];
    
        
   /**
     * Loosely sanitize fields.
     *------------------------------------------------------------------------ */
    protected $looseSanitizationFields = [
                    'title' => '', 
                    'short_description' => '' 
              ];
    
    /**
      * Authorization for request.
      *
      * @return  bool
      *-----------------------------------------------------------------------*/

    public function authorize()
    {
       return true; 
    }
    
    /**
      * Validation rules.
      *
      * @return  bool
      *-----------------------------------------------------------------------*/

    public function rules()
    { 
        return [
            "title" => "required",
            "status"  => "required",
            "short_description"  => "nullable|min:5|max:500",
            "package_frequency.*.package_price" => 'sometimes|min:0|amount_validation|decimal_validation'
        ];
    }


    /**
    * Set custom msg for field
    *
    * @return array
    *-----------------------------------------------------------------------*/

    public function messages()
    {
        return [

            'package_frequency.*.package_price.decimal_validation' => __tr('Digit after decimal may not be greater than __max__.', [
                    '__attribute__' => ':attribute',
                    '__max__'       => '4'
                ]),
            'package_frequency.*.package_price.amount_validation' => __tr('The price may not be greater than __max__.', [
                    '__attribute__' => ':attribute',
                    '__max__'       => '999999999'
                ]),
        ];
    }
}