<?php
/*
* SlotsRepository.php - Repository file
*
* This file is part of the Slots component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Slots\Repositories;

use App\Yantrana\Base\BaseRepository;
 
use App\Yantrana\Components\Slots\Models\SlotsModel;
use App\Yantrana\Components\Slots\Models\UserSlotModel;
use App\Yantrana\Components\Slots\Interfaces\SlotsRepositoryInterface;

class SlotsRepository extends BaseRepository
                          implements SlotsRepositoryInterface 
{ 
    
    /**
    * Fetch the record of Slots
    *
    * @param    int || string $idOrUid
    *
    * @return    eloquent collection object
    *---------------------------------------------------------------- */

    public function fetch($idOrUid)
    {   
        if (is_numeric($idOrUid)) {

            return SlotsModel::where('_id', $idOrUid)->first();
        }

        return SlotsModel::where('_uid', $idOrUid)->first();
    }


    /**
    * Fetch the record of item Slots
    *
    * @param    int || string $itemId
    *
    * @return    eloquent collection object
    *---------------------------------------------------------------- */

    public function fetchItemSlots($itemId)
    {   
        return SlotsModel::with('userSlots')
    				->where('items__id', $itemId)->get();
    }

    /**
    * Delete lots of items
    *
    * @param int || string $itemId
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */

    public function deleteItemSlots($itemId)
    {   
        return SlotsModel::whereIn('_id', $itemId)->delete();
    }

    /**
    * Delete lots of items
    *
    * @param int || string $itemId
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */

    public function slotBunchUpdate($updateData, $indentifier)
    {   
        return SlotsModel::bunchUpdate($updateData, $indentifier);
    }

    /**
    * Insert data insert
    *
    * @param  array $slots
    * @param  int || string $itemId
    *
    * @return  eloquent collection object
    *---------------------------------------------------------------- */
    public function insert($slots)
    {   
        
        $slotModel = new SlotsModel;

        if ($slotModel->prepareAndInsert($slots)) {
            return true;
        }

        return false;
    }

    /**
	  * Store assignes package slots
	  *
	  * @param  array $userSlots
	  *
	  * @return  mixed
	  *---------------------------------------------------------------- */

	public function storeUserPackageSlots($userSlots)
	{   
	    $items = new UserSlotModel;

	    // Check if task testing record added then return positive response
	    if ($items->prepareAndInsert($userSlots)) {

	        return $userSlots;

	    }

	    return false;
	}
	
	/*
    * Fetch slots of items
    *
    * @param int $itemsIds
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */

    public function fetchSlotsOfItems($itemsIds)
    {   
        return SlotsModel::whereIn('items__id', $itemsIds)
                    ->select('_id', 'title', 'slot_time', 'slot_duration_minutes', 'items__id')
                    ->groupBy('items__id')
                    ->get();
    }
    
    /**
    * Fetch the record of Slots
    *
    * @param    int || string $idOrUid
    *
    * @return    eloquent collection object
    *---------------------------------------------------------------- */

    public function fetchUserSlot($idOrUid)
    {   
        if (is_numeric($idOrUid)) {

            return UserSlotModel::where('_id', $idOrUid)->first();
        }

        return UserSlotModel::where('_uid', $idOrUid)->first();
    }

    /**
  	  * Update slot
  	  *
  	  * @param  object $item
  	  * @param  array $inputData
  	  *
  	  * @return  mixed
  	  *---------------------------------------------------------------- */

	public function updateSlot($slot, $inputData)
	{       
    	// Check if slot updated then return positive response
    	if ($slot->modelUpdate($inputData)) {

        	return true;
    	}

    	return false;
	}

	/**
    * Delete user slots 
    *
    * @param int || string $itemId
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */

    public function deleteUserSlot($userslotsId)
    {   
        return UserSlotModel::where('_uid', $userslotsId)->delete();
    }
}