<?php
/*
* TransactionController.php - Controller file
*
* This file is part of the Transaction component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Transaction\Controllers;

use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Transaction\TransactionEngine;

use App\Yantrana\Components\Transaction\Requests\{
    TransactionAddRequest, TransactionEditRequest, TransactionBillPayRequest
    
}; 

class TransactionController extends BaseController 
{    
    /**
     * @var  TransactionEngine $transactionEngine - Transaction Engine
     */
    protected $transactionEngine;

    /**
      * Constructor
      *
      * @param  TransactionEngine $transactionEngine - Transaction Engine
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    function __construct(TransactionEngine $transactionEngine)
    {
        $this->transactionEngine = $transactionEngine;
    }
    
     
    /**
      * list of Transaction
      *
      * @return  json object
      *---------------------------------------------------------------- */

  	public function prepareTransactionList() 
  	{    
  		return $this->transactionEngine->prepareTransactionList();
  	}

 
    /**
      * Transaction Add Support Data 
      *
      * @return  json object
      *---------------------------------------------------------------- */

    public function prepareTransactionSupportData()
    {   
        $processReaction = $this->transactionEngine
                                ->prepareTransactionSupportData();

        return __processResponse($processReaction, [], [], true);
    }
 
	/**
	  * Transaction create process 
	  *
	  * @param  object TransactionListRequest $request
	  *
	  * @return  json object
	  *---------------------------------------------------------------- */

	public function processTransactionCreate(TransactionAddRequest $request)
	{

        try{
            $processReaction = $this->transactionEngine->processTransactionCreate($request->all());
        }catch (\Exception $exception){dd($exception->getMessage(),$exception->getFile(),$exception->getLine());}


	    return __processResponse($processReaction, [], [], true);
	}

	/**
  	  * get user bills 
  	  *
  	  * @param  mix $memberId
  	  *
  	  * @return  json object
  	  *---------------------------------------------------------------- */

	public function getUserBills($memberId)
	{   try{
        $processReaction = $this->transactionEngine->prepareUserBills($memberId);
    }catch (\Exception $exception){dd($exception->getMessage(),$exception->getFile(),$exception->getLine());}



    	return __processResponse($processReaction, [], [], true);
	}

    /**
    * Transaction process bill pay update 
    * 
    * @param  mix @param  mix $billUid
    * @param  object TransactionListRequest $request
    *
    * @return  json object
    *---------------------------------------------------------------- */

	public function processPayBill($billUid, TransactionBillPayRequest $request)
	{   
    	$processReaction = $this->transactionEngine->processPayBill($billUid, $request->all());

    	return __processResponse($processReaction, [], [], true);
	}
    
    

}