<?php
/*
 *  YesAuthority Configurations
 *
 *  This configuration file is part of YesAuthority
 *
 *------------------------------------------------------------------------------------------------*/
return [
    /* authority configurations
     *--------------------------------------------------------------------------------------------*/
    'config' => [
        /*
         *   @required - if you want use name other than 'authority.checkpost'
         *   middleware_name - YesAuthority Middleware name
        */
        'middleware_name'       => 'authority.checkpost',
        /*
         *   @required
         *   col_user_id - ID column name for users table
        */
        'col_user_id'           => 'users__id',

        /*
         *   @required
         *   col_role - Your Role ID column name for users table
        */
        'col_role'              => 'user_roles__id',

        /*
         *   @optional - if you want to use dynamic permissions
         *   col_user_permissions - Dynamic Permissions(json) column on users table
         *   This column should contain json encoded array containing 'allow' & 'deny' arrays
        */
        'col_user_permissions'  => '__permissions',

        /*
         *   @required
         *   user_model - User Model
        */
        'user_model'            => 'App\Yantrana\Components\User\Models\UserAuthorityModel',
        /*
         *   @optional
         *   role_model - Role Model
        */
        'role_model'            => 'App\Yantrana\Components\User\Models\UserRole',
        /*
         *   @optional
         *   col_role_id - ID column name for role table
        */
        //'col_role_id'           => '_id',

        /*
         *   @optional
         *   ccol_role_permissions - Dynamic Permissions(json) column on role table,
         *   This column should contain json encoded array containing 'allow' & 'deny' arrays
        */
        //'col_role_permissions'  => '__permissions'

        'pseudo_access_ids'       => [
            'admin', 'staff_member', 'customer'
        ],

        'default_allowed_access_ids' => []
    ],
    /*
     *  Authority rules
     *
     *  Rules item needs to have 2 arrays with keys allow & deny value of it will be array
     *  containing access ids as required.
     *  wildcard entries are accepted using *
     *  for each section level deny will be more powerful than allow
     *  also key length also matters more is length more
     *--------------------------------------------------------------------------------------------*/
    'rules' => [
        // base rules for all roles user
        'base' => [
            'allow' => [
                '*',
            ],
            'deny' => [
            ]            
        ],
        /*
         *  Role Based rules
         *  First level of defence
         *----------------------------------------------------------------------------------------*/
        'roles' => [
            /*
             *  Rules for the Roles for using id (key will be id)
             *------------------------------------------------------------------------------------*/
            // @example given for role id of 1
            // Admin Role 
            1 => [
                'allow' => [
                    'admin'
                ],
                'deny'  => [
                    'staff_member',
                ],
            ],
            // Staff member
            2 => [ 
                'allow' => [
                    'staff_member',
					'view_only_manage_packages',
					'add_package',
					'edit_package',
					'delete_and_restore_package',
					'view_package_details',
					'view_only_manage_bills',
					'add_bill',
					'edit_bill',
					'view_bill_details',
					'pay_bill',
					'cancel_bill',
					'delete_bill',
					'view_only_manage_transactions',
                    'add_transaction'
                ],
                'deny'  => [
                    'admin',
                    'customer',
                    'manage_confguration_setting',
                    'view_only_manage_users',
                    'add_user',
                    'edit_user',
                    'delete_and_restore_user',
                    'view_user_details',
                    'view_only_manage_role',
                    'add_role',
                    'manage_role_permission',
                    'delete_role',
                    'activity_log',
                    'view_activity_log',
					'view_only_manage_customers',
					'add_customer',
					'edit_customer',
					'delete_and_restore_customer',
					'view_customer_details',
					'assign_package_to_user',
                    'view_only_manage_items',
                    'member_view_customer_details',
                    'assign_package_to_member',
					'add_item',
					'edit_item',
					'delete_and_restore_item',
                    'view_item_details'
                ],
            ],
            // Customer
            3 => [ 
                'allow' => [
                    'customer',
                    'manage.dashboard.read.support_data',
                    'user.*',
					'media.upload.read_user_profile',
					'media.upload.write.user_profile',
                ],
                'deny'  => [
                    'admin', 'staff_member', '*'
                    
                ],
            ]
        ],

        /*
         *  User based rules
         *  2nd level of defense
         *  Will override the rules of above 1st level(roles) if matched
         *----------------------------------------------------------------------------------------*/
        'users' => [
             /*
             *  Rules for the Users for using id (key will be id)
             *------------------------------------------------------------------------------------*/
            // @example given for user id of 1
            1 => [ // this may be admin user id
                'allow' => [],
                'deny'  => [],
            ]
        ],

        /*
         *  DB Role Based rules
         *  3rd level of defense
         *  Will override the rules of above 2nd level(user) if matched
         *  As it will be database based you don't need to do anything here
         *----------------------------------------------------------------------------------------*/

        /*
         *  DB User Based rules
         *  4th level of defense
         *  Will override the rules of above 3rd level(db roles) if matched
         *  As it will be database based you don't need to do anything here
         *----------------------------------------------------------------------------------------*/

        /*  Dynamic permissions based on conditions
         *  Will override the rules of above 4th level(db user) if matched
         *  5th level of defense
         * each condition will be array with following options available:
         *  @key - string - name
         *      @value - string - it will be condition identifier (alpha-numeric-dash)
         *  @key - string - access_ids
         *      @value - array - of ids (alpha-numeric-dash)
         *  @key - string - uses
         *      @value - string - of of classNamespace@method
         *          OR
         *      @value - anonymous function -
         *  @note - both the function/method receive following 3 parameters so you can
         *          run your own magic of logic using it.
         *  $accessIdKey            - string - requested id key
         *  $isAccess               - bool - what is the access received from the above level/condition
         *  $currentRouteAccessId   - current route/accessIds being checked.
         *----------------------------------------------------------------------------------------*/
        'conditions' => [
            // Example conditions
            //  It should return boolean values, true for access allow & false for deny
            [
                'name'       => 'demo_authority',
                'access_ids' => ['demo_authority', 'file_manager.*'],
                'uses'       => function($accessId, $isAccess, $currentRouteAccessId) {
               
                    if(isDemo()) {

                        $demouserIds     = [6, 7, 8];
                        $currentUserId   = Auth::id();

                        if (in_array($currentUserId, $demouserIds) and in_array($currentRouteAccessId, [
                                'user.change_password.process',
                                'user.change_email.process',
                                'user.profile.update.process'
                            ])) {

                            return false;
                        }

                        if(($currentUserId !== 1) 
                            and ((in_array($currentRouteAccessId, [
                                'manage.configuration.process',
                                'media.upload.delete',
                                'media.upload.delete_multiple',
                            ]))
                                or (str_is('media.upload*.write*', $currentRouteAccessId) === true)
                            )
                        ) {
                            return false;
                        }
                    }

                    return true;
                }
            ]
        ]
    ],

    /*
     *  Dynamic access zones
     *
     *  Zones can be created for various reasons, when using dynamic permission system
     *  its bad to store direct access ids into database in that case we can create dynamic access
     *  zones which is the group of access ids & these can be handled with one single key id.
     *----------------------------------------------------------------------------------------*/
    'dynamic_access_zones' => [

        // Manage Users
        'manage_users' => [
            'title' => "Manage Staff Members",
            'access_ids' => []
        ],
        'view_only_manage_users' => [
            'title' => "Read",
            'access_ids' => [
                'manage.user.read.datatable.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_users'
        ],
        // Add User
        'add_user' => [
            'title' => 'Create',
            'access_ids' => [
                'manage.user.write.create',
                'manage.user.read.create.support_data'
            ],
            'dependencies' => [
                'view_only_manage_users'
            ],
            'parent' => 'manage_users'
        ],
        // Edit User
        'edit_user' => [
            'title' => 'Update',
            'access_ids' => [
                'manage.user.read.edit_suppport_data',
                'manage.user.write.update_process'
            ],
            'dependencies' => [
                'view_only_manage_users'
            ],
            'parent' => 'manage_users'
        ],
        // Delete User
        'delete_and_restore_user' => [
            'title' => 'Delete',
            'access_ids' => [
                'manage.user.write.delete',
                'manage.user.write.restore'
            ],
            'dependencies' => [
                'view_only_manage_users'
            ],
            'parent' => 'manage_users'
        ],

        'member_view_customer_details' => [
            'title' => "View Details",
            'access_ids' => [
                'manage.customer.read.detail.data'
            ],
            'dependencies' => [
                'view_only_manage_users'
            ],
            'parent' => 'manage_users'
        ],

        'assign_package_to_member' => [
            'title' => "Assign Package",
            'access_ids' => [
                'manage.customer.package.read.package_assign_member',
                'manage.customer.package.write.package_assign_member',
                'manage.customer.package.read.package_assign_data',
                'manage.customer.package.write.assign_to_user',
                'manage.customer.read.detail.data',
                'manage.customer.package.write.assign_to_user',
                'manage.bill.read.bill_payment_details',
                //'manage.transaction.write.process_pay_bill'
            ],
            'dependencies' => [
                'view_only_manage_users'
            ],
            'parent' => 'manage_users'
        ],

        /*------------------- Manage Customer-----------------------------*/
        'manage_customers' => [
            'title' => "Manage Members",
            'access_ids' => []
        ],
        'view_only_manage_customers' => [
            'title' => "Read",
            'access_ids' => [
                'manage.customer.read.datatable.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_customers'
        ],
        // Add User
        'add_customer' => [
            'title' => 'Create',
            'access_ids' => [
                'manage.customer.write.create'
            ],
            'dependencies' => [
                'view_only_manage_customers'
            ],
            'parent' => 'manage_customers'
        ],
        // Edit User
        'edit_customer' => [
            'title' => 'Update',
            'access_ids' => [
                'manage.customer.read.edit_suppport_data',
                'manage.customer.write.update_process'
            ],
            'dependencies' => [
                'view_only_manage_customers'
            ],
            'parent' => 'manage_customers'
        ],
        // Delete User
        'delete_and_restore_customer' => [
            'title' => 'Delete',
            'access_ids' => [
                'manage.customer.write.delete',
                'manage.customer.write.restore'
            ],
            'dependencies' => [
                'view_only_manage_customers'
            ],
            'parent' => 'manage_customers'
        ],

        'view_customer_details' => [
            'title' => "View Details",
            'access_ids' => [
                'manage.customer.read.detail.data'
            ],
            'dependencies' => [
                'view_only_manage_customers'
            ],
            'parent' => 'manage_customers'
        ],

        'assign_package_to_user' => [
            'title' => "Assign Package",
            'access_ids' => [
                'manage.customer.package.read.package_assign_member',
                'manage.customer.package.write.package_assign_member',
                'manage.customer.package.read.package_assign_data',
                'manage.customer.package.write.assign_to_user',
                'manage.bill.read.bill_payment_details',
               // 'manage.transaction.write.process_pay_bill'
            ],
            'dependencies' => [
                'view_only_manage_customers'
            ],
            'parent' => 'manage_customers'
        ],

        /*------------------- / Manage Customer-----------------------------*/

       

        /*------------------- Manage Items-----------------------------*/
        'manage_items' => [
            'title' => "Manage Items",
            'access_ids' => []
        ],
        'view_only_manage_items' => [
            'title' => "Read",
            'access_ids' => [
                'manage.item.read.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_items'
        ],
        // Add Iten
        'add_item' => [
            'title' => 'Create',
            'access_ids' => [
				'manage.item.read.support_data',
				'manage.item.write.create',
            ],
            'dependencies' => [
                'view_only_manage_items'
            ],
            'parent' => 'manage_items'
        ],
        // Edit Iten
        'edit_item' => [
            'title' => 'Update',
            'access_ids' => [
                'manage.item.read.update.data',
                'manage.item.write.update'
            ],
            'dependencies' => [
                'view_only_manage_items'
            ],
            'parent' => 'manage_items'
        ],
        // Delete Iten
        'delete_and_restore_item' => [
            'title' => 'Delete',
            'access_ids' => [
                'manage.item.write.delete'
            ],
            'dependencies' => [
                'view_only_manage_items'
            ],
            'parent' => 'manage_items'
        ],
        'view_item_details' => [
            'title' => "View Details",
            'access_ids' => [
                'manage.item.read.deatils'
            ],
            'dependencies' => [
                'view_only_manage_items'
            ],
            'parent' => 'manage_items'
        ],

        /*------------------- / Manage Items-----------------------------*/

        /*------------------- Manage Packages-----------------------------*/
        'manage_packages' => [
            'title' => "Manage Packages",
            'access_ids' => []
        ],
        'view_only_manage_packages' => [
            'title' => "Read",
            'access_ids' => [
                'manage.package.read.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_packages'
        ],
        // Add Package
        'add_package' => [
            'title' => 'Create',
            'access_ids' => [
				'manage.package.read.support_data',
				'manage.package.write.create',
            ],
            'dependencies' => [
                'view_only_manage_packages'
            ],
            'parent' => 'manage_packages'
        ],
        // Edit Package
        'edit_package' => [
            'title' => 'Update',
            'access_ids' => [
                'manage.package.read.update.data',
                'manage.package.write.update'
            ],
            'dependencies' => [
                'view_only_manage_packages'
            ],
            'parent' => 'manage_packages'
        ],
        // Delete Package
        'delete_and_restore_package' => [
            'title' => 'Delete',
            'access_ids' => [
                'manage.package.write.delete'
            ],
            'dependencies' => [
                'view_only_manage_packages'
            ],
            'parent' => 'manage_packages'
        ],

        'view_package_details' => [
            'title' => "View Details",
            'access_ids' => [
                'manage.package.read.details'
            ],
            'dependencies' => [
                'view_only_manage_packages'
            ],
            'parent' => 'manage_packages'
        ],

        /*------------------- / Manage Packages-----------------------------*/

        /*------------------- Manage Bills-----------------------------*/
        'manage_bills' => [
            'title' => "Manage Bills",
            'access_ids' => []
        ],

        'view_only_manage_bills' => [
            'title' => "Read",
            'access_ids' => [
                'manage.bill.read.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_bills'
        ],

        // Add Package
        'add_bill' => [
            'title' => 'Create',
            'access_ids' => [
				'manage.user_purchses.read.data',
				'manage.bill.write.generate',
            ],
            'dependencies' => [
                'view_only_manage_bills'
            ],
            'parent' => 'manage_bills'
        ],

        // Add Package
        'edit_bill' => [
            'title' => 'Update',
            'access_ids' => [
				'manage.bill.read.edit_support_data',
				'manage.bill.write.update',
            ],
            'dependencies' => [
                'view_only_manage_bills'
            ],
            'parent' => 'manage_bills'
        ],

        'view_bill_details' => [
            'title' => "View Details",
            'access_ids' => [
                'manage.bill.read.details',
                'manage.bill.read.print_bill'
            ],
            'dependencies' => [
                'view_only_manage_bills'
            ],
            'parent' => 'manage_bills'
        ],

		'pay_bill' => [
            'title' => "Pay Bill Amount",
            'access_ids' => [
               'manage.bill.read.bill_payment_details',
               'manage.transaction.write.process_pay_bill'
            ],
            'dependencies' => [
                'view_only_manage_bills'
            ],
            'parent' => 'manage_bills'
        ],

        'cancel_bill' => [
            'title' => "Cancel Bill",
            'access_ids' => [
               'manage.bill.write.process_cancel'
            ],
            'dependencies' => [
                'view_only_manage_bills'
            ],
            'parent' => 'manage_bills'
        ],

        'delete_bill' => [
            'title' => "Delete Bill",
            'access_ids' => [
               'manage.bill.write.process_delete'
            ],
            'dependencies' => [
                'view_only_manage_bills'
            ],
            'parent' => 'manage_bills'
        ],

        /*------------------- / Manage Bills-----------------------------*/

        /*------------------- Manage Transactions-----------------------------*/
        'manage_transactions' => [
            'title' => "Manage Transactions",
            'access_ids' => []
        ],
        'view_only_manage_transactions' => [
            'title' => "Read",
            'access_ids' => [
                'manage.transaction.read.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_transactions'
        ],
        // Add Transaction
        'add_transaction' => [
            'title' => 'Create',
            'access_ids' => [
				'manage.transaction.read.support_data',
				'manage.transaction.write.create'
            ],
            'dependencies' => [
                'view_only_manage_transactions'
            ],
            'parent' => 'manage_transactions'
        ],
        /*------------------- / Manage Transactions-----------------------------*/


        // Roles
        'manage_roles' => [
            'title' => "Manage Roles",
            'access_ids' => []
        ],
        'view_only_manage_role' => [
            'title' => "Read",
            'access_ids' => [
                'manage.user.role_permission.read.list'
            ],
            'dependencies' => [
            ],
            'parent' => 'manage_roles'
        ],
        'add_role' => [
            'title' => 'Create',
            'access_ids' => [
                'manage.user.role_permission.read.add_support_data',
                'manage.user.role_permission.read.using_id',
                'manage.user.role_permission.write.role.create'
            ],
            'dependencies' => [
                'view_only_manage_role'
            ],
            'parent' => 'manage_roles'
        ],
        'manage_role_permission' => [
            'title' => 'Update',
            'access_ids' => [
                'manage.user.role_permission.read',
                'manage.user.role_permission.write.create'
            ],
            'dependencies' => [
                'view_only_manage_role'
            ],
            'parent' => 'manage_roles'
        ],
        'delete_role' => [
            'title' => 'Delete',
            'access_ids' => [
                'manage.user.role_permission.write.delete'
            ],
            'dependencies' => [
                'view_only_manage_role'
            ],
            'parent' => 'manage_roles'
        ],
        // Configuration & Setings
        'confguration_setting' => [
            'title' => "Manage Settings",
            'access_ids' => []
        ],
        'manage_confguration_setting' => [
            'title' => "Change Configuration",
            'access_ids' => [
                'manage.configuration.get.support.data',
                'manage.configuration.process',
                'media.upload.read_logo',
                'media.upload.read_favicon'
            ],
            'parent' => 'confguration_setting'
        ],
        // Activity Log 
        'activity_log' => [
            'title' => "Activity Log",
            'access_ids' => []
        ],
        'view_activity_log' => [
            'title' => "Read",
            'access_ids' => [
                'manage.activity_log.read.list',
                'manage.activity_log.action_type.read.data'
            ],
            'parent' => 'activity_log'
        ]
    ],

    /*'entities' => [
        'project' => [
            'model' => 'App\Yantrana\Components\Project\Models\ProjectUserModel',
            'id_column' => 'projects__id',
            'permission_column' => '__permissions',
            'user_id_column' => 'users__id'
        ]
    ]*/
];