<?php
// Response Codes & other global configurations
$techConfig = require app_path('Yantrana/__Laraware/Config/tech-config.php');

$techAppConfig = [

    /* Paths - required item will be replaced in custom tech config
    ------------------------------------------------------------------------- */
    'storage_paths' => [
        'media_storage' => [
            'other/no_thumb_image.jpg'	    => 'key@no_thumb',
            'other/no-user-thumb-icon.png'  => 'key@no_user_thumb',
            'users' => [
                '{_uid}' => [
                    'logo' => 'key@logo',
                    'favicon'   => 'key@favicon',
                    'profile'      => 'key@user_photo',
                    'temp_uploads' => 'key@user_temp_uploads'
                ]
            ],
            'logo' => 'key@logo',
            'favicon'   => 'key@favicon',
            'file-manager-assets/{userUid}/'			=> 'key@user_file_manager',
            'file-manager-assets/{userUid}/thumbnails/'	=> 'key@user_file_manager_thumb',
        ]
    ],

    /* Media extensions
    ------------------------------------------------------------------------- */
    'media' => [
        'extensions' => [
            1 => ["jpg", "png", "gif","jpeg"],
            2 => ["3gp","mp4", "flv"],
            3 => [],
            4 => ["mp3", "wav"],
            5 => ["pdf", "doc", "docx", "xls", "ppt", "txt"],
            6 => ["pdf", "jpg", "png", "gif","jpeg"],
            7 => ["png"], // for logo & profile
            8 => ["pdf", "doc", "docx", "xls", "ppt", "txt", "jpg", "png", "gif","jpeg"],
            9 => ["ico"],
            10 => ["png", "ico"]
        ]
    ],

	'login_url' => '/#!/login',
	'reset_password_url' => '/#!/reset-password/',

	/* Status Code Multiple Uses
    ------------------------------------------------------------------------- */

    'status_codes' => [
        1 => 'Active',
        2 => 'Inactive',
        3 => 'Banned',
        4 => 'Never Activated',
        5 => 'Deleted',
        6 => 'Suspended',
        7 => 'On Hold',
        8 => 'Completed',
        9 => 'Invite'
    ],

    /* Durations Value
	------------------------------------------------------------------------*/
	'durations' => [
		1 => 'Current Month',
		2 => 'Last Month',
		3 => 'Current Week',
		4 => 'Last Week',
		5 => 'Today',
		6 => 'Yesterday',
		7 => 'Last Year',
		8 => 'Current Year',
		9 => 'Last 30 Days',
		10 => 'Custom'
	],

    /* Email Config
    ------------------------------------------------------------------------- */

    'mail_from'         =>  [
        env('MAIL_FROM_ADD', 'your@domain.com'),
        env('MAIL_FROM_NAME', 'E-Mail Service')
    ],

    'recaptcha'	=>  [
    	'site_key' => env('RECAPTCHA_PUBLIC_KEY',''),
        'secret_key' => env('RECAPTCHA_PRIVATE_KEY','')
    ],


    /* Description String limit
    ------------------------------------------------------------------------- */
    'string_limit' => 30,

    'account_activation'  => (60*60*48),

    /* logo name & landing page image
    ------------------------------------------------------------------------- */

    'logoName'  => 'logo.png',

    /* favicon name
    ------------------------------------------------------------------------- */

    'faviconName'  => 'favicon.ico',

     /* Account related
    ------------------------------------------------------------------------- */

    'account' => [
        'activation_expiry'         => 24 * 2, // hours
        'change_email_expiry'       => 24 * 2, // hours
        'password_reminder_expiry'  => 24 * 2, // hours
        'passwordless_login_expiry' => 5, // minutes
    ],

    /* User
    ------------------------------------------------------------------------- */
    'user' => [
        'statuses' => [
            1,			// Active
            2,			// InActive
            5, 			// Deleted
            12			// Never Activated
        ],
        'roles' => [
            1 => 'Admin',
        ],
        'permission_status' => [
            1 => 'Role Inheritance',
            2 => 'Allow',
            3 => 'Deny'
        ]
    ],

    /* Activity log status
    ------------------------------------------------------------------------- */
    'activity_log' => [
        'entity_type' => [
            1 => 'User',
            2 => 'Profile',
            3 => 'Role Permission',
            4 => 'User Login Log',
            5 => 'Password of',
            6 => 'User Permission(s)',
            7 => 'Package',
            8 => 'Service(s)',
            9 => 'Bill(s)',
            10 => 'Bill Items',
            11 => 'Transaction',
            12 => 'Item',
            13 => 'Item Slot(s)'
        ],
        'action_type' => [
        	1 => 'Created',
        	2 => 'Updated',
        	3 => 'Deleted',
        	4 => 'Soft Deleted',
        	5 => 'Restored',
            6 => 'Logged in',
			7 => 'Logged Out',
			8 => 'Removed',
            9 => 'Assigned',
            10 => 'Cancelled',
            11 => 'Generated',
            12 => 'Stored'
        ]
    ],

     /* Package 
    --------------------------------------------------------------------------*/
    'packages' => [

    	'frequencies' => [
	        [
	            'id'    => 1,
	            'title' => 'Daily',
	            'name'  => 'Daily'
	        ],
	        [
	            'id'    => 2,
	            'title' => 'Weekly',
	            'name'  => 'Weekly'
	        ],
	        [
	            'id'    => 3,
	            'title' => 'Monthly',
	            'name'  => 'Monthly'
	        ],
	        [
	            'id'    => 4,
	            'title' => 'Quarterly',
	            'name'  => 'Quarterly'
	        ],
	        [
	            'id'    => 5,
	            'title' => 'Half-Yearly',
	            'name'  => 'Half-Yearly'
	        ],
	        [
	            'id'    => 6,
	            'title' => 'Yearly',
	            'name'  => 'Yearly'
	        ],
	    ],

    ],

     /* Items 
    --------------------------------------------------------------------------*/
    'item' => [
        'type' => [
            [
                'id'    => 1,
                'title' => 'Enrollment',
                'name'  => 'enrollment'
            ],
            [
                'id'    => 3,
                'title' => 'Session',
                'name'  => 'session'
            ],
            [
                'id'    => 2,
                'title' => 'Product',
                'name' => 'product'
            ]
        ],
        'slot_frequency' => [
            [
                'id'    => 1,
                'title' => 'Mon-Sat',
                'name'  => 'Every Day'
            ],
            [
                'id'    => 2,
                'title' => 'Every Day',
                'name'  => 'Every Day'
            ]
        ]
    ],


    /* Bills 
    --------------------------------------------------------------------------*/
    'bills' => [
        'status' => [
            [
                'id'    => 1,
                'title' => 'Unpaid'
            ],
            [
                'id'    => 2,
                'title' => 'Partial Paid'
            ],
            [
                'id'    => 3,
                'title' => 'Paid'
            ],
            [
                'id'    => 4,
                'title' => 'Cancelled'
            ]
        ]
    ],


    'transaction' => [

        'types' => [
            [
                'id' => 1,
                'title' => 'Credit'
            ],
            [
                'id' => 2,
                'title' => 'Debit'
            ]
        ],
        'subtypes' => [
            [
                'id' => 1,
                'title' => 'Payment'
            ],
            [
                'id' => 2,
                'title' => 'Refund'
            ],
        ],
        'status' => [
            [
                'id'    => 1,
                'title' => 'Initiated'
            ],
            [
                'id'    => 2,
                'title' => 'Completed'
            ],
            [
                'id'    => 3,
                'title' => 'Cancelled'
            ],
            [
                'id'    => 4,
                'title' => 'Failed'
            ]
        ],

        'payment_methods' => [
            [
                'id'    => 1,
                'title' => 'Cash'
            ],
            [
                'id'    => 2,
                'title' => 'Cheque'
            ],
            [
                'id'    => 3,
                'title' => 'Bank Transfer'
            ],
            [
                'id'    => 4,
                'title' => 'Payment Gateway'
            ]
        ],

    ],

    'muster' => [
           
        'durations' => [
            1 => 'Current Month',
            2 => 'Last Month'
        ]

    ],

    /* Store Related Config Values
    --------------------------------------------------------------------------*/

    'currencies'         => [
        /* Zero-decimal currencies
        ----------------------------------------------------------------------*/
        'zero_decimal'  => [
            'BIF' =>  'Burundian Franc',
            'CLP' =>  'Chilean Peso',
            'DJF' =>  'Djiboutian Franc',
            'GNF' =>  'Guinean Franc',
            'JPY' =>  'Japanese Yen',
            'KMF' =>  'Comorian Franc',
            'KRW' =>  'South Korean Won',
            'MGA' =>  'Malagasy Ariary',
            'PYG' =>  'Paraguayan Guaraní',
            'RWF' =>  'Rwandan Franc',
            'VND' =>  'Vietnamese Đồng',
            'VUV' =>  'Vanuatu Vatu',
            'XAF:' => 'Central African Cfa Franc',
            'XOF' =>  'West African Cfa Franc',
            'XPF' =>  'Cfp Franc',
            // Paypal zero-decimal currencies
            'HUF' =>  'Hungarian Forint',
            'TWD' =>  'New Taiwan Dollar',
        ],
        /* Mostly PayPal Supported currencies
        ----------------------------------------------------------------------*/
        'options'   => [
            'AUD' => ('Australian Dollar'),
            'CAD' => ('Canadian Dollar'),
            'EUR' => ('Euro'),
            'GBP' => ('British Pound'),
            'USD' => ('U.S. Dollar'),
            'NZD' => ('New Zealand Dollar'),
            'CHF' => ('Swiss Franc'),
            'HKD' => ('Hong Kong Dollar'),
            'SGD' => ('Singapore Dollar'),
            'SEK' => ('Swedish Krona'),
            'DKK' => ('Danish Krone'),
            'PLN' => ('Polish Zloty'),
            'NOK' => ('Norwegian Krone'),
            'HUF' => ('Hungarian Forint'),
            'CZK' => ('Czech Koruna'),
            'ILS' => ('Israeli New Shekel'),
            'MXN' => ('Mexican Peso'),
            'BRL' => ('Brazilian Real (only for Brazilian members)'),
            'MYR' => ('Malaysian Ringgit (only for Malaysian members)'),
            'PHP' => ('Philippine Peso'),
            'TWD' => ('New Taiwan Dollar'),
            'THB' => ('Thai Baht'),
            'JPY' => ('Japanese Yen'),
            'TRY' => ('Turkish Lira (only for Turkish members)'),
            ''    => ('Other')
        ],

        /* Currencies options with details
        ----------------------------------------------------------------------*/
        'details'    => [

            'AUD' => [
                'name'   => ("Australian Dollar"),
                'symbol' => "A$",
                'ASCII'  => "A&#36;"
            ],

            'CAD' => [
                'name'   => ("Canadian Dollar"),
                'symbol' => "$",
                'ASCII'  => "&#36;"
            ],

            'CZK' => [
                'name'   => ("Czech Koruna"),
                'symbol' => "Kč",
                'ASCII'  => "K&#x10d;"
            ],

            'DKK' => [
                'name'   => ("Danish Krone"),
                'symbol' => "Kr",
                'ASCII'  => "K&#x72;"
            ],

            'EUR' => [
                'name'   => ("Euro"),
                'symbol' => "€",
                'ASCII'  => "&euro;"
             ],

            'HKD' => [
                'name'   => ("Hong Kong Dollar"),
                'symbol' => "$",
                'ASCII'  => "&#36;"
            ],

            'HUF' => [
                'name'   => ("Hungarian Forint"),
                'symbol' => "Ft",
                'ASCII'  => "F&#x74;"
            ],

            'ILS' => [
                'name'   => ("Israeli New Sheqel"),
                'symbol' => "₪",
                'ASCII'  => "&#8361;"
            ],

            'JPY' => [
                'name'   => ("Japanese Yen"),
                'symbol' => "¥",
                'ASCII'  => "&#165;"
            ],

            'MXN' => [
                'name'   => ("Mexican Peso"),
                'symbol' => "$",
                'ASCII'  => "&#36;"
            ],

            'NOK' => [
                'name'   => ("Norwegian Krone"),
                'symbol' => "Kr",
                'ASCII'  => "K&#x72;"
            ],

            'NZD' => [
                'name'   => ("New Zealand Dollar"),
                'symbol' => "$",
                'ASCII'  => "&#36;"
            ],

            'PHP' => [
                'name'   => ("Philippine Peso"),
                'symbol' => "₱",
                'ASCII'  => "&#8369;"
            ],

            'PLN' => [
                'name'   => ("Polish Zloty"),
                'symbol' => "zł",
                'ASCII'  => "z&#x142;"
            ],

            'GBP' => [
                'name'   => ("Pound Sterling"),
                'symbol' => "£",
                'ASCII'  => "&#163;"
            ],

            'SGD' => [
                'name'   => ("Singapore Dollar"),
                'symbol' => "$",
                'ASCII'  => "&#36;"
            ],

            'SEK' => [
                'name'   => ("Swedish Krona"),
                'symbol' => "kr",
                'ASCII'  => "K&#x72;"
            ],

            'CHF' => [
                'name'   => ("Swiss Franc"),
                'symbol' => "CHF",
                'ASCII'  => "&#x43;&#x48;&#x46;"
            ],

            'TWD' => [
                'name'   => ("Taiwan New Dollar"),
                'symbol' => "NT$",
                'ASCII'  => "NT&#36;"
            ],

            'THB' => [
                'name'   => ("Thai Baht"),
                'symbol' => "฿",
                'ASCII'  => "&#3647;"
            ],

            'USD' => [
                'name'   => ("U.S. Dollar"),
                'symbol' => "$",
                'ASCII'  => "&#36;"
            ],

            'INR' => [
                'name'   => ("Indian Rupee"),
                'symbol' => "₹",
                'ASCII'  => "&#8377;"
            ],
            
            'TRY' => [
                'name'   => ("Turkish Lira"), 
                'symbol' => "₺", 
                'ASCII'  => "&#x20BA;"
            ],
        ],
    ],

    /* Technical Items Codes
    ------------------------------------------------------------------------- */

    'tech_items' => [
        1 	=> [
			'title'		=> 'Active',
			'id'		=> 1,
			'action'	=> 'Activate',
			'state'		=> 'Activated'
        ],
        2 	=> [
			'title'		=> 'Inactive',
			'id'		=> 2,
			'action'	=> 'Inactivate',
			'state'		=> 'Inactivated'
        ],
        3 	=> [
			'title'		=> 'Ban',
			'id'		=> 3,
			'action'	=> 'Ban',
			'state'		=> 'Banned'
        ],
        4 	=> [
			'title'		=> 'Unban',
			'id'		=> 4,
			'action'	=> 'Unban',
			'state'		=> 'Unbanned'
        ],
        5 	=> [
			'title'		=> 'Deleted',
			'id'		=> 5,
			'action'	=> 'Delete',
			'state'		=> 'Deleted'
        ],
        6 	=> [
			'title'		=> 'Suspend',
			'id'		=> 6,
			'action'	=> 'Suspend',
			'state'		=> 'Suspended'
        ],
        7 	=> [
			'title'		=> 'On Hold',
			'id'		=> 7,
			'action'	=> 'On Hold',
			'state'		=> 'On Hold'
        ],
        8 	=> [
			'title'		=> 'Complete',
			'id'		=> 8,
			'action'	=> 'Complete',
			'state'		=> 'Completed'
        ],
        9 	=> [
			'title'		=> 'Invite',
			'id'		=> 9,
			'action'	=> 'Invite',
			'state'		=> 'Invited'
        ],
        10 	=> [
			'title'		=> 'Publish',
			'id'		=> 10,
			'action'	=> 'Publish',
			'state'		=> 'Published'
        ],
        11 	=> [
			'title'		=> 'Unpublish',
			'id'		=> 11,
			'action'	=> 'Unpublish',
			'state'		=> 'Unpublished'
        ],
        12 	=> [
			'title'		=> 'Never Activate',
			'id'		=> 12,
			'action'	=> 'Never Activate',
			'state'		=> 'Never Activated'
        ]
    ],
    // WARNING!! Do not change, it may break application
    'entity_ownership_id' => '598b748d-a2cd-4fda-881c-c54d853b66ac',  
    'PRODUCT_VER' => env('PRODUCT_VER', '1.1.1'),

    /* Security configurations for encrypting/decrypting form values
     * one can generate these keys using like given in below example:

        $ openssl genrsa -out rsa_1024_priv.pem 1024
        $ openssl rsa -pubout -in rsa_1024_priv.pem -out rsa_1024_pub.pem

        ---------- OR ------------

        $ openssl genrsa -out rsa_aes256_priv.pem -aes256
        $ openssl rsa -pubout -in rsa_aes256_priv.pem -out rsa_aes256_pub.pem

    ------------------------------------------------------------------------- */
    'form_encryption' => [

        /* Passphrse for RSA Key
        --------------------------------------------------------------------- */
        'default_rsa_passphrase' => '#&x3rteQmDN3KP!tmjKxb#RuWcG59@Xv&MXXmtEVp7p',

        /* Default Public RSA Key
        --------------------------------------------------------------------- */

        'default_rsa_public_key' => '-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA94WO9m6Rzq1d6y0qbxnk
1fU15S4hgGWv1/HgrfC4HU24PJz6Ob6d+Tr443aXSXYni6Yj+zB0pebTm2infIfD
2p2FffSjo9GfCcBvm07ne0+G6T70UAPYFj15EVzXkifd7BqfXdtyjtsJZD1Fh195
umGoWNeAltZkBfFJLgJHrUxIg05MiXFlLeB0Efb22sPg31BRui5z3WxD333gn5cU
0VQcjvoBngie3VoxX8C2f9JcJqHCXcxgMjc28qge5p4LIRpVF7gXuY2fg/fEoYj0
FXpvauZS4XrHTPLflWcKjf4y8bdWOPcgiXr0Nsk1SlSdmWcZXOegFz7HiuMaAk5u
HQIDAQAB
-----END PUBLIC KEY-----',

        /* Default Private RSA Key
        --------------------------------------------------------------------- */

        'default_rsa_private_key' => '-----BEGIN RSA PRIVATE KEY-----
Proc-Type: 4,ENCRYPTED
DEK-Info: DES-EDE3-CBC,A63B8E4B2CA64B30

hbwowuuTs33jxJRFwdzeoiXW0yhyyLg6++VTE3474+iysBppR9f81LAvMfkAd6Ds
tDJ7ZgAT0EbHxJskCP3Ef9zgzLrWU5RxpSlVrNdVSKu+jVzDyKgbSQPKlQx03p1G
kTyUN8+d5qx/0mHW/YqaND+m+AJZKk0YHqHGW7uA9RKvcKHma4jC2AIjT6dQWDXU
yeyIreDlA+LISlxh6keKvx/Def4LgsxwzDNGccQ3TiznafEI4YuQmlKioTPeXNME
IzPR0IaZCMcKsaRYO6O8cEvg/RR5GbxiVQQRFLYbn1/9FcVXRuZ9tQHx4EdYmwaM
khmfZ0rQcYMpu817xxFJlM216HFVTnsqAbQghU9S4JU+64A8EmMP+aL9pYS4t9rj
DjRF0JecK7VecY9mIT0bQCB3xFdNimhm3Qnuu5HvZjV6mGfkjZCvVz2xdOeXQ3L8
Kd+TCcmfatj8J33+5e2fZuEt4CtODNv83RDnoTCAjdakrQA2ULfJZPoyOPpV852u
kWi1qn+SjgHsDF1Gg68FgnNN/J9Ja63j7PfeYiDUxM+WO9wvogGP1Gce4SGXh8eI
B5aq8i+6eNd4k1zj+0kQ6jliAbdCDVYGpm4MLY1Qg5KviZ82MP5GruvMZOKa1w1R
KG0C5FeiHVTJOum1tdhqnbWkoegC1QTMzAn+AzxqtGfJrkzeitB8e8K8bylPvs3V
I0lsDEiLTUx4hyYZJhpzWg73jIPJAgiA+a6cd8Yx8xFYA9IGesjCzFdGqJBRlwTt
9wYPxkVtu1IQzB+Kc+m6HcfVtZxH6GVd7Df8N04Nue07RG8PLIsjhwoNdgMy2gsJ
ms/g87Z52FIwdPIrhykZPYIRL64muXl1U+tF1oBBMSby2jhMADnb4fn3P7qAP4YA
+a6ldRUkKhJHkN6KUJxgzEECdHKRafonoiB4clPF7umOugeR3HEZT0Q6vMruWV8h
gXGHkNuXkTIu3TqMOsvIP9UbZyiSrOF/a4/nbtz74mLFkDXPLDS33dUebMvOOIQC
rsxv0FClGaPi5umZ8uM0rdFFj1U+9sWXvnOIC6OZwtoVf24USpGXf9u1op1/fVIN
1S22WoJT4fkzjPfQBZjyWHCLpO2tv+/pxvdgkNlqyCY7762omr25EoBY91tQ2avz
Q1pXQZMYW4zr6waDG2+n1/+AqCRIeimGt+zDc+Lg4y1e4KSx/c5I2uWEf3iDmVGf
Frag96R+KDWz5OUdfzcOt9YjUaAqE3wmkmhAgZWf3Oe7+dIQoIgND/h4m+3Pbpbv
2b2NQqxW5vfbWYu3nB882RmgbOY3rpSxI2Ime+CFIM1wnltAgfg5tLkhj/oD6aNY
Bng65hu6z3o9QgD9REZ9rI4dSC2J8JEnK6d6ZuSS+h/xuA2Wr7r8qg7Og6EZfm3n
oj36oVqwQbB/JyH51iJhfAlU6CF6lRaVsVwJES47rBu+rpd+MEeWW9htsMYVjsgf
Y+MLSGYBVEt6H4qfHIINlicgYdD0Xeh7s0aazzGHnLbxqpvqgwgKScXmSnFGt9CT
/1XlhdSTH5ABydKRUlBjxdIB7hXQlFWzkSwBFyrBz9Luq+PDxdZAqqiDcK0gWJH9
-----END RSA PRIVATE KEY-----',
    ],
];

return array_merge( $techConfig, $techAppConfig );