<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Validation Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines contain the default error messages used by
    | the validator class. Some of these rules have multiple versions such
    | as the size rules. Feel free to tweak each of these messages here.
    |
    */

    'accepted' => 'must be accepted',
    'active_url' => 'is not a valid URL',
    'after' => 'must be a date after :date',
    'after_or_equal' => 'must be a date after or equal to :date',
    'alpha' => 'may only contain letters',
    'alpha_dash' => 'may only contain letters, numbers, dashes and underscores',
    'alpha_num' => 'may only contain letters and numbers',
    'array' => 'must be an array',
    'before' => 'must be a date before :date',
    'before_or_equal' => 'must be a date before or equal to :date',
    'between' => [
        'numeric' => 'must be between :min and :max',
        'file' => 'must be between :min and :max kilobytes',
        'string' => 'must be between :min and :max characters',
        'array' => 'must have between :min and :max items',
    ],
    'boolean' => 'field must be true or false',
    'confirmed' => 'confirmation does not match',
    'date' => 'is not a valid date',
    'date_format' => 'does not match the format :format',
    'different' => ':attribute and :other must be different',
    'digits' => 'must be :digits digits',
    'digits_between' => 'must be between :min and :max digits',
    'dimensions' => 'has invalid image dimensions',
    'distinct' => 'field has a duplicate value',
    'email' => 'must be a valid email address',
    'exists' => 'The selected :attribute is invalid',
    'file' => 'must be a file',
    'filled' => 'field must have a value',
    'gt' => [
        'numeric' => 'must be greater than :value',
        'file' => 'must be greater than :value kilobytes',
        'string' => 'must be greater than :value characters',
        'array' => 'must have more than :value items',
    ],
    'gte' => [
        'numeric' => 'must be greater than or equal :value',
        'file' => 'must be greater than or equal :value kilobytes',
        'string' => 'must be greater than or equal :value characters',
        'array' => 'must have :value items or more',
    ],
    'image' => 'must be an image',
    'in' => 'The selected :attribute is invalid',
    'in_array' => 'field does not exist in :other',
    'integer' => 'must be an integer',
    'ip' => 'must be a valid IP address',
    'ipv4' => 'must be a valid IPv4 address',
    'ipv6' => 'must be a valid IPv6 address',
    'json' => 'must be a valid JSON string',
    'lt' => [
        'numeric' => 'must be less than :value',
        'file' => 'must be less than :value kilobytes',
        'string' => 'must be less than :value characters',
        'array' => 'must have less than :value items',
    ],
    'lte' => [
        'numeric' => 'must be less than or equal :value',
        'file' => 'must be less than or equal :value kilobytes',
        'string' => 'must be less than or equal :value characters',
        'array' => 'must not have more than :value items',
    ],
    'max' => [
        'numeric' => 'may not be greater than :max',
        'file' => 'may not be greater than :max kilobytes',
        'string' => 'may not be greater than :max characters',
        'array' => 'may not have more than :max items',
    ],
    'mimes' => 'must be a file of type: :values',
    'mimetypes' => 'must be a file of type: :values',
    'min' => [
        'numeric' => 'must be at least :min',
        'file' => 'must be at least :min kilobytes',
        'string' => 'must be at least :min characters',
        'array' => 'must have at least :min items',
    ],
    'not_in' => 'The selected :attribute is invalid',
    'not_regex' => 'format is invalid',
    'numeric' => 'must be a number',
    'present' => 'field must be present',
    'regex' => 'format is invalid',
    'required' => 'required',
    'required_if' => 'required when :other is :value',
    'required_unless' => 'required unless :other is in :values',
    'required_with' => 'required when :values is present',
    'required_with_all' => 'required when :values are present',
    'required_without' => 'required when :values is not present',
    'required_without_all' => 'required when none of :values are present',
    'same' => ':attribute and :other must match',
    'size' => [
        'numeric' => 'must be :size',
        'file' => 'must be :size kilobytes',
        'string' => 'must be :size characters',
        'array' => 'must contain :size items',
    ],
    'string' => 'must be a string',
    'timezone' => 'must be a valid zone',
    'unique' => 'has already been taken',
    'uploaded' => 'failed to upload',
    'url' => 'format is invalid',
    'uuid' => 'must be a valid UUID',

    /*
    |--------------------------------------------------------------------------
    | Custom Validation Language Lines
    |--------------------------------------------------------------------------
    |
    | Here you may specify custom validation messages for attributes using the
    | convention "attribute.rule" to name the lines. This makes it quick to
    | specify a specific custom language line for a given attribute rule.
    |
    */

    'custom' => [
        'confirmation_code' => [
            'captcha' => __tr("The confirmation code is invalid."),
        ],
        'email' => [
                'unique_email' => __tr("The __attribute__ has already been taken.",[
                '__attribute__' => ':attribute'
                ]),
                'unique_in_email_change_request' => __tr("The __attribute__ has already been taken for new email request.",[
                    '__attribute__' => ':attribute'
                ]),
                'unique_client_member_email' => __tr("The __attribute__ has already been taken.",[
                    '__attribute__' => ':attribute'
                ])
        ],
        'new_email' => [
            'unique_email' => __tr("The __attribute__ has already been taken.",[
            '__attribute__' => ':attribute'
        ]),
        ],
        'custom_domain' => [
            'domain' => __tr("The __attribute__ format is invalid.",[
            '__attribute__' => ':attribute'
        ]),
        ],
        'comment' => [
            'verify_comment' => __tr("The __attribute__ required.", [
                '__attribute__' => ':attribute'
        ]),
        ]
    ],

    'reactions' => [
        14 => [
            __tr("Ooops... No changes made!!"),
            __tr("Ooops... Nothing to process!!"),
            __tr("It seems you didn't modified anything!!")
        ], // for reaction code 14
        15 => [
            __tr("Please wait files uploading in progress ..."),
            __tr("Please wait a while its in progress ...")
        ], // for reaction code 15
        16 => __tr("Files uploaded successfully"), // for reaction code 16
        4  => [
            __tr("Oh..no...Something left invalid!!"),
            __tr("Ooops... Validation Errrrors...!!"),
            __tr("Oh... it looks invalid...!!"),
            __tr("Something went wrong with validation!!")
        ], // for validation error
        1  => __tr("Request processed successfully"), // for request success
        6  => __tr("Invalid request access"), // for invalid request
        19 => __tr("Oooops ... something went wrong on server. Try after some time!!"), // for request success
        20 => [
            __tr("Invalid request token. Please reload page and try again!!"),
            __tr("Request token Expired. Please reload page and try again!!"),
        'attribute-name' => [
            'rule-name' => 'custom-message',
        ],
    ],

    /*
    |--------------------------------------------------------------------------
    | Custom Validation Attributes
    |--------------------------------------------------------------------------
    |
    | The following language lines are used to swap our attribute placeholder
    | with something more reader friendly such as "E-Mail Address" instead
    | of "email". This simply helps us make our message more expressive.
    |
    */

    'attributes' => [],
    ]
];
