<?php 
/*
*  Component  : Item
*  View       : Item Controller
*  Engine     : ItemEngine  
*  File       : add-dialog.blade.php  
*  Controller : ItemAddController
----------------------------------------------------------------------------- */
?>
<div>

    <!-- Modal Heading -->
    <div class="modal-header">
        <h3 class="modal-title"><?= __tr('Add New Item') ?></h3>
    </div> 
    <!-- /Modal Heading -->

    <!-- Add item dialog form -->
    <form class="ui form lw-form lw-ng-form" name="itemAddCtrl.[[itemAddCtrl.ngFormName]]"  novalidate >

        <!-- Modal Body -->
        <div class="modal-body">

            <!-- Title -->
            <lw-form-field field-for="title" label="<?= __tr('Title') ?>">
                <input type="text" 
                    class="lw-form-field form-control" 
                    ng-model="itemAddCtrl.itemData.title" name="title"   
                    ng-required="true"       
                    ng-minlength="2"       
                    ng-maxlength="150"             
                />
            </lw-form-field>
            <!-- /Title -->
           
            <!-- Short_Description -->
            <lw-form-field field-for="short_description" label="<?= __tr('Description') ?>">
                <textarea 
                    ng-model="itemAddCtrl.itemData.short_description"
                    cols="10" 
                    rows="3" 
                    class="lw-form-field form-control"
                    name="short_description"   
                    ng-required="true"                   
                ></textarea>
            </lw-form-field>
            <!-- /Short_Description -->

            <!-- Type -->
            <lw-form-radio-field field-for="type" label="Type">

				<label for="type_[[$index]]" class="p-2" ng-repeat="typeConfig in itemAddCtrl.itemConfig.type_config">

	            	<input type="radio" 
	                	ng-model="itemAddCtrl.itemData.type" 
	                	name="type" 
                        id="type_[[$index]]"
	                	ng-value="typeConfig.id" 
	               	 	ng-required="true">
                        [[::typeConfig.title]]    
				</label>

		    </lw-form-radio-field>
            <!-- /Type -->

            <!-- Slot -->
            <div class="mt-5" ng-if="itemAddCtrl.itemData.type === 1">

                <fieldset class="lw-fieldset-2">

                    <legend class="lw-fieldset-legend-font"><h3>Slots</h3></legend>

                    <div class="p-2">
                        <table class="table table-hover">
                            <thead>
                                <tr>
                                    <th scope="col">Title</th>
                                    <th scope="col">Frequency</th>
                                    <th scope="col">Slot Duration (In Min)</th>
                                    <th scope="col">Start Time</th>
                                </tr>
                            </thead>
                            <tbody>
                            <tr ng-repeat="slot in itemAddCtrl.itemData.slots" >
                                <td>
                                    <!-- Title -->
                                    <lw-form-field field-for="slot_title_[[slot.rowId]]" v-label="<?= __tr('Title') ?>" label=''>
                                        <input type="slot_title_[[slot.rowId]]" 
                                            class="lw-form-field form-control" 
                                            ng-model="slot.title" 
                                            name="slot_title_[[slot.rowId]]"   
                                            ng-required="true"       
                                            ng-minlength="2"       
                                            ng-maxlength="150"             
                                        />
                                    </lw-form-field>
                                    <!-- /Title -->
                                </td>
                                <td>
                                    <!-- Slot Frequency -->
                                    <lw-form-field field-for="slot_frequency_[[slot.rowId]]" label="" v-label='Frequency'>
                                        <selectize 
{{--                                            ng-disabled="true"--}}
                                            config="itemAddCtrl.slotFrequencyOp"
                                            class="lw-form-field form-control" 
                                            id="slot_frequency_[[slot.rowId]]" 
                                            ng-model="slot.slot_frequency" 
                                            name="slot_frequency_[[slot.rowId]]"   
                                            ng-required="true"
{{--                                            readonly--}}
                                            options="itemAddCtrl.itemConfig.slot_frequency_config">
                                        </selectize>
                                    </lw-form-field>
                                    <!-- /Slot Frequency -->
                                </td>

                                <td>
                                    <!-- Duration In Minutes -->
                                    <lw-form-field field-for="slot_duration_minutes_[[slot.rowId]]" v-label="<?= __tr('Duration') ?>" label>
                                        <input type="number" 
                                            class="lw-form-field form-control" 
                                            ng-model="slot.slot_duration_minutes" min="1" 
                                            name="slot_duration_minutes_[[slot.rowId]]"   
                                            ng-required="true"              
                                        />
                                    </lw-form-field>
                                    <!-- /Duration In Minutes -->
                                </td>
                                <td> 
                                    <!-- Slot Time -->
                                    <lw-form-field field-for="slot_time_[[slot.rowId]]" v-label="<?= __tr('Time') ?>" label>

                                        <div class="input-group input-group-sm">

                                            <input type="text" 
                                                class="lw-form-field form-control" 
                                                ng-model="slot.slot_time" 
                                                name="slot_time_[[slot.rowId]]"   
                                                ng-required="true" 
                                                readonly="readonly"
                                                lw-time-picker           
                                            />
                                        
                                            <div class="input-group-append">

                                                <button type="button" class="btn btn-outline-danger" ng-if="itemAddCtrl.itemData.slots.length <= 1" disabled><i class="fe fe-delete"></i></button> 

                                                <button type="button" class="btn btn-outline-danger" ng-if="itemAddCtrl.itemData.slots.length > 1" ng-click="itemAddCtrl.removeRow($index)"><i class="fe fe-delete"></i></button> 
                                                
                                            </div>

                                        </div>

                                    </lw-form-field>
                                    <!-- /Slot Time -->
                                </td>
                            </tr>
                            <tr>
                                <td colspan="5">
                                    <!-- Add More Slots -->
                                    <button  type="button" class="btn btn-sm btn-primary float-right" ng-click="itemAddCtrl.addNewRow()">Add Slot</button>
                                    <!-- /Add More Slots -->
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </div>

                </fieldset>

            </div>
            <!-- / Slot -->

            <!-- Status -->
            <lw-form-checkbox-field field-for="status" v-label="<?=  __tr( 'status' )  ?>">

                <label>
                    <input type="checkbox" 
                            ng-model="itemAddCtrl.itemData.status"  
                            name="status" 
                            id="status"
                            class="custom-switch-input"
                            ng-true-value="1"
                            ng-false-value="2">
                        <span class="custom-switch-indicator"></span>
                        <span class="custom-switch-description">Status</span>
                </label>

            </lw-form-checkbox-field>
            <!-- /Status -->

        </div>
        <!-- /Modal Body -->

        <!-- Modal footer -->
        <div class="modal-footer">
        
            <button type="submit" ng-click="itemAddCtrl.submit($event)" class="lw-btn btn btn-primary" title="<?= __tr('Add') ?>"><?= __tr('Add') ?></button>

            <button type="button" title="<?= __tr('Cancel') ?>" class="lw-btn btn btn-lite" ng-click="itemAddCtrl.closeDialog()"><?= __tr('Cancel') ?></button>
        </div>
        <!-- /Modal footer -->
    </form>
</div>