<?php 
/*
*  Component  : Transaction
*  View       : Transaction Controller
*  Engine     : TransactionEngine  
*  File       : add-dialog.blade.php  
*  Controller : TransactionAddController
----------------------------------------------------------------------------- */
?>
<div>
    
    <!-- Modal Heading -->
    <div class="modal-header">
        <h3 class="modal-title"><?= __tr('Add New Transaction ') ?></h3>
    </div>
    <!-- /Modal Heading -->
    

    <!-- Add transaction dialog form -->
    <form class="ui form lw-form lw-ng-form" name="transactionAddCtrl.[[transactionAddCtrl.ngFormName]]" ng-submit="transactionAddCtrl.submit()" novalidate >

        <!-- Modal Body -->
        <div class="modal-body">

            <div class="row">

            	<div class="col">
                    <!-- Subtype -->
                    <lw-form-selectize-field field-for="member_id" class="lw-selectize" label="<?= __tr('Select Towards User') ?>">
                        <selectize 
                            config='transactionAddCtrl.usersSelectizeConfig' 
                            class="lw-form-field" 
                            ng-model="transactionAddCtrl.transactionData.member_id"
                            ng-change="transactionAddCtrl.getUserBills(transactionAddCtrl.transactionData.member_id)"
                            options='transactionAddCtrl.users'
                            name="member_id"
                            placeholder="Select User"
                            ng-required="true"
                        >
                        </selectize>
                    </lw-form-selectize-field>
                    <!-- Subtype -->
                </div>

                <div class="col">

                    <!-- Select Bills -->
                    <lw-form-selectize-field field-for="bills__id" class="lw-selectize" label="<?= __tr('Select Bills') ?>">
                        <selectize 
                            config='transactionAddCtrl.billSelectizeConfig' 
                            class="lw-form-field" 
                            ng-model="transactionAddCtrl.transactionData.bills__id" 
                            options='transactionAddCtrl.bills'
                            name="bills__id" 
                            placeholder="Select Bill"
                            ng-required="true"
                            ng-change="transactionAddCtrl.onSelectBill(transactionAddCtrl.transactionData.bills__id)"                 
                        >
                        </selectize>
                    </lw-form-selectize-field>
                    <!-- Select Bills -->

                </div>
            </div>

            <div ng-if="transactionAddCtrl.selectedBillData != 0">

                <fieldset class="lw-fieldset-2">

                    <legend class="lw-fieldset-legend-font">
                        <?= __tr('Bill Payment Information') ?>
                    </legend>

                    <ul class="list-group list-group-flush">
                        <li class="list-group-item">
                            <?= __tr('Bill Amount') ?>
                            <span class="float-right" ng-bind="transactionAddCtrl.selectedBillData.formated_bill_amount"></span>
                        </li>
                        <li class="list-group-item">
                            <?= __tr('Paid Amount ') ?>
                            <span class="float-right" ng-bind="transactionAddCtrl.selectedBillData.formated_paid_amount"></span>
                        </li>
                        <li class="list-group-item">
                            <?= __tr('Balance Amount') ?>
                            <span class="float-right" ng-bind="transactionAddCtrl.selectedBillData.formated_balance_amount"></span>
                        </li>

                    </ul>

                <fieldset>

            </div>

            <div class="form-row">

                <div class="col">

                    <lw-form-field  field-for="subtype" label="<?= __tr('Payment Type') ?>">

                        <select class="lw-form-field form-control"
                            name="subtype"
                            ng-model="transactionAddCtrl.transactionData.subtype"  ng-required="true" 
                            ng-change="transactionAddCtrl.onSelectType(transactionAddCtrl.transactionData.subtype)">
                            <option ng-repeat="type in transactionAddCtrl.subtypes" ng-value="type.id">
                                [[type.title]]
                            </option>
                        </select>

                    </lw-form-field>

                </div>

                <div class="col">

                    <lw-form-field  field-for="payment_method" 
                        label="<?= __tr('Payment Method') ?>">
                        <select class="lw-form-field form-control"
                            name="payment_method"
                            ng-model="transactionAddCtrl.transactionData.payment_method"  ng-required="true">
                            <option ng-repeat="method in transactionAddCtrl.payment_methods" ng-value="method.id">
                                [[method.title]]
                            </option>
                        </select>
                    </lw-form-field>

                </div>
                
                <div class="col">
        
                    <!-- Amount -->
                    <lw-form-field field-for="amount" label="Amount">
                        <div class="input-group">
                            <div class="input-group-prepend">
                                <span class="input-group-text" ng-bind="getCurrencySymbol()"></span>
                            </div>
                            <input
                                type="number"
                                min="0"
                                class="lw-form-field form-control" 
                                ng-model="transactionAddCtrl.transactionData.amount" 
                                name="amount"  
                                ng-required="true"                
                            /> 
                            <div class="input-group-append">
                                <span class="input-group-text" ng-bind="getCurrency()"></span>
                            </div>
                        </div>
                    </lw-form-field>
                    <!-- /Amount -->
                </div>

            </div>
        
            <!-- Short_Description -->
            <lw-form-field field-for="short_description" label="<?= __tr('Short Description') ?>">
                <textarea 
                    ng-model="transactionAddCtrl.transactionData.short_description"
                    cols="10" 
                    rows="3" 
                    class="lw-form-field form-control"
                    name="short_description"      
                    ng-minlength="10"       
                    ng-maxlength="255"             
                ></textarea>
            </lw-form-field>
            <!-- /Short_Description -->

            
        </div>
        <!-- /Modal Body -->

        <!-- Modal footer -->
        <div class="modal-footer">
        
            <button type="submit" class="lw-btn btn btn-primary" title="<?= __tr('Add') ?>"><?= __tr('Add') ?></button>

            <button type="button" title="<?= __tr('Cancel') ?>" class="lw-btn btn btn-default" ng-click="transactionAddCtrl.closeDialog()"><?= __tr('Cancel') ?></button>
        </div>
        <!-- /Modal footer -->
    </form>
</div>