<?php 


/* Start - No need of filtration routes add here */
Route::get('', [
    'as'   => 'manage.app',
    'uses' => '__Igniter@manageIndex',
]);

// verify installation
Route::get('/app-configuration', [
    'as'    => 'installation.verify',
    'uses'    => 'Installation\InstallationVerification@verify'
]);

Route::get('/error-not-found', [
    'as' => 'error.public-not-found',
    'uses' => '__Igniter@errorNotFound',
]);

// get all application template using this route
Route::get('/get-template/{viewName}', [
    'as' => 'template.get',
    'uses' => '__Igniter@getTemplate',
]);

// captcha generate url
Route::get('/generate-captcha', [
    'as'    => 'security.captcha',
    'uses'    => '__Igniter@captcha'
]);

// get all application template using this route
Route::get('/email-view/{viewName}', [
    'as'    => 'template.email',
    'uses'    => '__Igniter@emailTemplate'
]);
/* End - No need of filtration routes add here */

/*
  Start here authority.checkpost filtered routes
-------------------------------------------------------------------------- */

Route::group(['middleware' => 'authority.checkpost'], function () 
{

	Route::get('base-data', [
	    'as'    => 'base_data',
	    'uses' => '__Igniter@baseData',
	]);

    /* Start User Routes*/
    Route::group([       
        'namespace' => 'User\Controllers',
        'prefix'    => 'user',
    ], function () {

        // profile
        Route::get('/profile', [
            'as' => 'user.profile',
            'uses' => 'UserController@profile',
        ]);

        // change password
        Route::get('/change-password', [
            'as' => 'user.change_password',
            'uses' => 'UserController@changePassword',
        ]);

        // new email activation
        Route::get('/{userID}/{activationKey}/new-email-activation', [
            'as' => 'user.new_email.activation',
            'uses' => 'UserController@newEmailActivation',
        ]);

        // profile edit support data
        Route::get('/get-country-list', [
            'as'    => 'user.get.country_list',
            'uses'  => 'UserController@getCountries',
        ]);

    });
    /* End User Routes*/

    /* Start Profile Media */
    Route::group([
        'namespace' => 'Media\Controllers',
        'prefix'    => 'profile-media',
    ], function () {
        // upload image media detail
        Route::get('/read-uploaded-user-profile-files-detail', [
            'as'   => 'media.upload.read_user_profile',
            'uses' => 'MediaController@readUserProfileFiles',
        ]);

        Route::post('/upload-user-profile', [
            'as'   => 'media.upload.write.user_profile',
            'uses' => 'MediaController@uploadUserProfile',
        ]);
    });
    /* End Profile Media*/


    /**
     * Start api section routes after login
     * ---------------------------------------------------------------------------------------- */
    Route::group([ 'prefix'  => 'api'], function () 
    {

        /*
        Start Dashboard Components Manage Section Related Routes
        ------------------------------------------------------------------- */
        Route::group([
            'namespace' => 'Dashboard\Controllers',
            'prefix'    => 'dashboard'
        ], function () {

            // get details of dashboard data
            Route::get('/get-data', [
                'as'   => 'manage.dashboard.read.support_data',
                'uses' => 'DashboardController@dashboardSupportData',
            ]);

        });
        /*
        End Dashboard Components Manage Section Related Routes
        ------------------------------------------------------------------- */

        /*
        Start Company Activity-log Component Related Routes 
        ----------------------------------------------------------------------- */
        //Activity Log
        Route::group([ 
            'namespace'     => 'ActivityLog\Controllers',
            'prefix'        => '/activity' 
        ], function() {

            // Activity list
            Route::get('/{startDate}/{endDate}/list', [
                'as'    => 'manage.activity_log.read.list',
                'uses'  => 'ActivityLogController@prepareActivityLogList'
            ]);

            // Action type details
            Route::get('/{activityLogId}/action-type-details', [
                'as'    => 'manage.activity_log.action_type.read.data',
                'uses'  => 'ActivityLogController@prepareActionTypeData'
            ]);

        });

        /*
        Start User Role Permission Components Manage Section Related Routes
        ------------------------------------------------------------------- */
        Route::group([
            'namespace'     => 'User\Controllers',
            'prefix'        => '/role-permission'
        ], function() {

            // Get Role Add Support Data
            Route::get('/role-add-support-data', [
                'as'    => 'manage.user.role_permission.read.add_support_data',
                'uses'  => 'RolePermissionController@getAddSuppotData'
            ]);

            // Get role all permission using id
            Route::get('/{roleId}/get-permission', [
                'as'    => 'manage.user.role_permission.read.using_id',
                'uses'  => 'RolePermissionController@getPermissionById'
            ]);

            // Add New Role Permissions
            Route::post('/add-process', [
                'as'    => 'manage.user.role_permission.write.role.create',
                'uses'  => 'RolePermissionController@addNewRole'
            ]);

            // Role Permission list
            Route::get('/list', [
                'as'    => 'manage.user.role_permission.read.list',
                'uses'  => 'RolePermissionController@prepareRolePermissionList'
            ]);

            // Role Permission delete process
            Route::post('/{rolePermissionIdOrUid}/delete-process', [
                'as'    => 'manage.user.role_permission.write.delete',
                'uses'  => 'RolePermissionController@processRolePermissionDelete'
            ]);

            // Get Role Permissions
            Route::get('/{roleId}/permissions', [
                'as'    => 'manage.user.role_permission.read',
                'uses'  => 'RolePermissionController@getPermissions'
            ]);

            // Create User Role Dynamic Permission
            Route::post('/{roleId}/add-dynamic-permission-process', [
                'as'    => 'manage.user.role_permission.write.create',
                'uses'  => 'RolePermissionController@processDynamicRolePermission'
            ]);

        });
        /*
        End User Role Permission Components Manage Section Related Routes
        ------------------------------------------------------------------- */


        /*
        Start User Components Manage Section Related Routes
        ------------------------------------------------------------------- */

        Route::group([
            'namespace' => 'User\Controllers',
            'prefix'    => 'user',
        ], function () {

            Route::post('/logout', [
                'as'   => 'user.logout',
                'uses' => 'UserController@logout',
            ]);

            // change email support data
            Route::get('/change-email-support-data', [
                'as' => 'user.change_email.support_data',
                'uses' => 'UserController@getChangeEmailSupportData',
            ]);
            
            // change password process
            Route::post('/change-password', [
                'as' => 'user.change_password.process',
                'uses' => 'UserController@changePasswordProcess',
            ]);

            // profile details
            Route::get('/profile-details', [
                'as' => 'user.profile.details',
                'uses' => 'UserController@profileDetails',
            ]);

            // profile edit support data
            Route::get('/profile/edit-support-data', [
                'as'    => 'user.profile.edit_support_data',
                'uses'    => 'UserController@profileEditSupportData',
            ]);

            // profile update
            Route::get('/profile/edit', [
                'as' => 'user.profile.update',
                'uses' => 'UserController@updateProfile',
            ]);

            // profile update process
            Route::post('/profile/edit', [
                'as'   => 'user.profile.update.process',
                'uses' => 'UserController@updateProfileProcess',
            ]);

            // change email process
            Route::post('/change-email', [
                'as' => 'user.change_email.process',
                'uses' => 'UserController@changeEmailProcess',
            ]);

            // fetch users list for datatable
            Route::get('/{status}/fetch-list', [
                'as'   => 'manage.user.read.datatable.list',
                'uses' => 'UserController@index',
            ])->where('status', '[0-9]+');

             // fetch users detail data
            Route::get('/{userID}/user-detail', [
                'as'   => 'manage.user.read.detail.data',
                'uses' => 'UserController@userDetailData',
            ]);

            // delete user
            Route::post('/{userID}/delete', [
                'as'   => 'manage.user.write.delete',
                'uses' => 'UserController@delete',
            ])->where('userID', '[0-9]+');

            // restore user
            Route::post('/{userID}/restore', [
                'as'   => 'manage.user.write.restore',
                'uses' => 'UserController@restore',
            ])->where('userID', '[0-9]+');

            // change password by admin process
            Route::post('/{userID}/change-password', [
                'as'   => 'manage.user.write.change_password.process',
                'uses' => 'UserController@changePasswordByAdmin',
            ])->where('userID', '[0-9]+');

            // fetch users details
            Route::get('/{userID}/contact', [
                'as'   => 'manage.user.read.contact',
                'uses' => 'UserController@contact',
            ])->where('status', '[0-9]+');

            // process contact form
            Route::get('/{userId}/get-user-info', [
                'as'   => 'manage.user.read.info',
                'uses' => 'UserController@getInfo',
            ]);

            // Get add support data
            Route::get('/add-support-data', [
                'as'    => 'manage.user.read.create.support_data',
                'uses'  => 'UserController@getAddSupportData',
            ]);

            // Get User available permissions
            Route::get('/{userId}/get-user-permissions', [
                'as'    => 'manage.user.read.get_user_permissions',
                'uses'  => 'UserController@getUserPermissions',
            ]);

            // Get User Edit Data
            Route::get('/{userId}/get-user-edit data', [
                'as'    => 'manage.user.read.edit_suppport_data',
                'uses'  => 'UserController@getUserEditSupportData',
            ]);

            // Store user dynamic permissions
            Route::post('/{userId}/user-update-process', [
                'as'    => 'manage.user.write.update_process',
                'uses'  => 'UserController@processUserUpdate',
            ]);

            // Store user dynamic permissions
            Route::post('/{userId}/user-dynamic-permission-process', [
                'as'    => 'manage.user.write.user_dynamic_permission',
                'uses'  => 'UserController@processUserPermissions',
            ]);

            // Add New User
            Route::post('/add', [
                'as'   => 'manage.user.write.create',
                'uses' => 'UserController@add',
            ]);

            // get users list
            Route::get('/get-users-list', [
                'as'   => 'manage.user.read.list',
                'uses' => 'UserController@prepareReadUsers',
            ]);

        });
        /*
        End User Components Manage Section Related Routes
        ------------------------------------------------------------------- */  

        /*
        Start package Manage Section Related Routes
        ------------------------------------------------------------------- */
        Route::group([ 
            'namespace'     => 'Package\Controllers',
            'prefix'        => '/package' 
        ], function() {


            // Package list
            Route::get('/list', [
                'as'    => 'manage.package.read.list',
                'uses'  => 'PackageController@preparePackageList'
            ]);

 
            // Package delete process
            Route::post('/{packageIdOrUid}/delete-process', [
                'as'    => 'manage.package.write.delete',
                'uses'  => 'PackageController@processPackageDelete'
            ]);

            // Package create support data
            Route::get('/add-support-data', [
                'as'    => 'manage.package.read.support_data',
                'uses'  => 'PackageController@preparePackageSupportData'
            ]);

            // Package create process
            Route::post('/add-process', [
                'as'    => 'manage.package.write.create',
                'uses'  => 'PackageController@processPackageCreate'
            ]);

            // Package get the data
            Route::get('/{packageIdOrUid}/get-update-data', [
                'as'    => 'manage.package.read.update.data',
                'uses'  => 'PackageController@updatePackageData'
            ]);

            // Package update process
            Route::post('/{packageIdOrUid}/update-process', [
                'as'    => 'manage.package.write.update',
                'uses'  => 'PackageController@processPackageUpdate'
            ]);


            // Package get details
            Route::get('/{packageIdOrUid}/details', [
                'as'    => 'manage.package.read.details',
                'uses'  => 'PackageController@preparePackageDetails'
            ]);

            // user Package update
            Route::post('/update-user-package-price', [
                'as'    => 'manage.package.write.update_user_package',
                'uses'  => 'PackageController@processUserPackageUpdate'
            ]);

        });

        /*
        End Package Components Manage Section Related Routes
        ------------------------------------------------------------------- */


        /*
        Start User Purchases Manage Section Related Routes
        ------------------------------------------------------------------- */
        Route::group([ 
            'namespace'     => 'Package\Controllers',
            'prefix'        => '/user-purchase' 
        ], function() {

            // Prepare User Purchases
            Route::get('/get-support-data', [
                'as'    => 'manage.user_purchses.read.users_purchases',
            'uses'  => 'PackageController@prepareUsersPurchases'
            ]);

            // Prepare User Purchases
            Route::get('/{requestFor}/{userPackageUid}/support-data', [
                'as'    => 'manage.user_purchses.read.user_purchase',
                'uses'  => 'PackageController@prepareUserPurchase'
            ]);

        });

        /*
        End User Purchases Components Manage Section Related Routes
        ------------------------------------------------------------------- */
              

        /*
        Start Customer Components Manage Section Related Routes
        ------------------------------------------------------------------- */

        Route::group([
            'namespace' => 'User\Controllers',
            'prefix'    => 'customer',
        ], function () {

            // fetch users list for datatable
            Route::get('/{status}/fetch-customers', [
                'as'   => 'manage.customer.read.datatable.list',
                'uses' => 'UserController@fetchCustomersList',
            ])->where('status', '[0-9]+');

             // fetch users detail data
            Route::get('/{memberId}/customer-detail', [
                'as'   => 'manage.customer.read.detail.data',
                'uses' => 'UserController@customerDetailData',
            ]);

            // delete customer
            Route::post('/{userID}/delete-customer', [
                'as'   => 'manage.customer.write.delete',
                'uses' => 'UserController@delete',
            ])->where('userID', '[0-9]+');

            // restore customer
            Route::post('/{userID}/restore', [
                'as'   => 'manage.customer.write.restore',
                'uses' => 'UserController@restore',
            ])->where('userID', '[0-9]+');

            // change password by admin process
            Route::post('/{userID}/change-password', [
                'as'   => 'manage.customer.write.change_password.process',
                'uses' => 'UserController@changePasswordByAdmin',
            ])->where('userID', '[0-9]+');

            // Get User Edit Data
            Route::get('/{userId}/get-customer-edit-data', [
                'as'    => 'manage.customer.read.edit_suppport_data',
                'uses'  => 'UserController@getUserEditSupportData',
            ]);

            // update customer
            Route::post('/{userId}/customer-update-process', [
                'as'    => 'manage.customer.write.update_process',
                'uses'  => 'UserController@processUserUpdate',
            ]);

            // Get add support data
            Route::get('/add-customer-support-data', [
                'as'    => 'manage.customer.read.create.support_data',
                'uses'  => 'UserController@getCustomerAddSupportData',
            ]);

            // Add New customer
            Route::post('/add', [
                'as'   => 'manage.customer.write.create',
                'uses' => 'UserController@add',
            ]);

             // chnage slot 
            Route::post('{userId}/change-slot', [
                'as'   => 'manage.customer.write.change_slot',
                'uses' => 'UserController@changeSlot',
            ]);

            // Package get details
            Route::get('/package-assign-data', [
                'as'    => 'manage.customer.package.read.package_assign_data',
                'uses'  => 'UserController@prepareAssignPackageData'
            ]);

            // Package get details
            Route::get('/customer-package', [
                'as'    => 'manage.customer.package.read.package_assign_member',
                'uses'  => 'UserController@prepareNewCustomerPackageData'
            ]);

            // Package update process
            Route::post('/{userId}/assign-package', [
                'as'    => 'manage.customer.package.write.assign_to_user',
                'uses'  => 'UserController@processPackageAssign'
            ]);

            // Package update process
            Route::post('/customer-package', [
                'as'    => 'manage.customer.package.write.package_assign_member',
                'uses'  => 'UserController@processNewCustomerPackageAssign'
            ]);

            // Package update process
            Route::post('/delete-user-slot', [
                'as'    => 'manage.slot.write.delete_item_slot',
                'uses'  => 'UserController@processUserSlotDelete'
            ]);

        });

        /*
        End Customer Components Manage Section Related Routes
        ------------------------------------------------------------------- */      

        /*
        Start Configuration Components Manage Section Related Routes
        ------------------------------------------------------------------- */

        Route::group([
            'namespace' => 'Configuration\Controllers',
            'prefix'    => 'configuration',
        ], function () {

            // process configuration
            Route::post('/process/{formType}', [
                'as'   => 'manage.configuration.process',
                'uses' => 'ConfigurationController@process',
            ]);


            // process get configuration
            Route::get('/get-support-data/{formType}', [
                'as'   => 'manage.configuration.get.support.data',
                'uses' => 'ConfigurationController@getSupportData',
            ]);
        });
        /*
        End Configuration Components Manage Section Related Routes
        ------------------------------------------------------------------- */

        /*
        Media Components Public Section Related Routes
        ----------------------------------------------------------------------- */

        Route::group([
            'namespace' => 'Media\Controllers',
            'prefix'    => 'media',
        ], function () {

            // upload all media
            Route::post('/upload-logo', [
                'as'   => 'media.upload.write.logo',
                'uses' => 'MediaController@uploadLogo',
            ]);

            // delete media file
            Route::post('/{fileName}/delete', [
                'as'   => 'media.upload.delete',
                'uses' => 'MediaController@delete',
            ]);

            // delete multiple media files
            Route::post('/multiple-delete', [
                'as'   => 'media.upload.delete_multiple',
                'uses' => 'MediaController@multipleDeleteFiles',
            ]);

            // upload image media detail
            Route::get('/read-uploaded-files-detail', [
                'as'   => 'media.upload.read_logo',
                'uses' => 'MediaController@readLogoFiles',
            ]);

            // upload image media detail
            Route::get('/read-uploaded-favicon-files-detail', [
                'as'   => 'media.upload.read_favicon',
                'uses' => 'MediaController@readFaviconFiles',
            ]);

        });
        
        /* Start Items Routes */
        
        Route::group([ 
            'namespace'     => 'Item\Controllers',
            'prefix'        => '/item' 
        ], function() {

             
            // Item list
            Route::get('/list', [
                'as'    => 'manage.item.read.list',
                'uses'  => 'ItemController@prepareItemList'
            ]);

 
            // Item delete process
            Route::post('/{itemIdOrUid}/delete-process', [
                'as'    => 'manage.item.write.delete',
                'uses'  => 'ItemController@processItemDelete'
            ]);

            // Item create support data
            Route::get('/add-support-data', [
                'as'    => 'manage.item.read.support_data',
                'uses'  => 'ItemController@prepareItemSupportData'
            ]);

            // Item create process
            Route::post('/add-process', [
                'as'    => 'manage.item.write.create',
                'uses'  => 'ItemController@processItemCreate'
            ]);

            // Item create process
            Route::post('/{itemIdOrUid}/add-new-slot-process', [
                'as'    => 'manage.item.add.slots',
                'uses'  => 'ItemController@processItemSlotCreate'
            ]);

            // Item get the data
            Route::get('/{itemIdOrUid}/get-update-data', [
                'as'    => 'manage.item.read.update.data',
                'uses'  => 'ItemController@updateItemData'
            ]);

            // Item update process
            Route::post('/{itemIdOrUid}/update-process', [
                'as'    => 'manage.item.write.update',
                'uses'  => 'ItemController@processItemUpdate'
            ]);

            // Item Details
            Route::get('/{itemIdOrUid}/details', [
                'as'    => 'manage.item.read.deatils',
                'uses'  => 'ItemController@details'
            ]);

        });
        
        /* End Items Routes */
        
        /* Start Bill Routes */
        Route::group([ 
            'namespace' => 'Bill\Controllers',
            'prefix'    => '/bill' 
        ], function() {
             
            // List
            Route::get('/{status}/list', [
                'as'    => 'manage.bill.read.list',
                'uses'  => 'BillController@prepareBillList'
            ]);

            // Generate bill
            Route::post('/process-generate-bill', [
                'as'    => 'manage.bill.write.generate',
                'uses'  => 'BillController@processGenerate'
            ]);

            // Get bill details
            Route::get('/{billUid}/get-details', [
                'as'    => 'manage.bill.read.details',
                'uses'  => 'BillController@readDetails'
            ]);

            // Get upcomming bills
            Route::get('/get-upcomming-bills', [
                'as'    => 'manage.bill.read.upcomming_bills',
                'uses'  => 'BillController@readUpcommingBills'
            ]);

            // Get bill edit data
            Route::get('/{billUid}/get-bill-edit-data', [
                'as'    => 'manage.bill.read.edit_support_data',
                'uses'  => 'BillController@readBillEditDetails'
            ]);

            // Generate bill
            Route::post('/{billUid}/process-edit-bill', [
                'as'    => 'manage.bill.write.update',
                'uses'  => 'BillController@processBillEdit'
            ]);

            // Get bill payment details
            Route::get('/{billUid}/get-bill-payment-details', [
                'as'    => 'manage.bill.read.bill_payment_details',
                'uses'  => 'BillController@readBillPaymentDetails'
            ]);

            // Invoice Print
            Route::get('/{billUid}/print-bill', [
                'as'    => 'manage.bill.read.print_bill',
                'uses'  => 'BillController@printBill'
            ]);

            // process-cancel
            Route::post('/{billUid}/process-cancel', [
                'as'    => 'manage.bill.write.process_cancel',
                'uses'  => 'BillController@processCancel'
            ]);

            // process-delete
            Route::post('/{billUid}/process-delete', [
                'as'    => 'manage.bill.write.process_delete',
                'uses'  => 'BillController@processDelete'
            ]);

        });
        /* End Bill Routes */
        
         /* Start Trnsaction Routes */
        Route::group([ 
            'namespace'     => 'Transaction\Controllers',
            'prefix'        => '/transaction' 
        ], function() {

             
            // Transaction list
            Route::get('/list', [
                'as'    => 'manage.transaction.read.list',
                'uses'  => 'TransactionController@prepareTransactionList'
            ]);

            // Transaction create support data
            Route::get('/add-support-data', [
                'as'    => 'manage.transaction.read.support_data',
                'uses'  => 'TransactionController@prepareTransactionSupportData'
            ]);

            // Transaction create process
            Route::post('/add-process', [
                'as'    => 'manage.transaction.write.create',
                'uses'  => 'TransactionController@processTransactionCreate'
            ]);

            // Transaction update process
            Route::post('/{billUid}/process-pay-bill', [
                'as'    => 'manage.transaction.write.process_pay_bill',
                'uses'  => 'TransactionController@processPayBill'
            ]);

            // Transaction get the data
            Route::get('/{memberId}/get-user-bills', [
                'as'    => 'manage.transaction.read.user_bills',
                'uses'  => 'TransactionController@getUserBills'
            ]);
        });
        /* End Trnsaction Routes */


        


    });
    /**
    * End api section routes after login
    * ---------------------------------------------------------------------------------------- */

});

/*
 end Start here authority.checkpost filtered routes
-------------------------------------------------------------------------- */

/*
  Start here authority.pre.checkpost filtered routes
-------------------------------------------------------------------------- */

Route::group(['middleware' => 'authority.pre.checkpost'], function () 
{
    /*
    Start User Components Public Section Related Routes
    ----------------------------------------------------------------------- */

    Route::group([
        'namespace' => 'User\Controllers', 
        'prefix'    => 'user'
    ], function () {

        // login process
        Route::post('/process-login', [
            'as' => 'user.login.process',
            'uses' => 'UserController@loginProcess',
        ]);

        // register
        Route::get('/register', [
            'as' => 'user.register',
            'uses' => 'UserController@register',
        ]);

        // register process
        Route::post('/register', [
            'as' => 'user.register.process',
            'uses' => 'UserController@registerProcess',
        ]);

        // register success
        Route::get('/register/success', [
            'as' => 'user.register.success',
            'uses' => 'UserController@registerSuccess',
        ]);

        // login attempts
        Route::get('/login-attempts', [
            'as' => 'user.login.attempts',
            'uses' => 'UserController@loginAttempts',
        ]);

        // forgot password
        Route::post('/forgot-password', [
            'as' => 'user.forgot_password.process',
            'uses' => 'UserController@forgotPasswordProcess',
        ]);

        // forgot password success
        Route::get('/forgot-password-success', [
            'as' => 'user.forgot_password.success',
            'uses' => 'UserController@forgotPasswordSuccess',
        ]);

        // reset password
        Route::get('/reset-password/{reminderToken}', [
            'as' => 'user.reset_password',
            'uses' => 'UserController@restPassword',
        ]);

        // reset password process
        Route::post('/reset-password/{reminderToken}', [
            'as' => 'user.reset_password.process',
            'uses' => 'UserController@restPasswordProcess',
        ]);

        // new user activation process
        Route::post('/resend-activation-email', [
            'as'   => 'user.resend.activation.email.proccess',
            'uses' => 'UserController@resendActivationEmailProccess',
        ]);

    });

    /*
    End User Components Public Section Related Routes
    ----------------------------------------------------------------------- */
});

/*
 end Start here authority.pre.checkpost filtered routes
-------------------------------------------------------------------------- */