(function () {
    'use strict';

    angular.module('ManageApp', [
        'ngMessages',
        'ngAnimate',
        'ngSanitize',
        'ui.router',
        'ui.router.state.events',
        'ngNotify',
        'ngDialog',
        'selectize',
        'NgSwitchery',
        'lw.core.utils',
        'lw.security.main',
        'lw.auth',
        'lw.data.datastore',
        'lw.data.datatable',
        'lw.form.main',
        'app.service',
        'app.http',
        'app.notification',
        'app.form',
        'app.directives',
        'app.fileUploader',

        'ManageApp.master',
        'app.UploaderDataService',
        'app.UploaderEngine',

        'ManageApp.ManageUserDataService',
        'ManageApp.users',

        'CommonApp.CommonUserDataService',
        'CommonApp.users',

        'Manage-app.users',
        'ManageApp.UserDataService',

        'ManageApp.ConfigurationDataService',
        'ManageApp.configuration',

        'ManageApp.DashboardDataService',
        'ManageApp.DashboardEngine',

        'app.RolePermissionDataServices',
        'app.RolePermissionEngine',

        'app.ActivityDataServices',
        'app.ActivityEngine',

        'app.ItemDataServices',
        'app.ItemEngine',
        'app.SlotsDataServices',
        'app.SlotsEngine',

        'app.PackageDataServices',
        'app.PackageEngine',
        'app.BillDataServices',
        'app.BillEngine',

        'app.TransactionDataServices',
        'app.TransactionEngine'

    ]).
        //constant('__ngSupport', window.__ngSupport).
        run([
            '__Auth', '$state', '$rootScope', '$transitions', function (__Auth, $state, $rootScope, $transitions) {

                //
                _.defer(function () {
                    // __Auth.verifyRoute($state);
                });

                $rootScope.$on('$stateChangeStart', function (event, stateObj, paramObj) {
                    $rootScope.$broadcast('lw.events.state.change_start', stateObj, paramObj);
                });

                $rootScope.__ngSupport = window.__ngSupport;

            }
        ]).config([
        '$stateProvider', '$urlRouterProvider', '$interpolateProvider', '$compileProvider', routes
    ]);


    /**
     * Application Routes Configuration
     *
     * @inject $stateProvider
     * @inject $urlRouterProvider
     * @inject $interpolateProvider
     * @inject $compileProvider
     *
     * @return void
     *---------------------------------------------------------------- */

    function routes($stateProvider, $urlRouterProvider, $interpolateProvider, $compileProvider) {

        if (window.appConfig && window.appConfig.debug === false) {
            $compileProvider.debugInfoEnabled(false);
        }

        $interpolateProvider.startSymbol('[[');
        $interpolateProvider.endSymbol(']]');

        /*$urlRouterProvider
       .otherwise('/dashboard');*/

        $urlRouterProvider.otherwise(function ($injector, $location, $transitions) {
            var state = $injector.get('$state'),
                auth = $injector.get('__Auth'),
                redirectState = __globals.appTemps('stateViaRoute');
            if (_.has(redirectState, 'stateName')
                && !_.isUndefined(redirectState.stateName)
                && !_.isEmpty(redirectState.stateName)) {
                if (_.isEmpty(redirectState.stateParams)) {
                    state.go(redirectState.stateName);
                } else {
                    state.go(redirectState.stateName, redirectState.stateParams);
                }
            } else {
                state.go('dashboard');
            }

            return $location.path();
        });

        //state configurations
        $stateProvider

            .state('base', {
                abstract: true,
                resolve: {
                    baseData: ['$stateParams', 'BaseDataService', function ($stateParams, BaseDataService) {

                        return BaseDataService.getBaseData('account_logged');
                    }]
                }
            })

            // login
            .state('login',
                __globals.stateConfig('/login', 'user.login', {
                    access: {
                        authority: 'user.login.process'
                    },
                    parent: 'base'
                }))

            // forgot password
            .state('forgot_password',
                __globals.stateConfig('/forgot-password', 'user.forgot-password', {
                    access: {
                        authority: 'user.forgot_password.process'
                    },
                })
            )

            // forgot password
            .state('reset_password',
                __globals.stateConfig('/reset-password/{reminderToken}', 'user.reset-password', {})
            )

            // Forgot Password Success
            .state('forgot_password_sucess',
                __globals.stateConfig('/forgot-password', 'user.forgot-password-success', {})
            )

            // home
            .state('home',
                __globals.stateConfig('/home', 'home', {
                        access: {
                            authority: 'public.app'
                        },
                        parent: 'base'
                    }
                )
            )

            // invalid request
            .state('invalid_request', __globals.stateConfig('/invalid-request',
                'errors/invalid-request', {
                    parent: 'base'
                }
            ))

            // not found
            .state('not_found', __globals.stateConfig('/not-found',
                'errors.manage-not-exist', {
                    parent: 'base'
                }
            ))

            // not exist
            .state('not_exist', __globals.stateConfig('/not-exist',
                'errors.manage-not-exist', {
                    parent: 'base'
                }
            ))

            // unauthorized
            .state('unauthorized', __globals.stateConfig('/unauthorized',
                'errors.unauthorized', {
                    parent: 'base'
                }
            ))

            // dashboard
            .state('dashboard',
                __globals.stateConfig('/dashboard', 'dashboard.admin-dashboard', {
                        controller: 'AdminDashboardController as AdminDashboardCtrl',
                        access: {
                            authority: 'manage.dashboard.read.support_data'
                        },
                        parent: 'base',
                        resolve: {
                            GetDatshboardData: ["DashboardDataService", function (DashboardDataService) {
                                return DashboardDataService.getSupportData();
                            }]
                        },
                    }
                )
            )

            // customers
            .state('customers',
                __globals.stateConfig('/members', 'user/customer/list', {
                        controller: 'ManageCustomersController as manageCustomersCtrl',
                        access: {
                            authority: 'manage.customer.read.datatable.list'
                        },
                        parent: 'base'
                    }
                )
            )

            // customers
            .state('customer_detail',
                __globals.stateConfig('/member/:memberId/detail', 'user/customer/user-detail-dialog', {
                        controller: 'ManageCustomerDetailController as ManageCustomerDetailCtrl',
                        access: {
                            authority: 'manage.customer.read.detail.data'
                        },
                        parent: 'base'
                    }
                )
            )

            .state('assign_package',
                __globals.stateConfig('/assign/{memberId}/package', 'user/customer/assign-package-dialog', {
                        controller: 'AssignPackageController as AssignPackageCtrl',
                        parent: 'base',
                        access: {
                            authority: 'manage.user_purchses.read.data'
                        },
                        resolve: {
                            AssignPackageData: ['PackageDataService', function (PackageDataService) {
                                return PackageDataService.getAssignPackageSupportData();
                            }],
                            userData: ['ManageUserDataService', '$stateParams', function (ManageUserDataService, $stateParams) {
                                console.log($stateParams.memberId);
                                return ManageUserDataService.getCustomerDetailData($stateParams.memberId);
                            }],

                        }
                    }
                )
            )

            .state('new_customer',
                __globals.stateConfig('/new-assign/package', 'user/customer/add-customer', {
                        controller: 'NewCustomerPackageController as NewCustomerPackageCtrl',
                        parent: 'base',
                        access: {
                            authority: 'manage.customer.package.read.package_assign_member'
                        },
                        resolve: {
                            AssignPackageData: ['PackageDataService', function (PackageDataService) {
                                return PackageDataService.getAssignPackageSupportData();
                            }]

                        }
                    }
                )
            )

            // users
            .state('users',
                __globals.stateConfig('/users', 'user/manage/list', {
                        controller: 'ManageUsersController as manageUsersCtrl',
                        access: {
                            authority: 'manage.user.read.datatable.list'
                        },
                        parent: 'base'
                    }
                )
            )

            // RolePermission list
            .state('role_permission',
                __globals.stateConfig('/role-permissions', 'user/role-permission/list', {
                    access: {
                        authority: 'manage.user.role_permission.read.list'
                    },
                    controller: 'RolePermissionListController as rolePermissionListCtrl',
                    parent: 'base'
                }))

            // profile
            .state('profile',
                __globals.stateConfig('/profile', 'user/manage-profile', {
                        access: {
                            authority: 'user.profile.update'
                        },
                        parent: 'base'
                    }
                )
            )

            // profile edit
            .state('profileEdit',
                __globals.stateConfig('/profile/edit', 'user/profile-edit', {
                        access: {
                            authority: 'user.profile.update.process'
                        },
                        controller: 'UserProfileEditController as profileEditCtrl',
                        resolve: {
                            profileData: ['CommonUserDataService', function (CommonUserDataService) {
                                return CommonUserDataService.getUserProfileEditData();
                            }]
                        },
                        parent: 'base'
                    }
                )
            )

            // change password
            .state('changePassword',
                __globals.stateConfig('/change-password', 'user/change-password', {
                        access: {
                            authority: 'user.change_password.process'
                        },
                        parent: 'base'
                    }
                )
            )

            // change email
            .state('changeEmail',
                __globals.stateConfig('/change-email', 'user/change-email', {
                        access: {
                            authority: 'user.change_email.process'
                        },
                        parent: 'base'
                    }
                )
            )

            // activity log
            .state('activity_log',
                __globals.stateConfig('/activity-log', 'activity/activity-log', {
                    controller: 'ActivityLogListController as activityLogListCtrl',
                    parent: 'base'
                }))

            // categories
            .state('categories',
                __globals.stateConfig('/categories', 'category/list', {
                    controller: 'CategoryListController as CategoryListCtrl',
                    access: {
                        authority: 'manage.category.read.list'
                    },
                    parent: 'base'
                }))


            // Customer list
            .state('customer',
                __globals.stateConfig('/customers', 'customer/list', {
                    // access: {},
                    controller: 'CustomerListController as customerListCtrl',
                    access: {
                        authority: 'manage.customer.read.list'
                    },
                    parent: 'base'
                }))


            // Package list
            .state('package',
                __globals.stateConfig('/packages', 'package/list', {
                    access: {},
                    controller: 'PackageListController as packageListCtrl',
                    parent: 'base'
                }))

            // Package add
            .state('package_add',
                __globals.stateConfig('/package/add', 'package/add', {
                    access: {},
                    controller: 'PackageAddController as packageAddCtrl',
                    parent: 'base',
                    resolve: {
                        packageAddData: ['PackageDataService', function (PackageDataService) {
                            return PackageDataService.getAddSupportData();
                        }]
                    }
                }))


            // Package edit
            .state('package_edit',
                __globals.stateConfig('/package/:packageIdOrUid/edit', 'package/edit', {
                    access: {},
                    controller: 'PackageEditController as packageEditCtrl',
                    parent: 'base',
                    resolve: {
                        packageEditData: ['PackageDataService', '$stateParams', function (PackageDataService, $stateParams) {
                            return PackageDataService.getEditSupportData($stateParams.packageIdOrUid);
                        }]
                    }
                }))


            // Item list
            .state('items',
                __globals.stateConfig('/items', 'item/list', {
                    parent: 'base',
                    access: {},
                    controller: 'ItemListController as itemListCtrl'
                }))



            // Bill list
            .state('bill',
                __globals.stateConfig('/bills', 'bill/list', {
                    parent: 'base',
                    access: {
                        authority: 'manage.bill.read.list'
                    }
                }))

            // list bill general
            .state('bill.general',
                __globals.stateConfig('/general', 'bill/list', {
                        access: {
                            authority: 'manage.bill.read.list'
                        }
                    }
                )
            )

            // list bill paid
            .state('bill.paid',
                __globals.stateConfig('/paid', 'bill/list', {
                        access: {
                            authority: 'manage.bill.read.list'
                        }
                    }
                )
            )

            // list bill upcoming
            .state('bill.upcoming',
                __globals.stateConfig('/upcoming', 'bill/list', {
                        access: {
                            authority: 'manage.bill.read.list'
                        }
                    }
                )
            )

            // list bill cancelled
            .state('bill.cancelled',
                __globals.stateConfig('/cancelled', 'bill/list', {
                        access: {
                            authority: 'manage.bill.read.list'
                        }
                    }
                )
            )


            // Bill list
            .state('generate_bill',
                __globals.stateConfig('/generate-bill', 'bill/add', {
                    parent: 'base',
                    access: {
                        authority: 'manage.user_purchses.read.users_purchases'
                    },
                    controller: 'BillAddController as BillAddCtrl',
                    resolve: {
                        GetUserPurchases: ['PackageDataService', function (PackageDataService) {
                            return PackageDataService.getUsersPurchases();
                        }]
                    }
                }))

            // Bill list
            .state('edit_bill',
                __globals.stateConfig('/edit/:billUid/bill', 'bill/edit', {
                    parent: 'base',
                    access: {
                        authority: 'manage.bill.read.edit_support_data'
                    },
                    controller: 'BillEditController as BillEditCtrl',
                    resolve: {
                        BillEditData: ['BillDataService', '$stateParams', function (BillDataService, $stateParams) {
                            return BillDataService.prepareBillEditData($stateParams.billUid);
                        }]
                    }
                }))

            // Bill list
            .state('generate_package_bill',
                __globals.stateConfig('/generate/:userPackageId/bill', 'bill/generate-user-package-bill', {
                    parent: 'base',
                    access: {
                        authority: 'manage.user_purchses.read.user_purchase'
                    },
                    controller: 'GenerateUserPackageBillController as BillAddCtrl',
                    resolve: {
                        GetUserPurchases: ['PackageDataService', '$stateParams', function (PackageDataService, $stateParams) {
                            return PackageDataService.getUserPurchase($stateParams.userPackageId, 'new');
                        }]
                    }
                }))

            // Renewal of bill
            .state('renew_package_bill',
                __globals.stateConfig('/renew/:userPackageId/bill', 'bill/generate-user-package-bill', {
                    parent: 'base',
                    access: {
                        authority: 'manage.user_purchses.read.user_purchase'
                    },
                    controller: 'GenerateUserPackageBillController as BillAddCtrl',
                    resolve: {
                        GetUserPurchases: ['PackageDataService', '$stateParams', function (PackageDataService, $stateParams) {
                            return PackageDataService.getUserPurchase($stateParams.userPackageId, 'renew');
                        }]
                    }
                }))

            // Details
            .state('bill_details',
                __globals.stateConfig('/bill/{billUid}/details', 'bill/details', {
                    parent: 'base',
                    access: {
                        authority: 'manage.bill.read.details'
                    },
                    controller: 'BillDetailsController as billDetailsCtrl',
                    resolve: {
                        GetDetails: ['BillDataService', '$stateParams', function (BillDataService, $stateParams) {
                            return BillDataService.getDetails($stateParams.billUid);
                        }]
                    }
                }))



            // Transaction list
            .state('transactions',
                __globals.stateConfig('/transactions', 'transaction/list', {
                    parent: 'base',
                    access: {
                        authority: 'manage.transaction.read.list'
                    },
                    controller: 'TransactionListController as transactionListCtrl'
                }))

            // configuration general
            .state('configuration_general',
                __globals.stateConfig('/general', 'configuration.general', {
                        parent: 'base',
                        controller: 'GeneralController as generalCtrl',
                        access: {
                            authority: 'manage.configuration.process'
                        },
                        resolve: {
                            getGeneralData: ["ConfigurationDataService", function (ConfigurationDataService) {

                                return ConfigurationDataService.readConfigurationData(1) // general form
                                    .then(function (response) {
                                        return response;
                                    });
                            }]
                        }

                    }
                )
            )

            // configuration currency
            .state('configuration_currency',
                __globals.stateConfig('/currency', 'configuration.currency', {
                        parent: 'base',
                        controller: 'CurrencyConfigurationController as currencyConfigurationCtrl',
                        access: {
                            authority: 'manage.configuration.process'
                        },
                        resolve: {
                            getCurrencyData: ["ConfigurationDataService", function (ConfigurationDataService) {

                                return ConfigurationDataService
                                    .readConfigurationData(2) // Currency form
                                    .then(function (response) {
                                        return response;
                                    });
                            }]
                        }
                    }
                )
            )

        ;
    };

})();
;
(function () {
    'use strict';

    /*
	 ManageController
	-------------------------------------------------------------------------- */

    angular
        .module('ManageApp.master', [])
        .controller('ManageController', [
            '$rootScope',
            '__DataStore',
            '$scope',
            '__Auth',
            'appServices',
            'appNotify',
            '__Form',
            '$state',
            'appToastNotify',
            'ConfigurationDataService',
            'BaseDataService',
            ManageController
        ]).controller('HelpController', [
        '$rootScope',
        '$scope',
        HelpController
    ]);

    /**
     * ManageController for manage page application
     *
     * @inject $rootScope
     * @inject __DataStore
     * @inject $scope
     * @inject __Auth
     * @inject appServices
     * @inject appNotify
     *
     * @return void
     *-------------------------------------------------------- */

    function ManageController($rootScope, __DataStore, $scope, __Auth, appServices, appNotify, __Form, $state, appToastNotify, ConfigurationDataService, BaseDataService) {

        var scope = this;


        scope.pageStatus = false;

        scope.refreshAuthObj = function () {

            __Auth.refresh(function (authInfo) {

                scope.auth_info = authInfo;
            });

        };
        scope.refreshAuthObj();

        scope.notify = __globals.getAppImmutables('notifyToAdmin');
        scope.restrict_user_email_update = __globals.getAppImmutables('restrict_user_email_update');

        scope.unhandledError = function () {

            appNotify.error(__globals.getReactionMessage(19)); // Unhanded errors

        };

        $rootScope.isAdmin = function () {
            return scope.auth_info.designation === 1;
        };

        $rootScope.$on('auth_info_updated', function (event, args) {
            $rootScope.auth_info = args.auth_info;
            if (!_.isEmpty(args.userFullName)) {
                scope.userUpdateData = args.userFullName;
            }
            scope.auth_info = $rootScope.auth_info;

        });

        $rootScope.$on('lw.events.logged_in_user', function () {
            scope.refreshAuthObj();
        });

        $rootScope.$on('lw.events.state.change_start', function () {
            appServices.closeAllDialog();
        });

        $rootScope.$on('lw.datastore.event.post.started', __globals.showButtonLoader);

        $rootScope.$on('lw.datastore.event.fetch.started', __globals.showFormLoader);

        $rootScope.$on('lw.form.event.process.started');

        $rootScope.$on('lw.form.event.fetch.started', __globals.showFormLoader);

        $rootScope.$on('lw.datastore.event.fetch.finished', __globals.hideFormLoader);

        $rootScope.$on('lw.datastore.event.post.finished', __globals.hideButtonLoader);

        $rootScope.$on('lw.form.event.process.finished', __globals.hideButtonLoader);

        $rootScope.$on('lw.datastore.event.fetch.error', scope.unhandledError);

        $rootScope.$on('lw.form.event.process.error', scope.unhandledError);

        $rootScope.$on('$stateChangeSuccess', function ($stateEvent, $stateInfo) {

            var scrollOffsets = __globals.getScrollOffsets(),
                yOffset = Math.round(scrollOffsets.y);
            // document.body.scrollTop = document.documentElement.scrollTop = 0;
            $('html, body').animate({scrollTop: 0}, yOffset < 500 ? 500 : yOffset);
        });

        // Dialog Opened Event
        $rootScope.$on('ngDialog.opened', function (e, $dialog) {
            _.defer(function () {
                $('.ngdialog').scrollTop(0);
            });
        });

        scope.showUploadManagerDialog = function () {
            appServices.showDialog(scope, {
                    templateUrl: __globals.getTemplateURL('upload-manager.upload-manager-dialog')
                },
                function (promiseObj) {

                });
        };

        /**
         * Check if user logged in
         *
         * @return boolean
         *---------------------------------------------------------------- */

        scope.isLoggedIn = function () {
            return __Auth.isLoggedIn();   // isLoggedIn
        };

        /**
         * Check if user logged in
         *
         * @return boolean
         *---------------------------------------------------------------- */

        scope.logoutUser = function () {

            __Auth.registerIntended("dashboard");

            __DataStore.post('user.logout')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        BaseDataService.getBaseData('account_logged').then(function (data) {

                            __globals.setCookie('auth_access_token', '');

                            __Auth.checkOut({}, function (authInfo) {
                                $state.go('login');
                            });

                        });

                    });

                });
        };

        /**
         * Open help dialog
         *
         * @return void
         *---------------------------------------------------------------- */

        scope.openHelpDailog = function (templateUrl, templateTitle) {

            scope.templateTitle = templateTitle;

            appServices.showDialog(scope,
                {
                    templateUrl: __globals.getTemplateURL(templateUrl),

                    controller: 'HelpController as helpCtrl'
                },
                function (promiseObj) {

                });

        };

        /*  scope.showGeneralSetting = function() {

            ConfigurationDataService
                .readConfigurationData(1)
                .then(function(responseData) {

                    var logo_background_color = responseData.data.configuration.logo_background_color;

                    appServices.showDialog({
                        'responseData' : responseData
                    }, {
                        templateUrl : __globals.getTemplateURL('configuration.general')
                    }, function(promiseObj) {

                        $('#lwchangeBgHeaderColor').css('background', "#"+logo_background_color);
                    });
                });
        };

        scope.showCurrencySetting = function() {

            ConfigurationDataService
                .readConfigurationData(2)
                .then(function(responseData) {

                    var logo_background_color = responseData.data.configuration.logo_background_color;

                    appServices.showDialog({
                        'responseData' : responseData
                    }, {
                        templateUrl : __globals.getTemplateURL('configuration.currency')
                    }, function(promiseObj) {

                    });
                });
        };*/

        $rootScope.$on('lw-open-login-dialog', function (event, response) {


            event.preventDefault();

            appServices.loginRequiredDialog('login-dialog', response.data, function (result, newData) {

                __DataStore.reset();

                if (result) {
                    scope.refreshAuthObj();
                    $state.reload();
                }
            });

        });
    };

    /**
     * HelpController for helping information
     *
     * @inject $rootScope
     * @inject $scope
     *
     * @return void
     *-------------------------------------------------------- */

    function HelpController($rootScope, $scope) {

        var scope = this;

        if (_.has($scope.ngDialogData, 'templateTitle')) {
            scope.templateTitle = $scope.ngDialogData.templateTitle;
        }

        /**
         * Close dialog
         *
         * @return void
         *---------------------------------------------------------------- */

        scope.closeDialog = function () {
            $scope.closeThisDialog();
        };

    };

})();
;
/*!
*  Component  : Manage Users
*  File       : ManageUserDataService.js
*  Engine     : ManageUserDataService
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.ManageUserDataService', [])

        /**
         Manage User Data Service
         ---------------------------------------------------------------- */
        .service('ManageUserDataService', [
            '$q',
            '__DataStore',
            '__Form',
            'appServices',
            ManageUserDataService
        ]);

    function ManageUserDataService($q, __DataStore, __Form, appServices) {

        /*
            Get User Info
            -----------------------------------------------------------------*/

        this.getMemberInfo = function (phone) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('https://batmobile.dietbox.pk/catcher/get-member?token=walkonwater&name=test&phone=' + phone
            ).success(function (responseData) {
                defferedObject.resolve(responseData);
            });
            //return promise to caller
            return defferedObject.promise;
        };

        this.createMember = function (userData) {

            //create a differed object
            var defferedObject = $q.defer();
            __DataStore.fetch('https://batmobile.dietbox.pk/catcher/get-member?token=walkonwater&name=' + userData.name +
                '&email=' + userData.email +
                '&createme=1' +
                '&phone=' + userData.phone
            ).success(function (responseData) {
                defferedObject.resolve(responseData);
            });
            //return promise to caller
            return defferedObject.promise;
        };


        /*
            Get User Info
            -----------------------------------------------------------------*/
        this.getUserInfo = function (userId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user.read.info',
                'userId': userId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get User Add Support Data
            -----------------------------------------------------------------*/
        this.getUserAddSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.user.read.create.support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get User Add Support Data
            -----------------------------------------------------------------*/
        this.getCustomerAddSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.customer.read.create.support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get User User Permissions
            -----------------------------------------------------------------*/
        this.getUserPermissions = function (userId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user.read.get_user_permissions',
                'userId': userId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get User User Permissions
            -----------------------------------------------------------------*/
        this.getUserDetailData = function (userId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user.read.detail.data',
                'userID': userId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Customer Detail Data
            -----------------------------------------------------------------*/
        this.getCustomerDetailData = function (memberId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.customer.read.detail.data',
                'memberId': memberId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get User Edit Data
            -----------------------------------------------------------------*/
        this.getUserEditData = function (userId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user.read.edit_suppport_data',
                'userId': userId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };
    }
    ;

})(window, window.angular);
;
/*!
*  Component  : Manage User
*  File       : ManageUserEngine.js
*  Engine     : ManageUserEngine
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.users', [])

        /**
         * Manage Users Controller
         *
         * @inject $scope
         * @inject __DataStore
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('ManageCustomersController', [
            '$scope',
            '__DataStore',
            'appServices',
            'ManageUserDataService',
            'ConfigurationDataService',
            function ManageCustomersController($scope, __DataStore, appServices, ManageUserDataService, ConfigurationDataService) {

                var dtUsersColumnsData = [
                        {
                            "name": null,
                            "template": "#profileImageColumnTemplate"
                        },
                        {
                            "name": "memberName",
                            "template": "#userNameColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": "memberPhone",
                            "orderable": false
                        },
                        {
                            "name": "updated_at",
                            "template": "#userUpdatedDateColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": null,
                            "template": "#userActionColumnTemplate"
                        }
                    ],
                    dtDeletedUsersColumnsData = [
                        {
                            "name": null,
                            "template": "#profileImageColumnTemplate"
                        },
                        {
                            "name": "memberName",
                            "template": "#userNameColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": "memberPhone",
                            "orderable": true
                        },
                        {
                            "name": "updated_at",
                            "template": "#userUpdatedDateColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": null,
                            "template": "#userActionColumnTemplate"
                        }
                    ],
                    tabs = {
                        'members': {
                            id: 'activeUsersTabList',
                            status: 1 // Active
                        },
                        'activePackages': {
                            id: 'inactiveUsersTabList',
                            status: 2 // Inactive
                        },
                        'expirePackages': {
                            id: 'deletedUsersTabList',
                            status: 5 // Soft Deleted
                        }
                    },
                    currentStatus = 1,
                    scope = this;

                // Manage users tab action
                // When clicking on tab, its related tab data load on same page

                $('#manageCustomersTabs a').click(function (e) {

                    e.preventDefault();

                    var $this = $(this),
                        tabName = $this.attr('aria-controls'),
                        selectedTab = tabs[tabName];

                    // Check if selected tab exist
                    if (!_.isEmpty(selectedTab)) {

                        $(this).tab('show')

                        currentStatus = selectedTab.status;
                        scope.getUsers(selectedTab.id, selectedTab.status);

                    }

                });

                _.defer(function (text) {
                    var selectedTab = $('.nav li a[href="#members"]');
                    selectedTab.triggerHandler('click', true)

                }, 0);

                /**
                 * Get users as a datatable source
                 *
                 * @param string tableID
                 * @param number status
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.getUsers = function (tableID, status) {
                    // destroy if existing instatnce available
                    if (scope.usersListDataTable) {
                        scope.usersListDataTable.destroy();
                    }

                    scope.usersListDataTable = __DataStore.dataTable('#' + tableID, {
                        url: {
                            'apiURL': 'manage.customer.read.datatable.list',
                            'status': status
                        },
                        dtOptions: {
                            "searching": true,
                            "pageLength": 25,
                            "order": [[1, "asc"]]
                        },
                        columnsData: status == 5 ? dtDeletedUsersColumnsData : dtUsersColumnsData,
                        scope: $scope

                    });

                };

                /*
                  Reload current datatable
                  ------------------------------------------------------------------- */

                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.usersListDataTable);
                };

                /**
                 * Delete user
                 *
                 * @param number userID
                 * @param string userName
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.delete = function (userID, userName, deleteType) {

                    scope.deletingUserName = unescape(userName);

                    _.defer(function () {

                        var $lwUserDeleteConfirmTextMsg = $('#lwUserDeleteConfirmTextMsg');

                        var $lwUserPerDeleteConfirmTextMsg = $('#lwUserPerDeleteConfirmTextMsg');

                        if (deleteType == 1) { // Soft delete
                            scope.deleteText = $lwUserDeleteConfirmTextMsg.attr('data-message');
                            scope.deleteConfirmBtnText = $lwUserDeleteConfirmTextMsg.attr('data-delete-button-text');
                            scope.successMsgText = $lwUserDeleteConfirmTextMsg.attr('success-msg');
                        } else { // Permanent delete
                            scope.deleteText = $lwUserPerDeleteConfirmTextMsg.attr('data-message');
                            scope.deleteConfirmBtnText = $lwUserPerDeleteConfirmTextMsg.attr('data-delete-button-text');
                            scope.successMsgText = $lwUserPerDeleteConfirmTextMsg.attr('success-msg');
                        }

                    });

                    _.defer(function () {

                        __globals.showConfirmation({
                                html: scope.deleteText,
                                confirmButtonText: scope.deleteConfirmBtnText
                            },
                            function () {

                                __DataStore.post({
                                    'apiURL': 'manage.customer.write.delete',
                                    'userID': userID,
                                })
                                    .success(function (responseData) {

                                        var message = responseData.data.message;

                                        appServices.processResponse(responseData, {

                                                error: function () {

                                                    __globals.showConfirmation({
                                                        title: 'Deleted!',
                                                        text: message,
                                                        type: 'error'
                                                    });

                                                }
                                            },
                                            function () {

                                                __globals.showConfirmation({
                                                    title: 'Deleted!',
                                                    text: scope.successMsgText,
                                                    type: 'success'
                                                });
                                                scope.reloadDT();   // reload datatable

                                            }
                                        );

                                    })

                            })

                    });

                };

                /**
                 * Restore deleted user
                 *
                 * @param number userID
                 * @param string userName
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.restore = function (userID, userName) {

                    scope.restoringUserName = unescape(userName);

                    _.defer(function () {

                        var $lwUserRestoreConfirmTextMsg = $('#lwUserRestoreConfirmTextMsg');

                        __globals.showConfirmation({
                                text: $lwUserRestoreConfirmTextMsg.attr('data-message'),
                                confirmButtonText: $lwUserRestoreConfirmTextMsg.attr('data-restore-button-text')
                            },
                            function () {

                                __DataStore.post({
                                    'apiURL': 'manage.customer.write.restore',
                                    'userID': userID,
                                })
                                    .success(function (responseData) {

                                        var message = responseData.data.message;

                                        appServices.processResponse(responseData, {

                                                error: function () {
                                                    __globals.showConfirmation({
                                                        title: 'Restore!',
                                                        text: message,
                                                        type: 'error'
                                                    });
                                                }
                                            },
                                            function () {

                                                __globals.showConfirmation({
                                                    title: 'Restore!',
                                                    text: message,
                                                    type: 'success'
                                                });
                                                scope.reloadDT();   // reload datatable

                                            })

                                    })

                            })

                    });

                };

                /**
                 * Change password of user by Admin
                 *
                 * @param number userID
                 * @param number name
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.changePassword = function (userID, name) {

                    // open change password dialog
                    appServices.showDialog({
                            userID: userID,
                            name: unescape(name)
                        },
                        {
                            templateUrl: __globals.getTemplateURL('user.manage.change-password'),
                            controller: 'ManageUserChangePasswordController as userChangePassword'
                        },
                        function (promiseObj) {

                        });
                };

                /**
                 * Assign package
                 *
                 * @param number userID
                 * @param number name
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.assignPackage = function (userUid, name) {

                    // open change password dialog
                    appServices.showDialog({
                            userID: userUid,
                            name: unescape(name)
                        },
                        {
                            templateUrl: __globals.getTemplateURL('user.customer.assign-package-dialog'),
                            controller: 'AssignPackageController as AssignPackageCtrl',
                            resolve: {
                                AssignPackageData: ['PackageDataService', function (PackageDataService) {
                                    return PackageDataService.getAssignPackageSupportData();
                                }]
                            }
                        },
                        function (promiseObj) {

                        });
                };
                /**
                 * Show add new user dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.showAddNewDialog = function () {

                    appServices.showDialog(
                        {
                            'showRoleSelectBox': true
                        },
                        {
                            templateUrl: __globals.getTemplateURL('user.customer.add-dialog'),
                        },
                        function (promiseObj) {

                            // Check if category updated
                            if (_.has(promiseObj.value, 'user_added')
                                && promiseObj.value.user_added == true && currentStatus == 1) {
                                scope.reloadDT();
                            }

                        });
                };

                /**
                 * Edit User Dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.editUserDialog = function (userId, name) {
                    appServices.showDialog(
                        {
                            'userId': userId,
                            'name': name
                        },
                        {
                            templateUrl: 'user.customer.edit-dialog',
                            controller: 'EditCustomerDialogController as EditCustomerDialogCtrl',
                            resolve: {
                                EditUserData: function () {
                                    return ManageUserDataService
                                        .getUserEditData(userId);
                                }
                            }
                        },
                        function (promiseObj) {

                            // Check if category updated
                            if (_.has(promiseObj.value, 'user_updated')
                                && promiseObj.value.user_updated == true) {
                                scope.reloadDT();
                            }

                        });
                };

                /**
                 * Show User Permission Dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                /*scope.openUserDetailsDialog = function(userId) {

                    appServices.showDialog({
                    	userId : userId
                    }, {
                        templateUrl : __globals.getTemplateURL('user.customer.user-detail-dialog'),
                        controller : 'ManageCustomerDetailController as ManageCustomerDetailCtrl',
                    }, function(promiseObj) {

                    });
                };*/

            }
        ])

        /**
         * Customer Detail Dialog Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ManageCustomerDetailController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$rootScope',
            '$stateParams',
            'appServices',
            'ManageUserDataService',
            function ($scope, __DataStore, __Form, $rootScope, $stateParams, appServices, ManageUserDataService) {

                var scope = this;
                scope.memberId = $stateParams.memberId;

                scope.fetchUserDetails = function () {
                    ManageUserDataService.getCustomerDetailData(scope.memberId).then(function (requestData) {
                        scope.userData = requestData.userData;
                        scope.assignedPackages = requestData.assignedPackages;
                    })
                }

                scope.dateTimePickerOptions = {
                    time: false,
                    okText: 'Select',
                    clearButton: true
                };

                scope.fetchUserDetails();

                $rootScope.$on('slot_changed', function (data) {
                    scope.fetchUserDetails();
                });

                //update package price
                scope.updatePackagePrice = function (price, userPackageId) {
                    __DataStore.post('manage.package.write.update_user_package', {
                        'price': price,
                        'userPackageId': userPackageId,
                        'requestType': 1
                    }).success(function (responseData) {
                        appServices.processResponse(responseData, null, function (reactionCode) {
                        });
                    });
                }

                //update package expiry
                scope.updatePackageExpiry = function (expiry_date, userPackageId) {
                    __DataStore.post('manage.package.write.update_user_package', {
                        'expiry_date': expiry_date,
                        'userPackageId': userPackageId,
                        'requestType': 2
                    }).success(function (responseData) {
                        appServices.processResponse(responseData, null, function (reactionCode) {
                        });
                    });
                }

                //delete user slot
                scope.deleteUserSlot = function (itemSlotUid, itemSlotTitle) {

                    scope.deletingItemSlotTitle = '';
                    scope.deletingItemSlotTitle = itemSlotTitle;

                    _.defer(function () {

                        var lwuserSlotremoveConfirmTextMsg = $('#lwuserSlotremoveConfirmTextMsg');

                        __globals.showConfirmation({
                                html: lwuserSlotremoveConfirmTextMsg.attr('data-message'),
                                confirmButtonText: lwuserSlotremoveConfirmTextMsg.attr('data-delete-button-text')
                            },
                            function () {

                                __DataStore.post('manage.slot.write.delete_item_slot', {
                                    'itemSlotUid': itemSlotUid
                                }).success(function (responseData) {
                                    appServices.processResponse(responseData, null, function (reactionCode) {
                                        if (responseData.reaction == 1) {
                                            scope.fetchUserDetails()
                                        }
                                    });
                                });

                            });

                    });
                }

                /**
                 * Chnage user slot
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.changeSlot = function (slotData, packageOrService, packageName) {

                    appServices.showDialog({
                        slotData: slotData,
                        requestType: packageOrService,
                        packageName: packageName,
                        userId: scope.userId
                    }, {
                        templateUrl: __globals.getTemplateURL('user.customer.change-slot-dialog'),
                        controller: 'ChangeSlotController as ChangeSlotCtrl',
                    }, function (promiseObj) {

                    });

                }

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

        /**
         * Add Customer Dialog Controller handle add new user dialog scope
         *
         * @inject $scope
         * @inject __Form
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */

        .controller('AddCustomerDialogController', [
            '$scope',
            '__Form',
            'appServices',
            'ManageUserDataService',
            function ($scope, __Form, appServices, ManageUserDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'add_customer_form', 'userData', {secured: true});
                scope.userData.role = 3;
                scope.countrySelectConfig = __globals.getSelectizeOptions();
                // Get User add Support Data
                ManageUserDataService
                    .getCustomerAddSupportData()
                    .then(function (responseData) {
                        var requestData = responseData.data;
                        scope.countries = requestData.countries;
                    });

                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process('manage.customer.write.create', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                // close dialog
                                $scope.closeThisDialog({
                                    user_added: true,
                                    'user_data': responseData.data.userData
                                });

                            });

                        });

                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])

        /**
         * Edit Customer Dialog Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('EditCustomerDialogController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'appServices',
            'EditUserData',
            function ($scope, __DataStore, __Form, $stateParams, appServices, EditUserData) {

                var scope = this,
                    requestData = EditUserData,
                    ngDialogData = $scope.ngDialogData;

                scope.countries = requestData.countries;
                scope.countrySelectConfig = __globals.getSelectizeOptions();

                scope = __Form.setup(scope, 'user_edit_form', 'userData');
                scope = __Form.updateModel(scope, requestData.userUpdateData);

                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.customer.write.update_process',
                        'userId': ngDialogData.userId
                    }, scope)
                        .success(function (responseData) {
                            appServices.processResponse(responseData, null, function () {
                                // close dialog
                                $scope.closeThisDialog({'user_updated': true});
                            });
                        });
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

        /**
         * Manage Users Controller
         *
         * @inject $scope
         * @inject __DataStore
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('ManageUsersController', [
            '$scope',
            '__DataStore',
            'appServices',
            'ManageUserDataService',
            'ConfigurationDataService',
            function ManageUsersController($scope, __DataStore, appServices, ManageUserDataService, ConfigurationDataService) {

                var dtUsersColumnsData = [
                        {
                            "name": null,
                            "template": "#profileImageColumnTemplate"
                        },
                        {
                            "name": "name",
                            "template": "#userNameColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": "username",
                            "orderable": true
                        },
                        {
                            "name": "email",
                            "orderable": true
                        },
                        {
                            "name": "updated_at",
                            "template": "#userUpdatedDateColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": "user_role",
                            "orderable": true
                        },
                        {
                            "name": null,
                            "template": "#userActionColumnTemplate"
                        }
                    ],
                    dtDeletedUsersColumnsData = [
                        {
                            "name": null,
                            "template": "#profileImageColumnTemplate"
                        },
                        {
                            "name": "name",
                            "template": "#userNameColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": "username",
                            "orderable": true
                        },
                        {
                            "name": "email",
                            "orderable": true
                        },
                        {
                            "name": "updated_at",
                            "template": "#userUpdatedDateColumnTemplate",
                            "orderable": true
                        },
                        {
                            "name": "user_role",
                            "orderable": true
                        },
                        {
                            "name": null,
                            "template": "#userActionColumnTemplate"
                        }
                    ],
                    tabs = {
                        'active': {
                            id: 'activeUsersTabList',
                            status: 1 // Active
                        },
                        'inactive': {
                            id: 'inactiveUsersTabList',
                            status: 2 // Inactive
                        },
                        'deleted': {
                            id: 'deletedUsersTabList',
                            status: 5 // Soft Deleted
                        }
                    },
                    currentStatus = 1,
                    scope = this;


                // Manage users tab action
                // When clicking on tab, its related tab data load on same page

                $('#manageUsersTabs a').click(function (e) {

                    e.preventDefault();

                    var $this = $(this),
                        tabName = $this.attr('aria-controls'),
                        selectedTab = tabs[tabName];

                    // Check if selected tab exist
                    if (!_.isEmpty(selectedTab)) {

                        $(this).tab('show')

                        currentStatus = selectedTab.status;
                        scope.getUsers(selectedTab.id, selectedTab.status);

                    }

                });

                /**
                 * Get users as a datatable source
                 *
                 * @param string tableID
                 * @param number status
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.getUsers = function (tableID, status) {

                    // destroy if existing instatnce available
                    if (scope.usersListDataTable) {
                        scope.usersListDataTable.destroy();
                    }

                    scope.usersListDataTable = __DataStore.dataTable('#' + tableID, {
                        url: {
                            'apiURL': 'manage.user.read.datatable.list',
                            'status': status
                        },
                        dtOptions: {
                            "searching": true,
                            "pageLength": 25,
                            "order": [[1, "asc"]]
                        },
                        columnsData: status == 5 ? dtDeletedUsersColumnsData : dtUsersColumnsData,
                        scope: $scope

                    });

                };

                _.defer(function (text) {
                    var selectedTab = $('.nav li a[href="#active"]');
                    selectedTab.triggerHandler('click', true)

                }, 0);

                /*
                  Reload current datatable
                  ------------------------------------------------------------------- */

                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.usersListDataTable);
                };

                /**
                 * Delete user
                 *
                 * @param number userID
                 * @param string userName
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.delete = function (userID, userName, deleteType) {

                    scope.deletingUserName = unescape(userName);

                    _.defer(function () {

                        var $lwUserDeleteConfirmTextMsg = $('#lwUserDeleteConfirmTextMsg');

                        var $lwUserPerDeleteConfirmTextMsg = $('#lwUserPerDeleteConfirmTextMsg');

                        if (deleteType == 1) { // Soft delete
                            scope.deleteText = $lwUserDeleteConfirmTextMsg.attr('data-message');
                            scope.deleteConfirmBtnText = $lwUserDeleteConfirmTextMsg.attr('data-delete-button-text');
                            scope.successMsgText = $lwUserDeleteConfirmTextMsg.attr('success-msg');
                        } else { // Permanent delete
                            scope.deleteText = $lwUserPerDeleteConfirmTextMsg.attr('data-message');
                            scope.deleteConfirmBtnText = $lwUserPerDeleteConfirmTextMsg.attr('data-delete-button-text');
                            scope.successMsgText = $lwUserPerDeleteConfirmTextMsg.attr('success-msg');
                        }

                    });

                    _.defer(function () {

                        __globals.showConfirmation({
                                html: scope.deleteText,
                                confirmButtonText: scope.deleteConfirmBtnText
                            },
                            function () {

                                __DataStore.post({
                                    'apiURL': 'manage.user.write.delete',
                                    'userID': userID,
                                })
                                    .success(function (responseData) {

                                        var message = responseData.data.message;

                                        appServices.processResponse(responseData, {

                                                error: function () {

                                                    __globals.showConfirmation({
                                                        title: 'Deleted!',
                                                        text: message,
                                                        type: 'error'
                                                    });

                                                }
                                            },
                                            function () {

                                                __globals.showConfirmation({
                                                    title: 'Deleted!',
                                                    text: scope.successMsgText,
                                                    type: 'success'
                                                });
                                                scope.reloadDT();   // reload datatable

                                            }
                                        );

                                    })

                            })

                    });

                };

                /**
                 * Restore deleted user
                 *
                 * @param number userID
                 * @param string userName
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.restore = function (userID, userName) {

                    scope.restoringUserName = unescape(userName);

                    _.defer(function () {

                        var $lwUserRestoreConfirmTextMsg = $('#lwUserRestoreConfirmTextMsg');

                        __globals.showConfirmation({
                                text: $lwUserRestoreConfirmTextMsg.attr('data-message'),
                                confirmButtonText: $lwUserRestoreConfirmTextMsg.attr('data-restore-button-text')
                            },
                            function () {

                                __DataStore.post({
                                    'apiURL': 'manage.user.write.restore',
                                    'userID': userID,
                                })
                                    .success(function (responseData) {

                                        var message = responseData.data.message;

                                        appServices.processResponse(responseData, {

                                                error: function () {
                                                    __globals.showConfirmation({
                                                        title: 'Restore!',
                                                        text: message,
                                                        type: 'error'
                                                    });
                                                }
                                            },
                                            function () {

                                                __globals.showConfirmation({
                                                    title: 'Restore!',
                                                    text: message,
                                                    type: 'success'
                                                });
                                                scope.reloadDT();   // reload datatable

                                            })

                                    })

                            })

                    });

                };

                /**
                 * Change password of user by Admin
                 *
                 * @param number userID
                 * @param number name
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.changePassword = function (userID, name) {

                    // open change password dialog
                    appServices.showDialog({
                            userID: userID,
                            name: unescape(name)
                        },
                        {
                            templateUrl: __globals.getTemplateURL('user.manage.change-password'),
                            controller: 'ManageUserChangePasswordController as userChangePassword'
                        },
                        function (promiseObj) {

                        });
                };

                /**
                 * Show add new user dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.showAddNewDialog = function () {

                    appServices.showDialog(
                        {
                            'showRoleSelectBox': true
                        },
                        {
                            templateUrl: __globals.getTemplateURL('user.manage.add-dialog'),
                        },
                        function (promiseObj) {

                            // Check if category updated
                            if (_.has(promiseObj.value, 'user_added')
                                && promiseObj.value.user_added == true && currentStatus == 1) {
                                scope.reloadDT();
                            }

                        });
                };

                /**
                 * Edit User Dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.editUserDialog = function (userId, name) {
                    appServices.showDialog(
                        {
                            'userId': userId,
                            'name': name
                        },
                        {
                            templateUrl: 'user.manage.edit-dialog',
                            controller: 'EditUserDialogController as EditUserDialogCtrl',
                            resolve: {
                                EditUserData: function () {
                                    return ManageUserDataService
                                        .getUserEditData(userId);
                                }
                            }
                        },
                        function (promiseObj) {

                            // Check if category updated
                            if (_.has(promiseObj.value, 'user_updated')
                                && promiseObj.value.user_updated == true) {
                                scope.reloadDT();
                            }

                        });
                };

                scope.showUsersConfigurationDialog = function () {

                    ConfigurationDataService
                        .readConfigurationData(5)
                        .then(function (responseData) {

                            appServices.showDialog({
                                'responseData': responseData
                            }, {
                                templateUrl: __globals.getTemplateURL('configuration.users')
                            }, function (promiseObj) {

                            });

                        });

                };

                /**
                 * Show User Permission Dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.usersPermissionDialog = function (userId, fullName) {

                    appServices.showDialog({
                        'userId': userId,
                        'fullName': _.unescape(fullName)
                    }, {
                        templateUrl: __globals.getTemplateURL('user.manage.user-dynamic-permission'),
                        controller: 'ManageUsersDynamicPermissionController as manageUsersDynamicPermissionCtrl',
                        resolve: {
                            UserPermission: function () {
                                return ManageUserDataService.getUserPermissions(userId);
                            }
                        }
                    }, function (promiseObj) {

                    });
                };

                /**
                 * Show User Permission Dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.openUserDetailsDialog = function (userId) {

                    appServices.showDialog({}, {
                        templateUrl: __globals.getTemplateURL('user.manage.user-detail-dialog'),
                        controller: 'ManageUsersDetailController as manageUsersDetailCtrl',
                        resolve: {
                            UserDetailData: function () {
                                return ManageUserDataService.getUserDetailData(userId);
                            }
                        }
                    }, function (promiseObj) {

                    });
                };

            }
        ])

        /**
         * User Detail Dialog Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ManageUsersDetailController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'appServices',
            'UserDetailData',
            function ($scope, __DataStore, __Form, $stateParams, appServices, UserDetailData) {

                var scope = this,
                    requestData = UserDetailData;

                scope.userData = requestData.userData;


                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

        /**
         * Add User Dialog Controller handle add new user dialog scope
         *
         * @inject $scope
         * @inject __Form
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */

        .controller('AddUserDialogController', [
            '$scope',
            '__Form',
            'appServices',
            'ManageUserDataService',
            function ($scope, __Form, appServices, ManageUserDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'add_user_form', 'userData', {secured: true});

                scope.showRoleSelectBox = $scope.ngDialogData.showRoleSelectBox;
                scope.countrySelectConfig = __globals.getSelectizeOptions();

                // Get User add Support Data
                ManageUserDataService
                    .getUserAddSupportData()
                    .then(function (responseData) {
                        var requestData = responseData.data;
                        scope.userRoles = requestData.userRoles;
                        scope.countries = requestData.countries;
                    });
                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process('manage.user.write.create', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                // close dialog
                                $scope.closeThisDialog({
                                    user_added: true,
                                    'user_data': responseData.data.userData
                                });

                            });

                        });

                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])

        /**
         * ManageUserChangePasswordController handle change password by admin
         *
         * @inject $scope
         * @inject __Form
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('ManageUserChangePasswordController', [
            '$scope',
            '__Form',
            'appServices',
            function ManageUserChangePasswordController($scope, __Form, appServices) {

                var scope = this;

                scope = __Form.setup(scope, 'change_password_form', 'changePasswordData', {
                    secured: true
                });

                scope.ngDialogData = $scope.ngDialogData;

                scope.title = unescape(scope.ngDialogData.name);

                // get id of user
                scope.userID = scope.ngDialogData.userID;


                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.user.write.change_password.process',
                        'userID': scope.userID
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                // close dialog
                                $scope.closeThisDialog();

                            });

                        });

                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

        /**
         * Manage User Details Dialog for manage product list
         *
         * @inject $scope
         * @inject __Form
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('ManageUserDetailsDialog', [
            '$scope',
            function ManageUserDetailsDialog($scope) {

                var scope = this;

                scope.ngDialogData = $scope.ngDialogData;
                scope.userDetails = scope.ngDialogData;

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])/*.filter('searchFilterList', function() {

		  	return function(items, searchText) {__pr(items);

				var filtered = [];

			    angular.forEach(items, function(el, index) {
					var oldObj = index;
					var obj = [];

			      	if (index && index.indexOf(searchText)>-1) {
			        	obj.push(el);
			      	}

					filtered.push(obj);

			    });
				__pr(filtered);
			    return filtered;
		  	};
		})
*/
        /**
         * Manage User Dynamic Permissions
         *
         * @inject $scope
         * @inject __Form
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('ManageUsersDynamicPermissionController', [
            '$scope',
            '__Form',
            '__DataStore',
            'appServices',
            'UserPermission',
            function ManageUsersDynamicPermissionController($scope, __Form, __DataStore, appServices, UserPermission) {

                var scope = this;

                scope = __Form.setup(scope, 'user_dynamic_access', 'accessData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.ngDialogData = $scope.ngDialogData;
                scope.userId = scope.ngDialogData.userId;
                scope.fullName = scope.ngDialogData.fullName;
                scope.requestData = UserPermission;
                scope.permissions = scope.requestData.permissions;

                scope.accessData.allow_permissions = scope.requestData.allow_permissions;
                scope.accessData.deny_permissions = scope.requestData.deny_permissions;
                scope.accessData.inherit_permissions = scope.requestData.inherit_permissions;

                scope.disablePermissions = function (eachPermission, permissionID) {

                    _.map(eachPermission.children, function (key) {
                        if (_.includes(key.dependencies, permissionID)) {
                            _.delay(function (text) {
                                $('input[name="' + key.id + '"]').attr('disabled', true);
                            }, 500);
                        }
                    });

                }

                scope.checkedPermission = {};

                _.map(scope.accessData.allow_permissions, function (permission) {
                    scope.checkedPermission[permission] = "2";
                });

                _.map(scope.accessData.deny_permissions, function (permission) {
                    scope.checkedPermission[permission] = "3";


                    _.map(scope.permissions, function (eachPermission) {

                        var pluckedIDs = _.pluck(eachPermission.children, 'id');

                        if (_.includes(pluckedIDs, permission)) {
                            scope.disablePermissions(eachPermission, permission)
                        }

                        if (_.has(eachPermission, 'children_permission_group')) {

                            _.map(eachPermission.children_permission_group, function (groupchild) {

                                var pluckedIDs = _.pluck(groupchild.children, 'id');

                                if (_.includes(pluckedIDs, permission)) {
                                    scope.disablePermissions(groupchild, permission)
                                }
                            });
                        }
                    });

                });

                _.map(scope.accessData.inherit_permissions, function (permission) {
                    scope.checkedPermission[permission] = "1";

                    _.map(scope.permissions, function (eachPermission) {

                        var pluckedIDs = _.pluck(eachPermission.children, 'id');

                        if (_.includes(pluckedIDs, permission) && eachPermission.children[0].inheritStatus == false && eachPermission.children[0].result == "1") {
                            scope.disablePermissions(eachPermission, permission);
                        }

                        if (_.has(eachPermission, 'children_permission_group')) {

                            _.map(eachPermission.children_permission_group, function (groupchild) {

                                var pluckedIDs = _.pluck(groupchild.children, 'id');

                                if (_.includes(pluckedIDs, permission) && groupchild.children[0].inheritStatus == false && groupchild.children[0].result == "1") {
                                    scope.disablePermissions(groupchild, permission);
                                }

                            });
                        }
                    });
                });

                /*
                 Prepare Permissions
                -------------------------------------------------------------------------- */
                // scope.preparePermissions = function() {
                //     scope.accessData.allow_permissions = [];
                //     scope.accessData.deny_permissions = [];

                //     if (!_.isEmpty(scope.accessData.selected_permissions)) {
                //         _.forEach(scope.accessData.selected_permissions, function(item) {
                //             var number = item.split("_").pop();

                //             if (number == 2) {
                //                 scope.accessData.allow_permissions.push(_.trimRight(item, '_'+number));
                //             } else if (number == 3) {
                //                 scope.accessData.deny_permissions.push(_.trimRight(item, '_'+number));
                //             }
                //         });
                //     }
                // }

                //for updating permissions
                scope.checkPermission = function (childId, status) {

                    if (!_.isString(status)) {
                        status = status.toString();
                    }

                    scope.checkedPermission[childId] = status;

                    if (status == "2") {
                        if (!_.includes(scope.accessData.allow_permissions, childId)) {
                            scope.accessData.allow_permissions.push(childId);
                        }
                        if (_.includes(scope.accessData.deny_permissions, childId)) {
                            scope.accessData.deny_permissions = _.without(scope.accessData.deny_permissions, childId);
                        }
                    } else if (status == "3") {

                        if (!_.includes(scope.accessData.deny_permissions, childId)) {
                            scope.accessData.deny_permissions.push(childId);
                        }
                        if (_.includes(scope.accessData.allow_permissions, childId)) {
                            scope.accessData.allow_permissions = _.without(scope.accessData.allow_permissions, childId);
                        }
                    } else {
                        if (_.includes(scope.accessData.deny_permissions, childId)) {
                            scope.accessData.deny_permissions = _.without(scope.accessData.deny_permissions, childId);
                        }
                        if (_.includes(scope.accessData.allow_permissions, childId)) {
                            scope.accessData.allow_permissions = _.without(scope.accessData.allow_permissions, childId);
                        }
                    }

                    _.map(scope.permissions, function (permission) {

                        var pluckedIDs = _.pluck(permission.children, 'id'),
                            keyPermissions = [];

                        if (_.includes(pluckedIDs, childId) && permission.children[0].id != childId) {

                            // 		 	_.map(permission.children, function(key) {
                            // 			 	if (key.id == childId && permission.children[0].id != childId) {
                            //                             _.map(key.dependencies, function(dependency) {

                            //                             	if (scope.checkedPermission[dependency] != '1' && scope.checkedPermission[dependency] != "2") {

                            //                             		if (permission.children[0].inheritStatus == true && permission.children[0].result == "1") {
                            //                                      scope.checkedPermission[dependency] = "1";
                            //                                  } else {
                            //                                      scope.checkedPermission[dependency] = "2";
                            //                                  }
                            //                             	}

                            //                             });
                            // 				 	}

                            // if (permission.children[0].id != key.id && !_.isUndefined(scope.checkedPermission[key.id])) {
                            // 	keyPermissions.push(scope.checkedPermission[key.id]);
                            // }
                            // 		});

                            //                     if (!_.includes(keyPermissions, "1") && !_.includes(keyPermissions, "2")) {
                            //                     	scope.checkedPermission[permission.children[0].id] = "3";
                            //                     }

                            // if (_.includes(keyPermissions, "1") || _.includes(keyPermissions, "2")) {
                            //   	scope.checkedPermission[permission.children[0].id] = "3";
                            //   }
                        }

                        if (_.includes(pluckedIDs, childId) && permission.children[0].id == childId) {

                            _.map(permission.children, function (key) {
                                if (_.includes(key.dependencies, childId) && status == "3") {

                                    $('input[name="' + key.id + '"]').attr('disabled', true);

                                } else if (_.includes(key.dependencies, childId) && status == "1" && permission.children[0].result && permission.children[0].inheritStatus == false) {

                                    $('input[name="' + key.id + '"]').attr('disabled', true);

                                } else {
                                    $('input[name="' + key.id + '"]').attr('disabled', false);
                                }

                                // if (_.includes(key.dependencies, childId) && status == "1" && key.inheritStatus == false) {
                                //     $('input[name="'+key.id+'"]').attr('disabled', true);
                                // }

                            });

                        }

                        if (_.has(permission, 'children_permission_group')) {
                            _.map(permission.children_permission_group, function (groupchild) {

                                var pluckedGroupChildIDs = _.pluck(groupchild.children, 'id'),
                                    keyPermissionsGroup = [];

                                if (_.includes(pluckedGroupChildIDs, childId) && groupchild.children[0].id != childId) {
                                    //                           _.map(groupchild.children, function(groupchildkey) {

                                    //                               if (groupchildkey.id == childId && groupchild.children[0].id != childId) {
                                    //                                   _.map(groupchildkey.dependencies, function(dependency) {
                                    //                                   	if (scope.checkedPermission[dependency] != '1' && scope.checkedPermission[dependency] != "2") {
                                    //                                   		if (groupchild.children[0].inheritStatus == true && groupchild.children[0].result == "1") {
                                    //                                            scope.checkedPermission[dependency] = "1";
                                    //                                        } else {
                                    //                                            scope.checkedPermission[dependency] = "2";
                                    //                                        }
                                    //                                   	}
                                    //                                   });
                                    //                               }

                                    //                               if (groupchild.children[0].id != groupchildkey.id && !_.isUndefined(scope.checkedPermission[groupchildkey.id])) {
                                    // 	keyPermissionsGroup.push(scope.checkedPermission[groupchildkey.id]);
                                    // }
                                    //                           });

                                    //                           if (!_.includes(keyPermissionsGroup, "1") && !_.includes(keyPermissionsGroup, "2")) {
                                    //                     	scope.checkedPermission[groupchild.children[0].id] = "3";
                                    //                     }
                                }

                                //for disabling options if read option  in denied
                                if (_.includes(pluckedGroupChildIDs, childId) && groupchild.children[0].id == childId) {

                                    _.map(groupchild.children, function (groupchildkey) {
                                        if (_.includes(groupchildkey.dependencies, childId) && status == "3") {
                                            $('input[name="' + groupchildkey.id + '"]').attr('disabled', true);

                                        } else if (_.includes(groupchildkey.dependencies, childId) && status == "1" && groupchild.children[0].result && groupchild.children[0].inheritStatus == false) {

                                            $('input[name="' + groupchildkey.id + '"]').attr('disabled', true);

                                        } else {
                                            $('input[name="' + groupchildkey.id + '"]').attr('disabled', false);
                                        }


                                    });

                                }
                            })
                        }
                    })
                }

                /*
                 Submit form action
                -------------------------------------------------------------------------- */
                //           scope.filterPermissions = function(match) {

                //         		var treeInstance = $("#userPermissionTree").fancytree("getTree"),
                //         			filteredNodes,
                //           		filteredbranches,
                //        opts = {
                //       	'autoApply' : true,
                // 	'autoExpand' : true,
                // 	'fuzzy' : false,
                // 	'hideExpanders' : true,
                // 	'highlight' : true,
                // 	'leavesOnly' : true,
                // 	'nodata' : 'No results found.',
                // 	'mode' : "hide",
                // 	'counter': true,
                //       };

                // 	// Pass function to perform match
                // filteredNodes = treeInstance.filterNodes(match, opts);
                // filteredbranches = treeInstance.filterBranches(match, opts);
                //             }

                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {
                    // scope.preparePermissions();
                    __Form.process({
                        'apiURL': 'manage.user.write.user_dynamic_permission',
                        'userId': scope.userId
                    }, scope)
                        .success(function (responseData) {
                            appServices.processResponse(responseData, null, function () {
                                // close dialog
                                $scope.closeThisDialog();
                            });
                        });
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])

        /**
         * Edit User Dialog Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('EditUserDialogController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'appServices',
            'EditUserData',
            function ($scope, __DataStore, __Form, $stateParams, appServices, EditUserData) {

                var scope = this,
                    requestData = EditUserData,
                    ngDialogData = $scope.ngDialogData;

                scope.userRoles = requestData.userRoles;
                scope.countries = requestData.countries;
                scope.countrySelectConfig = __globals.getSelectizeOptions();

                scope = __Form.setup(scope, 'user_edit_form', 'userData');
                scope = __Form.updateModel(scope, requestData.userUpdateData);

                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.user.write.update_process',
                        'userId': ngDialogData.userId
                    }, scope)
                        .success(function (responseData) {
                            appServices.processResponse(responseData, null, function () {
                                // close dialog
                                $scope.closeThisDialog({'user_updated': true});
                            });
                        });
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

})(window, window.angular);
;
/*!
*  Component  : Users
*  File       : UserDataService.js
*  Engine     : UserDataService
----------------------------------------------------------------------------- */
(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.UserDataService', [])

        /**
         User Data Service
         ---------------------------------------------------------------- */
        .service('UserDataService', [
            '$q',
            '__DataStore',
            '__Form',
            'appServices',
            UserDataService
        ]);

    function UserDataService($q, __DataStore, __Form, appServices) {

        /*
            Get Login attempts
            -----------------------------------------------------------------*/
        this.getLoginAttempts = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __Form.fetch('user.login.attempts')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Countries List
            -----------------------------------------------------------------*/
        this.getCountries = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('user.get.country_list')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };
    }
    ;
})(window, window.angular);
;
/*!
*  Component  : User
*  File       : UserEngine.js
*  Engine     : UserEngine
----------------------------------------------------------------------------- */
(function (window, angular, undefined) {

    'use strict';

    angular
        .module('Manage-app.users', [])

        /**
         * UserLoginController - login a user in application
         *
         * @inject __Form
         * @inject __Auth
         * @inject appServices
         * @inject __Utils
         *
         * @return void
         *-------------------------------------------------------- */

        .controller('UserLoginController', [
            '__Form',
            '__Auth',
            'appServices',
            '__Utils',
            'UserDataService',
            '$state',
            '$rootScope',
            'BaseDataService',
            function (__Form, __Auth, appServices, __Utils, UserDataService, $state, $rootScope, BaseDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'form_user_login', 'loginData', {
                    secured: true
                });

                scope.show_captcha = false;
                scope.request_completed = false;

                /**
                 * Get login attempts for this client ip
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                UserDataService.getLoginAttempts()
                    .then(function (responseData) {
                        scope.show_captcha = responseData.data.show_captcha;
                        scope.site_key = responseData.data.site_key;
                        scope.request_completed = true;
                    });

                /**
                 * Fetch captch url
                 *
                 * @return string
                 *---------------------------------------------------------------- */

                scope.getCaptchaURL = function () {
                    return __Utils.apiURL('security.captcha') + '?ver=' + Math.random();
                };

                /**
                 * Refresh captch
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.refreshCaptcha = function () {
                    scope.captchaURL = scope.getCaptchaURL();
                };

                scope.captchaURL = scope.getCaptchaURL();

                scope.redirectToIntended = function () {

                    if (__globals.intended && __globals.intended.name && __globals.intended.params) {

                        return $state.go(__globals.intended.name, __globals.intended.params);

                    } else if (__globals.intended && __globals.intended.name) {

                        return $state.go(__globals.intended.name);
                    }

                    return $state.go('dashboard');

                };

                /**
                 * Submit login form action
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    scope.isInActive = false;
                    scope.accountDeleted = false;

                    __Form.process('user.login.process', scope).success(function (responseData) {

                        var requestData = responseData.data;

                        appServices.processResponse(responseData, {
                                error: function () {

                                    scope.show_captcha = requestData.show_captcha;

                                    // reset password field
                                    scope[scope.ngFormModelName].password = "";

                                    // Check if show captcha exist then refresh captcha
                                    if (scope.show_captcha) {
                                        scope[scope.ngFormModelName].confirmation_code = "";
                                        scope.refreshCaptcha();
                                    }

                                },
                                otherError: function (reactionCode) {

                                    scope.isInActive = requestData.isInActive;
                                    scope.accountDeleted = requestData.accountDeleted;

                                    // If reaction code is Server Side Validation Error Then
                                    // Unset the form fields
                                    if (reactionCode == 3) {

                                        // Check if show captcha exist then refresh captcha
                                        if (scope.show_captcha) {
                                            scope.refreshCaptcha();
                                        }

                                    }

                                    // If reaction code 10 is already authenticate.
                                    if (reactionCode == 10) {

                                        // Check if show captcha exist then refresh captcha
                                        scope.redirectToIntended();
                                        //__globals.redirectBrowser(__Utils.apiURL('dashboard'));

                                    }

                                }
                            },
                            function () {

                                BaseDataService.getBaseData('account_logged').then(function (data) {

                                    __globals.setCookie('auth_access_token', requestData.access_token);

                                    __Auth.checkIn(requestData.auth_info, function () {

                                        __globals.setCookie('auth_access_token', requestData.access_token);

                                        if (requestData.availableRoutes) {
                                            __globals.appImmutable('availableRoutes',
                                                requestData.availableRoutes);
                                        }

                                        $rootScope.$emit('lw.events.logged_in_user', {data: true});

                                        scope.redirectToIntended();

                                    });

                                });


                            });

                    });

                };

            }

        ])


        /**
         * UserLogoutController for login logout
         *
         * @inject __DataStore
         * @inject __Auth
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('UserLogoutController', [
            '__DataStore',
            '__Auth',
            'appServices',
            function UserLogoutController(__DataStore, __Auth, appServices) {

                var scope = this;

                __DataStore.post('user.logout').success(function (responseData) {

                    appServices.processResponse(responseData, function (reactionCode) {

                        // set user auth information
                        __Auth.checkIn(responseData.data.auth_info);

                    });

                });

            }
        ])

        /**
         * UserForgotPasswordController - request to send password reminder
         *
         * @inject __Form
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */

        .controller('UserForgotPasswordController', [
            '__Form',
            'appServices',
            '__Utils',
            '$state',
            function (__Form, appServices, __Utils, $state) {

                var scope = this;


                scope = __Form.setup(scope, 'user_forgot_password_form', 'userData', {
                    secured: true
                });

                /**
                 * Submit form
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process('user.forgot_password.process', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                scope.userData.usernameOrEmail = '';

                            });

                        });

                };

            }

        ])

        /**
         * UserResetPasswordController for reset user password
         *
         * @inject __Form
         * @inject appServices
         * @inject __Utils
         *
         * @return void
         *-------------------------------------------------------- */

        .controller('UserResetPasswordController', [
            '__Form',
            'appServices',
            '__Utils',
            '$state',
            function (__Form, appServices, __Utils, $state) {

                var scope = this;

                scope = __Form.setup(scope, 'user_reset_password_form', 'userData', {
                    secured: true
                });

                /**
                 * Submit reset password form action
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'user.reset_password.process',
                        'reminderToken': $state.params.reminderToken
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null,
                                function (reactionCode) {
                                    $state.go('login');
                                });

                        });

                };

            }
        ])

        /**
         * UserContactController handle register form & send request to server
         * to submit form data.
         *
         * @inject __Form
         * @inject $state
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('UserContactController', [
            '__Form',
            '$state',
            'appServices',
            '__Utils',
            '__Auth',
            '$scope',
            function UserContactController(__Form, $state, appServices, __Utils, __Auth, $scope) {

                var scope = this;

                scope.showCaptcha = false;

                scope = __Form.setup(scope, 'user_contact_form', 'userData', {
                    secured: true,
                    unsecuredFields: ['message']
                });

                // additional option for ck-editor
                scope.AdditionalOptionOfCkeditor = {
                    toolbar: [
                        {
                            name: 'links',
                            items: [
                                'Link',
                                'Unlink',
                                'Anchor'
                            ]
                        },
                        {
                            name: 'insert',
                            items: ['Smiley']
                        }
                    ],
                    extraPlugins: 'smiley'
                };

                // get logged in user Info
                __Auth.refresh(function (authInfo) {
                    scope.auth_info = authInfo;
                });

                scope.isLoggedIn = scope.auth_info.authorized;

                // check if user is logged in or not
                if (!scope.isLoggedIn) {
                    scope.showCaptcha = true;
                }

                if (scope.auth_info.reaction_code != 9) { // not authenticate
                    scope.userData.email = scope.auth_info.profile.email;
                    scope.userData.name = scope.auth_info.profile.full_name;
                }

                if (!_.isEmpty($scope.ngDialogData)) {

                    scope.userData.orderUID = $scope.ngDialogData.orderUID;
                }

                /**
                 * Fetch captch url
                 *
                 * @return string
                 *---------------------------------------------------------------- */

                scope.getCaptchaURL = function () {
                    return __Utils.apiURL('security.captcha') + '?ver=' + Math.random();
                };

                /**
                 * Refresh captch
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.refreshCaptcha = function () {
                    scope.captchaURL = scope.getCaptchaURL();
                };

                scope.captchaURL = scope.getCaptchaURL();


                /**
                 * Submit register form action
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.requestSuccess = false;
                scope.submit = function (formType) {

                    scope.userData.formType = formType;

                    __Form.process('user.contact.process', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, {
                                    error: function () {

                                    },
                                    otherError: function () {
                                        // refresh captcha
                                        scope[scope.ngFormModelName].confirmation_code = "";
                                        scope.refreshCaptcha();
                                    }
                                },
                                function () {

                                    scope.userData = '';

                                    CKEDITOR.instances['message'].setData('');


                                    // Check if form type dialog or form
                                    if (scope.userData.formType == 2) { // dialog

                                        $scope.closeThisDialog();

                                    }

                                    scope.requestSuccess = true;

                                    $('.lw-contact-form').slideUp();
                                });

                        });

                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])

        /**
         * UserAddCountryController handle address form & send request to server
         * to submit form data.
         *
         * @inject $scope
         * @inject __Form
         * @inject $state
         * @inject appServices
         * @inject __Utils
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('UserAddCountryController', [
            '$scope',
            '__Form',
            '$state',
            'appServices',
            '__Utils',
            'UserDataService',
            function UserAddCountryController($scope, __Form, $state, appServices, __Utils, UserDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'user_country_add_form', 'userData');

                scope.request_completed = false;
                scope.countries = [];

                UserDataService.getCountries()
                    .then(function (responseData) {
                        var requestData = responseData.data;
                        scope.countries = requestData.countries;
                        scope.request_completed = true;
                    });

                scope.countrySelectConfig = __globals.getSelectizeOptions();

                /**
                 * Submit profile edit form action
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process('user.add.country.process', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                $scope.closeThisDialog({'countryAdded': true});

                            });

                        });

                };


                /**
                 * Close dialog and return promise object
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.close = function () {

                    $scope.closeThisDialog({'countryAdded': false});
                };

            }
        ])
})(window, window.angular);
;
/*!
*  Component  : Configuration
*  File       : ConfigurationDataService.js
*  Engine     : ConfigurationDataService
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.ConfigurationDataService', [])
        .service('ConfigurationDataService', [
            '$q',
            '__DataStore',
            'appServices',
            ConfigurationDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function ConfigurationDataService($q, __DataStore, appServices) {

        /*
            Get the data of configuration
            -----------------------------------------------------------------*/

        this.readConfigurationData = function (formType) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.configuration.get.support.data',
                'formType': formType // different form type like 1, 2,3,4 etc
            }).success(function (responseData) {
                console.log(responseData);
                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

    };


})(window, window.angular);
;

/*!
*  Component  : Configuration
*  File       : ConfigurationEngine.js
*  Engine     : ConfigurationEngine
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.configuration', [])

        /**
         * GeneralController for update request
         *
         * @inject $scope
         * @inject __DataStore
         * @inject appServices
         * @inject __Form
         *
         * @return void
         *-------------------------------------------------------- */

        .controller('GeneralController', [
            '$scope',
            '__Form',
            '$state',
            'appServices',
            'lwFileUploader',
            '__Utils',
            '$rootScope',
            'getGeneralData',
            function ($scope, __Form, $state, appServices, lwFileUploader, __Utils, $rootScope, getGeneralData) {
// working
                var scope = this;

                scope.logo_background_color = getGeneralData.data.configuration.default_logo_backgroundColor;
                scope.primary_color = getGeneralData.data.configuration.default_primaryColor;

                scope = __Form.setup(scope, 'general_edit', 'editData', {
                    secured: true,
                    modelUpdateWatcher: true,
                    unsecuredFields: ['logoURL', 'faviconURL']
                });

                // scope.openon = scope.editData.openOn;
                // scope.expDate = moment(date).add(1, 'days').format(format);

                scope.editData.openon = getGeneralData.data.configuration.openOn;
                scope.editData.closeon = getGeneralData.data.configuration.closeOn;
                console.log(scope.editData.openon);
                scope.dateTimePickerOptions = {
                    date: false,
                    okText: 'Select',
                    // format    : 'HH:mm',
                    format: 'LTS',
                    shortTime: true
                };

                scope.pageStatus = false;

                scope.timezone_select_config = __globals.getSelectizeOptions({
                    valueField: 'value',
                    labelField: 'text',
                    searchField: ['text']
                });

                scope.home_page_select_config = __globals.getSelectizeOptions({
                    valueField: 'id'
                });

                scope.checkLogo = function (from) {
                    var isSame = false;

                    if (scope.editData.invoice_logo_image == scope.editData.logo_image) {
                        isSame = true;
                    }

                    if (isSame) {
                        if (from == 1) { // Logo
                            scope.editData.invoice_logo_image = '';
                        } else if (from == 2) { // Invoice Logo
                            scope.editData.logo_image = '';
                        }
                    }
                };

                /**
                 * Clear Color
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.clearColor = function () {
                    // var logo_background_color = getGeneralData.data.configuration.logo_background_color;
                    var logo_background_color = scope.logo_background_color;
                    scope.editData.logo_background_color = logo_background_color;

                }

                /**
                 * Clear Color
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.clearPrimaryColor = function () {
                    var primary_color = scope.primary_color;
                    scope.editData.primary_color = primary_color;
                }

                /**
                 * Fetch support data
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                var requestData = getGeneralData.data;
                // scope.timezoneData = requestData.configuration.timezone_list;
                scope.homePageData = __globals.generateKeyValueItems(requestData.configuration.home_page_list);

                // scope.languages = requestData.configuration.locale_list;
                var configuration = requestData.configuration;

                __Form.updateModel(scope, configuration);

                scope.pageStatus = true;

                scope.imagesSelectConfig = __globals.getSelectizeOptions({
                    valueField: 'name',
                    labelField: 'name',
                    render: {
                        item: function (item, escape) {
                            return __Utils.template('#imageListItemTemplate',
                                item
                            );
                        },
                        option: function (item, escape) {
                            return __Utils.template('#imageListOptionTemplate',
                                item
                            );
                        }
                    },
                    searchField: ['name']
                });


                /**
                 * Retrieve files required for account logo
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.retrieveSpecificFiles = function () {

                    lwFileUploader.getTempUploadedFiles(scope, {
                        'url': __Utils.apiURL('media.upload.read_logo')
                    }, function (uploadedFile) {
                        scope.logoFiles = uploadedFile;
                        scope.logoFilesCount = uploadedFile.length;
                    });

                };
                scope.retrieveSpecificFiles();

                scope.retrieveFaviconFiles = function () {
                    lwFileUploader.getTempUploadedFiles(scope, {
                        'url': __Utils.apiURL('media.upload.read_favicon')
                    }, function (uploadedFile) {
                        scope.faviconFiles = uploadedFile;
                        scope.faviconFilesCount = uploadedFile.length;
                    });
                };

                scope.retrieveFaviconFiles();

                $rootScope.$on('lw-loader-event-start', function (event, data) {
                    $scope.loading = true;
                    $("#lwFileupload").attr("disabled", true);
                });

                $rootScope.$on('lw-loader-event-stop', function (event, data) {
                    $scope.loading = false;
                    $("#lwFileupload").attr("disabled", false);
                });

                // uploader file instance
                $scope.upload = function () {

                    lwFileUploader.upload({
                        'url': __Utils.apiURL('media.upload.write.logo')
                    }, function (response) {

                        scope.retrieveSpecificFiles();
                        scope.retrieveFaviconFiles();

                    });
                };

                /**
                 * Show uploaded media files
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                $scope.showUploadedMediaDialog = function () {

                    lwFileUploader.openDialog(scope, {
                            'url': __Utils.apiURL('media.upload.read_logo')
                        },
                        function (promiseObject) {
                            scope.retrieveSpecificFiles();
                            scope.retrieveFaviconFiles();
                        });

                };

                /**
                 * update blog data
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.configuration.process',
                        'formType': 1
                    }, scope).success(function (responseData) {
                        appServices.processResponse(responseData, null, function () {

                            var requestData = responseData.data;
                            if (requestData.showRealodButton == true) {
                                __globals.showConfirmation({
                                    title: responseData.data.message,
                                    text: responseData.data.textMessage,
                                    type: "success",
                                    confirmButtonClass: "btn-success",
                                    confirmButtonText: $("#lwReloadBtnText").attr('data-message'),
                                    confirmButtonColor: "#337ab7",
                                }, function () {
                                    location.reload();
                                });
                            }
                        });
                    });
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                var logo_background_color = getGeneralData.data.configuration.logo_background_color;

                scope.closeDialog = function () {

                    // $('#lwchangeHeaderColor').css('background', "#"+logo_background_color);
                    $scope.closeThisDialog();
                };
            }
        ])

        /**
         * CurrencyConfigurationController for manage currency of store
         *
         * @inject $scope
         * @inject __Form
         * @inject appServices
         *
         * @return void
         *-------------------------------------------------------- */
        .controller('CurrencyConfigurationController', [
            '$scope',
            '__Form',
            'appServices',
            'getCurrencyData',
            function CurrencyConfigurationController($scope, __Form, appServices, getCurrencyData) {

                var scope = this;

                scope.isZeroDecimalCurrency = false;

                /**
                 * Generate key value
                 *
                 * @param bool responseKeyValue
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.generateCurrenciesArray = function (currencies, responseKeyValue) {

                    if (!responseKeyValue) {
                        return currencies;
                    }

                    var currenciesArray = [];

                    _.forEach(currencies, function (value, key) {

                        currenciesArray.push({
                            'currency_code': key,
                            'currency_name': value.name
                        });

                    });

                    var $lwCurrencySettingTxtMsg = $('#lwCurrencySettingTxtMsg');

                    currenciesArray.push({
                        'currency_code': 'other',
                        'currency_name': $lwCurrencySettingTxtMsg.attr('other-text')
                    });

                    return currenciesArray;

                };

                /**
                 *  Check the the currency match with zero decimal
                 *
                 * @param array zeroDecimalCurrecies
                 * @param string selectedCurrencyValue
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.checkIsZeroDecimalCurrency = function (zeroDecimalCurrecies, selectedCurrencyValue) {

                    var isMatch = _.filter(zeroDecimalCurrecies, function (value, key) {

                        return (key === selectedCurrencyValue);
                    });

                    scope.isZeroDecimalCurrency = Boolean(isMatch.length);

                };

                /**
                 * Check if current currency is Paypal supported or not
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.checkIsPaypalSupported = function (currencyValue) {

                    var isPaypalSupported = _.filter(scope.options, function (value, key) {

                        return (key == currencyValue);
                    });

                    scope.isPaypalSupport = Boolean(isPaypalSupported.length);
                };

                /**
                 * format currency symbol and currency value
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.formatCurrency = function (currencySymbol, currency) {

                    _.defer(function () {

                        var $lwCurrencyFormat = $('#lwCurrencyFormat');

                        var string = $lwCurrencyFormat.attr('data-format');

                        scope.currency_format_preview = string.split('{__currencySymbol__}').join(currencySymbol)
                            .split('{__amount__}').join(100)
                            .split('{__currencyCode__}').join(currency);
                    });
                };

                scope.pageStatus = false;

                scope = __Form.setup(scope, 'edit_currency_configuration', 'editData', {
                    secured: true,
                    unsecuredFields: [
                        'currency_symbol',
                        'currency_format'
                    ]
                });

                scope.currencies_select_config = __globals.getSelectizeOptions({
                    valueField: 'currency_code',
                    labelField: 'currency_name',
                    searchField: ['currency_code', 'currency_name']
                });

                scope.multi_currencies_select_config = __globals.getSelectizeOptions({
                    valueField: 'currency_code',
                    labelField: 'currency_name',
                    searchField: ['currency_code', 'currency_name'],
                    plugins: ['remove_button'],
                    maxItems: 1000,
                    delimiter: ',',
                    persist: false
                });

                scope.is_support_paypal = true;


                var requestData = getCurrencyData.data,
                    currenciesData = requestData.configuration.currencies;

                scope.options = currenciesData.options;
                scope.currencies = currenciesData.details;
                scope.zeroDecimal = currenciesData.zero_decimal;
                /*scope.currencies_options
                    = scope.generateCurrenciesArray(currenciesData.details, true);*/

                _.defer(function () {
                    scope.currencies_options
                        = scope.generateCurrenciesArray(currenciesData.details, true);
                });

                scope.checkIsZeroDecimalCurrency(scope.zeroDecimal, requestData.configuration.currency_value);

                scope.checkIsPaypalSupported(requestData.configuration.currency);

                scope.default_currency_format = requestData.configuration.default_currency_format;

                scope.placeholders = requestData.placeholders;
                scope = __Form.updateModel(scope, requestData.configuration);

                _.forEach(scope.currencies, function (currencyObj, key) {

                    if (key == scope.editData.currency_value) {
                        scope.currencySymbol = currencyObj.symbol;
                    }
                });

                if (requestData.configuration.currency == 'other') {
                    scope.currencySymbol = requestData.configuration.currency_symbol;
                }

                scope.formatCurrency(scope.currencySymbol, scope.editData.currency_value);


                scope.pageStatus = true;


                /**
                 * Use default format for currency
                 *
                 * @param string defaultCurrencyFormat
                 *
                 * @return string
                 *---------------------------------------------------------------- */
                scope.useDefaultFormat = function (defaultCurrencyFormat, currency_symbol, currency_value) {

                    scope.editData.currency_format = defaultCurrencyFormat;

                    var string = scope.editData.currency_format;

                    scope.currency_format_preview = string.split('{__currencySymbol__}').join(currency_symbol)
                        .split('{__amount__}').join(100)
                        .split('{__currencyCode__}').join(currency_value);
                };


                /**
                 * Use default format for currency
                 *
                 * @param string defaultCurrencyFormat
                 *
                 * @return string
                 *---------------------------------------------------------------- */
                scope.updateCurrencyPreview = function (currency_symbol, currency_value) {

                    if (_.isUndefined(currency_symbol)) {
                        currency_symbol = '';
                    }

                    if (_.isUndefined(currency_value)) {
                        currency_value = '';
                    }

                    var $lwCurrencyFormat = $('#lwCurrencyFormat');

                    var string = $lwCurrencyFormat.attr('data-format');

                    scope.currency_format_preview = string.split('{__currencySymbol__}').join(currency_symbol)
                        .split('{__amount__}').join(100)
                        .split('{__currencyCode__}').join(currency_value);

                };

                /**
                 * Submit currency Data
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.configuration.process',
                        'formType': 2 // currency
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                var requestData = responseData.data;

                                if (requestData.showRealodButton == true) {

                                    __globals.showConfirmation({
                                        title: responseData.data.message,
                                        text: responseData.data.textMessage,
                                        type: "success",
                                        showCancelButton: true,
                                        confirmButtonClass: "btn-success",
                                        confirmButtonText: $("#lwReloadBtnText").attr('data-message'),
                                        confirmButtonColor: "#337ab7"
                                    }, function () {

                                        location.reload();

                                    });
                                }
                            });

                        });
                };


                /**
                 * currency change
                 *
                 * @param selectedCurrency
                 * @return void
                 *---------------------------------------------------------------- */
                scope.currencyChange = function (selectedCurrency) {

                    scope.checkIsZeroDecimalCurrency(scope.zeroDecimal, selectedCurrency);

                    if (!_.isEmpty(selectedCurrency) && selectedCurrency != 'other') {

                        _.forEach(scope.currencies, function (currencyObj, key) {

                            if (key == selectedCurrency) {
                                scope.editData.currency_value = selectedCurrency;
                                scope.editData.currency_symbol = currencyObj.ASCII;
                                scope.currencySymbol = currencyObj.symbol;
                            }

                        });

                        scope.is_support_paypal = true;

                    } else {

                        scope.editData.currency_value = '';
                        scope.editData.currency_symbol = '';

                    }

                    scope.updateCurrencyPreview(scope.currencySymbol, scope.editData.currency_value);

                    scope.checkIsPaypalSupported(scope.editData.currency_value);

                };

                /**
                 * currency value change
                 *
                 * @param currencyValue
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.currencyValueChange = function (currencyValue) {

                    scope.checkIsZeroDecimalCurrency(scope.zeroDecimal, currencyValue);

                    if (!_.isEmpty(currencyValue) && currencyValue != 'other') {

                        var currency = {};
                        _.forEach(scope.currencies, function (currencyObj, key) {

                            if (key == currencyValue) {
                                currency = currencyObj;
                            }

                        });

                        if (_.isEmpty(currency)) {
                            //scope.is_support_paypal = false;
                            scope.editData.currency = 'other';
                        } else {
                            //scope.is_support_paypal     = true;
                            scope.editData.currency = currencyValue;
                            scope.editData.currency_symbol = currency.ASCII;
                            scope.currencySymbol = currency.symbol;
                        }

                    } else if (!_.isEmpty(currencyValue)) {

                        //scope.is_support_paypal     = false;
                        scope.editData.currency = 'other';

                    } else {

                        //scope.is_support_paypal  = true;
                        scope.editData.currency = '';

                    }

                    scope.checkIsPaypalSupported(currencyValue);

                    if (_.isUndefined(scope.editData.currency_value)) {
                        scope.currencySymbol = '';
                    }

                    scope.updateCurrencyPreview(scope.currencySymbol, scope.editData.currency_value);
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog(scope.currencySymbol, scope.editData.currency);
                };
            }
        ])
    ;

})(window, window.angular);
;
/*!
*  Component  : Dashboard
*  File       : DashboardDataService.js
*  Engine     : DashboardDataService
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.DashboardDataService', [])
        .service('DashboardDataService', [
            '$q',
            '__DataStore',
            'appServices',
            DashboardDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function DashboardDataService($q, __DataStore, appServices) {

        /*
            Get support data
            -----------------------------------------------------------------*/

        this.getSupportData = function (formType) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.dashboard.read.support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };


        /*
            Get Dashboard count details
            -----------------------------------------------------------------*/

        this.getCountDetail = function (componentType) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.dashboard.read.specific_data',
                'componentType': componentType
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };
    };


})(window, window.angular);
;
/*!
*  Component  : Dashboard
*  File       : DashboardEngine.js
*  Engine     : DashboardEngine
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('ManageApp.DashboardEngine', [])

        /**
         * Admin Dashboard Controller
         *
         * @inject object $scope
         * @inject object DashboardDataService
         * @inject object __Form
         * @inject object $stateParams
         *
         * @return void
         *---------------------------------------------------------------- */

        .controller('AdminDashboardController', [
            '$scope',
            'GetDatshboardData',
            '$rootScope',
            'appServices',
            function ($scope, GetDatshboardData, $rootScope, appServices) {

                var scope = this;
                scope.pageStatus = false;

                scope.dashboard = GetDatshboardData.dashboard;
                scope.userInfo = __globals.getAppImmutables('auth_info');
            }
        ])
    ;

})(window, window.angular);
;
/*!
*  Component  : RolePermission
*  File       : RolePermissionDataServices.js
*  Engine     : RolePermissionServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.RolePermissionDataServices', [])
        .service('RolePermissionDataService', [
            '$q',
            '__DataStore',
            'appServices',
            RolePermissionDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function RolePermissionDataService($q, __DataStore, appServices) {

        /*
            Get Permissions
            -----------------------------------------------------------------*/
        this.getPermissions = function (roleId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user.role_permission.read',
                'roleId': roleId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Add Role Support Data
            -----------------------------------------------------------------*/
        this.getAddSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.user.role_permission.read.add_support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get add support Data
            -----------------------------------------------------------------*/
        this.getAllPermissionsById = function (roleId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user.role_permission.read.using_id',
                'roleId': roleId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

    };

})(window, window.angular);
;
/*!
*  Component  : RolePermission
*  File       : RolePermission.js
*  Engine     : RolePermission
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.RolePermissionEngine', [])

        /**
         * Role Permission Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('RolePermissionController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            function ($scope, __DataStore, __Form, $stateParams) {

                var scope = this;

            }
        ])


        /**
         * Role Permission List Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object RolePermissionDataService
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('RolePermissionListController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'RolePermissionDataService',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, RolePermissionDataService) {
                var dtColumnsData = [
                        {
                            "name": "title",
                            "orderable": true,
                        },
                        {
                            "name": null,
                            "template": "#rolePermissionActionColumnTemplate"
                        }
                    ],
                    scope = this;

                /**
                 * Get general user test as a datatable source object
                 *
                 * @return  void
                 *---------------------------------------------------------- */

                scope.rolePermissionDataTable = __DataStore.dataTable('#lwrolePermissionList', {
                    url: 'manage.user.role_permission.read.list',
                    dtOptions: {
                        "searching": true,
                        "pageLength": 25
                    },
                    columnsData: dtColumnsData,
                    scope: $scope
                });

                /*
                Reload current datatable
                ------------------------------------------------------------ */
                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.rolePermissionDataTable);
                };

                /**
                 * rolePermission delete
                 *
                 * inject rolePermissionIdUid
                 *
                 * @return    void
                 *---------------------------------------------------------------- */

                scope.delete = function (rolePermissionIdOrUid, name) {

                    var $lwRolePermissionDeleteTextMsg = $('#lwRolePermissionDeleteTextMsg');

                    __globals.showConfirmation({
                        html: __globals.getReplacedString($lwRolePermissionDeleteTextMsg,
                            '__name__',
                            _.unescape(name)
                        ),
                        confirmButtonText: $lwRolePermissionDeleteTextMsg.attr('data-delete-button-text')
                    }, function () {

                        __DataStore.post({
                            'apiURL': 'manage.user.role_permission.write.delete',
                            'rolePermissionIdOrUid': rolePermissionIdOrUid
                        }).success(function (responseData) {

                            var message = responseData.data.message;

                            appServices.processResponse(responseData, {

                                error: function (data) {
                                    __globals.showConfirmation({
                                        title: $lwRolePermissionDeleteTextMsg.attr('data-error-text'),
                                        text: message,
                                        type: 'error'
                                    });
                                }

                            }, function (data) {
                                __globals.showConfirmation({
                                    title: $lwRolePermissionDeleteTextMsg.attr('data-success-text'),
                                    text: message,
                                    type: 'success'
                                });
                                scope.reloadDT();
                            });

                        });

                    });
                };

                /**
                 * Show add new role dialog
                 *
                 * @return    void
                 *---------------------------------------------------------------- */
                scope.showAddNewDialog = function () {

                    appServices.showDialog({},
                        {
                            templateUrl: __globals.getTemplateURL(
                                'user.role-permission.add-dialog'
                            ),
                            controller: 'AddRoleController as addRoleCtrl',
                            resolve: {
                                addSupportData: function () {
                                    return RolePermissionDataService
                                        .getAddSupportData();
                                }
                            }
                        },
                        function (promiseObj) {
                            if (_.has(promiseObj.value, 'role_Added')
                                && (promiseObj.value.role_Added === true)) {
                                scope.reloadDT();
                            }
                        });
                };

                /**
                 * Role Permission Dialog
                 *
                 * inject roleId
                 *
                 * @return    void
                 *---------------------------------------------------------------- */
                scope.rolePermissionDialog = function (roleId, title) {

                    appServices.showDialog({
                            'roleId': roleId,
                            'title': _.unescape(title)
                        },
                        {
                            templateUrl: __globals.getTemplateURL(
                                'user.role-permission.dynamic-role-permissions'
                            ),
                            controller: 'DynamicRolePermissionController as DynamicRolePermissionCtrl',
                            resolve: {
                                permissionData: function () {
                                    return RolePermissionDataService
                                        .getPermissions(roleId);
                                }
                            }
                        },
                        function (promiseObj) {

                        });
                };
            }
        ])
        // Role Permission List Controller ends here

        /**
         * Dynamic Role Permission Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('DynamicRolePermissionController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'appServices',
            'permissionData',
            function ($scope, __DataStore, __Form, $stateParams, appServices, permissionData) {
                var scope = this,
                    ngDialog = $scope.ngDialogData,
                    roleId = ngDialog.roleId;

                scope = __Form.setup(scope, 'user_role_dynamic_access', 'accessData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.title = ngDialog.title;
                scope.permissions = permissionData.permissions;

                scope.accessData.allow_permissions = permissionData.allow_permissions;
                scope.accessData.deny_permissions = permissionData.deny_permissions;
                scope.checkedPermission = {};

                scope.disablePermissions = function (eachPermission, permissionID) {

                    _.map(eachPermission.children, function (key) {
                        if (_.includes(key.dependencies, permissionID)) {
                            _.delay(function (text) {
                                $('input[name="' + key.id + '"]').attr('disabled', true);
                            }, 500);
                        }
                    });

                }

                _.map(scope.accessData.allow_permissions, function (permission) {
                    scope.checkedPermission[permission] = "2";
                })
                _.map(scope.accessData.deny_permissions, function (permission) {
                    scope.checkedPermission[permission] = "3";

                    _.map(scope.permissions, function (eachPermission) {

                        var pluckedIDs = _.pluck(eachPermission.children, 'id');

                        if (_.includes(pluckedIDs, permission)) {
                            scope.disablePermissions(eachPermission, permission)
                        }

                        if (_.has(eachPermission, 'children_permission_group')) {

                            _.map(eachPermission.children_permission_group, function (groupchild) {

                                var pluckedIDs = _.pluck(groupchild.children, 'id');

                                if (_.includes(pluckedIDs, permission)) {
                                    scope.disablePermissions(groupchild, permission)
                                }
                            });
                        }
                    });
                })

                scope = __Form.updateModel(scope, scope.accessData);

                //for updating permissions
                scope.checkPermission = function (childId, status) {

                    if (!_.isString(status)) {
                        status = status.toString();
                    }

                    scope.checkedPermission[childId] = status;

                    if (status == "2") {
                        if (!_.includes(scope.accessData.allow_permissions, childId)) {
                            scope.accessData.allow_permissions.push(childId);
                        }
                        if (_.includes(scope.accessData.deny_permissions, childId)) {
                            scope.accessData.deny_permissions = _.without(scope.accessData.deny_permissions, childId);
                        }
                    } else if (status == "3") {

                        if (!_.includes(scope.accessData.deny_permissions, childId)) {
                            scope.accessData.deny_permissions.push(childId);
                        }
                        if (_.includes(scope.accessData.allow_permissions, childId)) {
                            scope.accessData.allow_permissions = _.without(scope.accessData.allow_permissions, childId);
                        }
                    } else {

                        if (_.includes(scope.accessData.deny_permissions, childId)) {
                            scope.accessData.deny_permissions = _.without(scope.accessData.deny_permissions, childId);
                        }
                        if (_.includes(scope.accessData.allow_permissions, childId)) {
                            scope.accessData.allow_permissions = _.without(scope.accessData.allow_permissions, childId);
                        }
                    }

                    _.map(scope.permissions, function (permission) {

                        var pluckedIDs = _.pluck(permission.children, 'id'),
                            keyPermissions = [];
                        if (_.includes(pluckedIDs, childId) && permission.children[0].id != childId) {
                            // _.map(permission.children, function(key) {

                            // 	if (permission.children[0].id != key.id && !_.isUndefined(scope.checkedPermission[key.id])) {
                            // 		keyPermissions.push(scope.checkedPermission[key.id]);
                            // 	}
                            //     // if (key.id == childId && permission.children[0].id != childId) {
                            //     //     _.map(key.dependencies, function(dependency) {
                            //     //         scope.checkedPermission[dependency] = "2";
                            //     //     })
                            //     // }
                            // });

                            // scope.checkedPermission[permission.children[0].id] = "3";

                            // if (_.includes(keyPermissions, "2")) {
                            // 	scope.checkedPermission[permission.children[0].id] = "2";
                            // }

                        } else if (_.includes(pluckedIDs, childId) && permission.children[0].id == childId) {

                            _.map(permission.children, function (key) {

                                if (key.id != permission.children[0].id) {
                                    _.map(key.dependencies, function (dependency) {

                                        if (_.includes(key.dependencies, childId) && status == "3") {

                                            $('input[name="' + key.id + '"]').attr('disabled', true);

                                        } else {
                                            $('input[name="' + key.id + '"]').attr('disabled', false);

                                        }
                                    });
                                }
                            })
                        }

                        if (_.has(permission, 'children_permission_group')) {
                            _.map(permission.children_permission_group, function (groupchild) {

                                var pluckedGroupChildIDs = _.pluck(groupchild.children, 'id'),
                                    keyPermissionsGroup = [];

                                if (_.includes(pluckedGroupChildIDs, childId) && groupchild.children[0].id != childId) {
                                    //                            _.map(groupchild.children, function(groupchildkey) {
                                    //                                // if (groupchildkey.id == childId && groupchild.children[0].id != childId) {
                                    //                                //     _.map(groupchildkey.dependencies, function(dependency) {
                                    //                                //         scope.checkedPermission[dependency] = "2";
                                    //                                //     })
                                    //                                // }

                                    //                                if (groupchild.children[0].id != groupchildkey.id && !_.isUndefined(scope.checkedPermission[groupchildkey.id])) {
                                    //                      		keyPermissionsGroup.push(scope.checkedPermission[groupchildkey.id]);
                                    //                      	}

                                    //                            });

                                    //                            scope.checkedPermission[groupchild.children[0].id] = "3";

                                    // if (_.includes(keyPermissionsGroup, "2")) {
                                    // 	scope.checkedPermission[groupchild.children[0].id] = "2";
                                    // }

                                } else if (_.includes(pluckedGroupChildIDs, childId) && groupchild.children[0].id == childId) {

                                    _.map(groupchild.children, function (key2) {

                                        if (key2.id != groupchild.children[0].id) {
                                            _.map(key2.dependencies, function (dependency) {

                                                if (_.includes(key2.dependencies, childId) && status == "3") {

                                                    $('input[name="' + key2.id + '"]').attr('disabled', true);

                                                } else {
                                                    $('input[name="' + key2.id + '"]').attr('disabled', false);

                                                }
                                            })
                                        }
                                    });
                                }
                            })
                        }
                    })
                }

                // scope.preparePermissionData = function() {
                //     scope.accessData.allow_permissions = [];
                //     scope.accessData.deny_permissions = [];

                //     if (!_.isEmpty(scope.accessData.selected_permissions)) {
                //         _.forEach(scope.accessData.selected_permissions, function(item) {
                //             var number = item.split("_").pop();
                //             if (number == 2) {
                //                 scope.accessData.allow_permissions.push(_.trimRight(item, '_'+number));
                //             } else if (number == 3) {
                //                 scope.accessData.deny_permissions.push(_.trimRight(item, '_'+number));
                //             }
                //         });
                //     }
                // }

                /*
                 Submit form action
                -------------------------------------------------------------------------- */
                //           scope.filterPermissions = function(match) {

                //         		var treeInstance = $("#permissionTree").fancytree("getTree"),
                //         			filteredNodes,
                //           		filteredbranches,
                //        opts = {
                //       	'autoApply' : true,
                // 	'autoExpand' : true,
                // 	'fuzzy' : false,
                // 	'hideExpanders' : true,
                // 	'highlight' : true,
                // 	'leavesOnly' : true,
                // 	'nodata' : 'No results found.',
                // 	'mode' : "hide",
                // 	'counter': true,
                //       };

                // 	// Pass function to perform match
                // filteredNodes = treeInstance.filterNodes(match, opts);
                // filteredbranches = treeInstance.filterBranches(match, opts);
                //             }

                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {
                    // scope.preparePermissionData();
                    __Form.process({
                        'apiURL': 'manage.user.role_permission.write.create',
                        'roleId': roleId
                    }, scope)
                        .success(function (responseData) {
                            appServices.processResponse(responseData, null, function () {
                                // close dialog
                                $scope.closeThisDialog();
                            });
                        });
                };

                /*
                 * Check if value updated then enable and disable radio button according to
                 * current radio button
                 *
                 * @param string name
                 * @param number value
                 * @param array dependencies
                 * @param bool inheritStatus
                 *
                 * return array
                 * -------------------------------------------------------------------------- */
                scope.valueUpdated = function (name, value, dependencies, inheritStatus) {

                    _.forEach(scope.accessData.permissions, function (permission) {
                        if (permission[0].name == name) {

                            if (permission[0].allow == 2) { //Allow

                                _.map(permission, function (item) {
                                    if (!_.isEmpty(item.dependencies)) {
                                        item.disabled = false;
                                    }
                                });

                            } else if (permission[0].allow == 3) { // Deny

                                _.map(permission, function (item) {
                                    if (!_.isEmpty(item.dependencies)) {
                                        item.disabled = true;
                                        item.allow = 3;
                                    }
                                });

                            } else if (permission[0].allow == 1) { // Inherited

                                if (permission[0].currentStatus) {

                                    _.map(permission, function (item) {
                                        if (!_.isEmpty(item.dependencies)) {
                                            item.disabled = false;
                                            item.allow = 1;
                                        }
                                    });

                                } else {

                                    _.map(permission, function (item) {
                                        if (!_.isEmpty(item.dependencies)) {
                                            item.disabled = true;
                                            item.allow = 1;
                                        }
                                    });
                                }
                            }
                        }
                    });
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

        /**
         * Add new Role Permission Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('AddRoleController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'addSupportData',
            'appServices',
            'RolePermissionDataService',
            function ($scope, __DataStore, __Form, $stateParams, addSupportData, appServices, RolePermissionDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'add_role', 'roleData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.userRoles = addSupportData.userRoles;
                scope.permissions = addSupportData.permissionData;
                scope.roleData.allow_permissions = [];
                scope.roleData.deny_permissions = [];
                scope.checkedPermission = {};


                /*
                 Get Permission basis on the role id
                -------------------------------------------------------------------------- */
                scope.getPermissions = function (roleId) {

                    RolePermissionDataService
                        .getAllPermissionsById(roleId)
                        .then(function (responseData) {

                            scope.permissions = responseData.permissionData;
                            scope.roleData.selected_permissions = responseData.allowedData;

                            scope.roleData.allow_permissions = responseData.allow_permissions;
                            scope.roleData.deny_permissions = responseData.deny_permissions;
                            scope.checkedPermission = {};

                            _.map(scope.roleData.allow_permissions, function (permission) {
                                scope.checkedPermission[permission] = "2";
                            })
                            _.map(scope.roleData.deny_permissions, function (permission) {
                                scope.checkedPermission[permission] = "3";
                            })
                        })
                };

                //for updating permissions
                scope.checkPermission = function (childId, status) {

                    if (!_.isString(status)) {
                        status = status.toString();
                    }

                    scope.checkedPermission[childId] = status;

                    if (status == "2") {

                        if (!_.includes(scope.roleData.allow_permissions, childId)) {
                            scope.roleData.allow_permissions.push(childId);
                        }
                        if (_.includes(scope.roleData.deny_permissions, childId)) {
                            scope.roleData.deny_permissions = _.without(scope.roleData.deny_permissions, childId);
                        }

                    } else if (status == "3") {

                        if (!_.includes(scope.roleData.deny_permissions, childId)) {
                            scope.roleData.deny_permissions.push(childId);
                        }
                        if (_.includes(scope.roleData.allow_permissions, childId)) {
                            scope.roleData.allow_permissions = _.without(scope.roleData.allow_permissions, childId);
                        }
                    }

                    _.map(scope.permissions, function (permission) {

                        var pluckedIDs = _.pluck(permission.children, 'id'),
                            keyPermissions = [];
                        if (_.includes(pluckedIDs, childId) && permission.children[0].id != childId) {

                            // _.map(permission.children, function(key) {
                            // 	if (permission.children[0].id != key.id && !_.isUndefined(scope.checkedPermission[key.id])) {
                            // 		keyPermissions.push(scope.checkedPermission[key.id]);
                            // 	}

                            //     // if (key.id == childId && permission.children[0].id != childId) {
                            //     //     _.map(key.dependencies, function(dependency) {
                            //     //         scope.checkedPermission[dependency] = "2";
                            //     //     });
                            //     // }
                            // });

                            // scope.checkedPermission[permission.children[0].id] = "3";

                            // if (_.includes(keyPermissions, "2")) {
                            // 	scope.checkedPermission[permission.children[0].id] = "2";
                            // }

                        } else if (_.includes(pluckedIDs, childId) && permission.children[0].id == childId) {

                            _.map(permission.children, function (key) {

                                if (key.id != permission.children[0].id) {
                                    _.map(key.dependencies, function (dependency) {

                                        if (_.includes(key.dependencies, childId) && status == "3") {

                                            $('input[name="' + key.id + '"]').attr('disabled', true);

                                        } else {
                                            $('input[name="' + key.id + '"]').attr('disabled', false);

                                        }
                                    });
                                }
                            })
                        }

                        if (_.has(permission, 'children_permission_group')) {
                            _.map(permission.children_permission_group, function (groupchild) {

                                var pluckedGroupChildIDs = _.pluck(groupchild.children, 'id'),
                                    keyPermissionsGroup = [];

                                if (_.includes(pluckedGroupChildIDs, childId) && groupchild.children[0].id != childId) {
                                    //                            _.map(groupchild.children, function(groupchildkey) {

                                    //                            	if (groupchild.children[0].id != groupchildkey.id && !_.isUndefined(scope.checkedPermission[groupchildkey.id])) {
                                    //                      		keyPermissionsGroup.push(scope.checkedPermission[groupchildkey.id]);
                                    //                      	}

                                    //                                // if (groupchildkey.id == childId && groupchild.children[0].id != childId) {
                                    //                                //     _.map(groupchildkey.dependencies, function(dependency) {
                                    //                                //         scope.checkedPermission[dependency] = "2";
                                    //                                //     })
                                    //                                // }
                                    //                            });

                                    //                            scope.checkedPermission[groupchild.children[0].id] = "3";

                                    // if (_.includes(keyPermissionsGroup, "2")) {
                                    // 	scope.checkedPermission[groupchild.children[0].id] = "2";
                                    // }

                                } else if (_.includes(pluckedGroupChildIDs, childId) && groupchild.children[0].id == childId) {

                                    _.map(groupchild.children, function (key2) {

                                        if (key2.id != groupchild.children[0].id) {
                                            _.map(key2.dependencies, function (dependency) {

                                                if (_.includes(key2.dependencies, childId) && status == "3") {

                                                    $('input[name="' + key2.id + '"]').attr('disabled', true);

                                                } else {
                                                    $('input[name="' + key2.id + '"]').attr('disabled', false);

                                                }
                                            })
                                        }
                                    });
                                }

                            });
                        }
                    })
                }

                /*
                 Prepare Permissions
                -------------------------------------------------------------------------- */
                // scope.preparePermissions = function() {
                //     scope.roleData.allow_permissions = [];
                //     scope.roleData.deny_permissions = [];

                //     if (!_.isEmpty(scope.roleData.selected_permissions)) {
                //         _.forEach(scope.roleData.selected_permissions, function(item) {
                //             var number = item.split("_").pop();

                //             if (number == 2) {
                //                 scope.roleData.allow_permissions.push(_.trimRight(item, '_'+number));
                //             } else if (number == 3) {
                //                 scope.roleData.deny_permissions.push(_.trimRight(item, '_'+number));
                //             }
                //         });
                //     }
                // }

                /*
                 Submit form action
                -------------------------------------------------------------------------- */
                scope.submit = function () {
                    // scope.preparePermissions();
                    __Form.process('manage.user.role_permission.write.role.create', scope)
                        .success(function (responseData) {
                            appServices.processResponse(responseData, null, function () {
                                // close dialog
                                $scope.closeThisDialog({'role_Added': true});
                            });
                        });
                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
    ;

})(window, window.angular);
;
/*!
*  Component  : Dashboard
*  File       : ActivityDataServices.js
*  Engine     : ActivityDataServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.ActivityDataServices', [])
        .service('ActivityDataServices', [
            '$q',
            '__DataStore',
            'appServices',
            ActivityDataServices
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function ActivityDataServices($q, __DataStore, appServices) {

        /*
            Get support data
            -----------------------------------------------------------------*/

        this.getActionTypeData = function (activityLogId, actionTypeId) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.activity_log.action_type.read.data',
                'activityLogId': activityLogId
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

    };


})(window, window.angular);
;
/*!
*  Component  : Activity
*  File       : ActivityEngine.js
*  Engine     : ActivityEngine
----------------------------------------------------------------------------- */
(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.ActivityEngine', [])

        /**
         * Calendar Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ActivityLogListController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'appServices',
            'ActivityDataServices',
            function ($scope, __DataStore, __Form, $stateParams, appServices, ActivityDataServices) {
                var dtColumnsData = [
                        {
                            "name": "created_at",
                            "orderable": true,
                        },
                        {
                            "name": "created_by_user",
                            "orderable": true,
                        },
                        {
                            "name": "entity_type",
                            "orderable": true,
                        },
                        {
                            "name": "ip",
                            "orderable": false,
                        },
                        {
                            "name": null,
                            "orderable": false,
                            "template": '#actionColumnLinkTemplate'
                        }
                    ],
                    scope = this;

                //form setup
                scope = __Form.setup(scope, 'activity_form_filters', 'activityLogsData', {
                    secured: false,
                    unsecuredFields: []
                });

                /**
                 * Declare separate variables do not use non-repeatedly
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                scope.getDate = function (duration, dateFrom, dateTo) {
                    scope.startDate = dateFrom;
                    scope.endDate = dateTo;
                    scope.duration = duration;
                };

                /**
                 * Start Date greater than end date ,then convert start date to end date
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                scope.changeDate = function (startDate, endDate) {

                    if (scope.startDate > scope.endDate) {

                        scope.endDate = scope.startDate;
                    }
                };

                /**
                 * define all Variables data
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                var dateFrom, dateTo,

                    startCurrentMonth = moment().startOf('month').format('YYYY-MM-D'),
                    endCurrentMonth = moment().endOf('month').format('YYYY-MM-D'),

                    startLastMonth = moment().subtract(1, 'months').startOf('month').format('YYYY-MM-D'),
                    endLastMonth = moment().subtract(1, 'months').endOf('month').format('YYYY-MM-D'),

                    startCurrentWeek = moment().startOf('week').format('YYYY-MM-D'),
                    endCurrentWeek = moment().endOf('week').format('YYYY-MM-D'),

                    startLastWeek = moment().subtract(1, 'week').startOf('week').format('YYYY-MM-D'),
                    endLastWeek = moment().subtract(1, 'week').endOf('week').format('YYYY-MM-D'),

                    startToday = moment().startOf('day').format('YYYY-MM-D'),
                    endToday = moment().endOf('day').format('YYYY-MM-D'),

                    startYesterday = moment().subtract(1, 'day').format('YYYY-MM-D'),
                    endYesterday = moment().subtract(1, 'day').format('YYYY-MM-D'),

                    startLastYear = moment().subtract(1, 'years').startOf('years').format('YYYY-MM-D'),
                    endLastYear = moment().subtract(1, 'years').endOf('years').format('YYYY-MM-D'),

                    startCurrentYear = moment().startOf('years').format('YYYY-MM-D'),
                    endCurrentYear = moment().endOf('years').format('YYYY-MM-D'),

                    startLastThirtyDays = moment().subtract(30, 'days').format('YYYY-MM-D'),
                    endLastThirtyDays = moment().format('YYYY-MM-D');

                /**
                 * Get the all Duration value and use moment library to fetch date
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                scope.activityDataTable = function (duration, startDate, endDate) {

                    switch (parseInt(duration)) {
                        case 1:
                            dateFrom = startCurrentMonth;
                            dateTo = endCurrentMonth;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;

                        case 2:
                            dateFrom = startLastMonth;
                            dateTo = endLastMonth;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;

                        case 3:
                            dateFrom = startCurrentWeek;
                            dateTo = endCurrentWeek;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;

                        case 4:
                            dateFrom = startLastWeek;
                            dateTo = endLastWeek;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;

                        case 5:
                            dateFrom = startToday;
                            dateTo = endToday;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;

                        case 6:
                            dateFrom = startYesterday;
                            dateTo = endYesterday;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;

                        case 7:
                            dateFrom = startLastYear;
                            dateTo = endLastYear;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;
                        case 8:
                            dateFrom = startCurrentYear;
                            dateTo = endCurrentYear;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;
                        case 9:
                            dateFrom = startLastThirtyDays;
                            dateTo = endLastThirtyDays;
                            scope.getDate(duration, dateFrom, dateTo);

                            break;
                        case 10:
                            var manipulateDate = "Add Custom date";
                            break;
                    }
                    ;

                };

                /**
                 * Request to server
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                scope.dateChange = function () {

                    if (scope.activityLogDataTable) {
                        scope.activityLogDataTable.destroy();
                    }

                    scope.activityLogDataTable = __DataStore.dataTable('#lwActivityLogList', {
                        url: {
                            'apiURL': 'manage.activity_log.read.list',
                            'startDate': scope.startDate,
                            'endDate': scope.endDate
                        },
                        dtOptions: {
                            "searching": true,
                            "order": [[0, 'desc']],
                            "pageLength": 25
                        },
                        columnsData: dtColumnsData,
                        scope: $scope
                    }, null, function (responseData) {
                        scope.durations = responseData._options.durations;
                    });
                };
                /*
	            Reload current datatable
	            ------------------------------------------------------------ */
                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.activityLogDataTable);
                };

                // when add new record
                $scope.$on('activity_added_or_updated', function (data) {

                    if (data) {
                        // scope.reloadDT();
                        scope.dateChange('Today', moment().format('YYYY-MM-D'), moment().format('YYYY-MM-D'));
                    }

                });

                /**
                 * Fire Event for caching start date is greater than end date.
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                $scope.$watch('activityLogListCtrl.startDate', function (currentValue, oldValue) {

                    var $element = angular.element('#endDate');

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });


                /**
                 * Calling activityDataTable() function to get the current value.
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                scope.activityDataTable('1', moment().format('YYYY-MM-D'), moment().format('YYYY-MM-D'));

                scope.dateChange();


                //show all action types in this dialog start
                scope.openActionTypeDialog = function (activityLogId) {

                    appServices.showDialog({}, {
                        templateUrl: "activity.action-type-dialog",
                        controller: "ActionTypeController as actionTypeCtrl",
                        resolve: {
                            actionTypeData: ["ActivityDataServices", function (ActivityDataServices) {

                                return ActivityDataServices
                                    .getActionTypeData(activityLogId)
                            }]
                        }
                    }, function (promiseObj) {

                    });

                };
                //show all action types in this dialog end
            }
        ])

        /**
         * Calendar Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ActionTypeController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            'appServices',
            'actionTypeData',
            function ($scope, __DataStore, __Form, $stateParams, appServices, actionTypeData) {
                var scope = this;

                scope.actionTypeData = actionTypeData.actionTypeData;

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
    ;

})(window, window.angular);
;
/*!
*  Component  : Package
*  File       : PackageDataServices.js
*  Engine     : PackageServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.PackageDataServices', [])
        .service('PackageDataService', [
            '$q',
            '__DataStore',
            'appServices',
            PackageDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function PackageDataService($q, __DataStore, appServices) {


        /*
            Get Add Support Data
            -------------------------------------------------------------- */
        this.getAddSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.package.read.support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);
                    });
                });

            //return promise to caller
            return defferedObject.promise;
        };
        /*
            Get Edit Support Data
            -------------------------------------------------------------- */
        this.getEditSupportData = function (packageIdOrUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.package.read.update.data',
                'packageIdOrUid': packageIdOrUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Edit Support Data
            -------------------------------------------------------------- */
        this.getAssignPackageSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.customer.package.read.package_assign_data',
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Purcases
            -------------------------------------------------------------- */

        this.getUsersPurchases = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.user_purchses.read.users_purchases')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);

                    });

                });

            //return promise to caller
            return defferedObject.promise;
        };


        /*
            Get Purcases
            -------------------------------------------------------------- */
        this.getUserPurchase = function (purchaseUid, requestFor) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.user_purchses.read.user_purchase',
                'requestFor': requestFor,
                'userPackageUid': purchaseUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Purcases
            -------------------------------------------------------------- */
        this.getPackageDetails = function (packageIdOrUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.package.read.details',
                'packageIdOrUid': packageIdOrUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);

                });

            });

            //return promise to caller
            return defferedObject.promise;
        };

    };

})(window, window.angular);
;
/*!
*  Component  : Package
*  File       : Package.js
*  Engine     : Package
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.PackageEngine', [])


        /**
         * Package List Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object PackageDataService
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('PackageListController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'PackageDataService',
            'ItemDataService',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, PackageDataService, ItemDataService) {

                var dtColumnsData = [
                        {
                            "name": "title",
                            "orderable": true,
                            "template": "#packageDetailsColumnTemplate"
                        },
                        {
                            "name": "formatted_status",
                            "orderable": true,
                        },
                        {
                            "name": "created_at",
                            "orderable": true,
                        },
                        {
                            "name": "updated_at",
                            "orderable": true,
                        },
                        {
                            "name": null,
                            "template": "#packageActionColumnTemplate"
                        }
                    ],
                    scope = this;

                /**
                 * Get general user test as a datatable source object
                 *
                 * @return  void
                 *---------------------------------------------------------- */

                scope.packageDataTable = __DataStore.dataTable('#lwpackageList', {
                    url: 'manage.package.read.list',
                    dtOptions: {
                        "searching": true
                    },
                    columnsData: dtColumnsData,
                    scope: $scope
                });

                /*
            Reload current datatable
            ------------------------------------------------------------ */
                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.packageDataTable);
                };

                // when add new record
                $scope.$on('package_added_or_updated', function (data) {

                    if (data) {
                        scope.reloadDT();
                    }

                });

                scope.openItemAddDialog = function () {
                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("item.add-dialog"),
                        controller: 'ItemAddController as itemAddCtrl',
                        resolve: {
                            itemAddData: function () {
                                return ItemDataService.getAddSupportData();
                            }
                        }
                    }, function (promiseObj) {
                    });
                }

                /**
                 * Show User Permission Dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.openPackageDetailsDialog = function (packageIdOrUid) {

                    appServices.showDialog({}, {
                        templateUrl: __globals.getTemplateURL('package.package-detail-dialog'),
                        controller: 'PackageDetailsController as PackageDetailsCtrl',
                        resolve: {
                            packageDetail: ['PackageDataService', function (PackageDataService) {
                                return PackageDataService.getPackageDetails(packageIdOrUid);
                            }]
                        }
                    }, function (promiseObj) {

                    });
                };

                /**
                 * package delete
                 *
                 * inject packageIdUid
                 *
                 * @return    void
                 *---------------------------------------------------------------- */

                scope.delete = function (packageIdOrUid, packageName) {

                    scope.packageName = unescape(packageName);

                    _.defer(function () {

                        var $lwPackageDeleteConfirmTextMsg = $('#lwPackageDeleteConfirmTextMsg');

                        var $lwPackageDeleteConfirmTextMsg = $('#lwPackageDeleteConfirmTextMsg');

                        scope.deleteText = $lwPackageDeleteConfirmTextMsg.attr('data-message');
                        scope.deleteConfirmBtnText = $lwPackageDeleteConfirmTextMsg.attr('data-delete-button-text');
                        scope.successMsgText = $lwPackageDeleteConfirmTextMsg.attr('success-msg');

                    });

                    _.defer(function () {

                        __globals.showConfirmation({
                                html: scope.deleteText,
                                confirmButtonText: scope.deleteConfirmBtnText
                            },
                            function () {

                                __DataStore.post({
                                    'apiURL': 'manage.package.write.delete',
                                    'packageIdOrUid': packageIdOrUid
                                }).success(function (responseData) {

                                    var message = responseData.data.message;

                                    appServices.processResponse(responseData, {

                                            error: function () {

                                                __globals.showConfirmation({
                                                    title: 'Deleted!',
                                                    text: message,
                                                    type: 'error'
                                                });

                                            }
                                        },
                                        function () {

                                            __globals.showConfirmation({
                                                title: 'Deleted!',
                                                text: scope.successMsgText,
                                                type: 'success'
                                            });
                                            scope.reloadDT();   // reload datatable

                                        }
                                    );

                                });

                            })

                    });
                };
            }
        ])
        // Package List Controller ends here


        /**
         * Package Add Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope

         * inject object packageAddData
         * @return  void
         *---------------------------------------------------------------- */
        .controller('PackageAddController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'packageAddData',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, packageAddData) {

                var scope = this;
                scope.packageitems = packageAddData.packageitems;
                scope.currency_symbol = packageAddData.currency_symbol;
                scope.packageFrequencies = packageAddData.package_frequencies;

                scope = __Form.setup(scope, 'package_form', 'packageData', {
                    secured: true,
                    unsecuredFields: []
                });
                scope.packageData.selected_items = {};
                scope.packageData.status = 1;

                scope.packageFreq = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                scope.packageData.package_price = 0;
                scope.calculatePackagePrice = function () {

                    var selectedItems = [];
                    _.map(scope.packageitems.services, function (item) {
                        if (item.checked == true) {
                            selectedItems.push(item.item_id);
                        }
                    });

                    _.map(scope.packageitems.products, function (item) {
                        if (item.checked == true) {
                            selectedItems.push(item.item_id);
                        }
                    });

                    scope.packageData.selected_items = selectedItems;
                }

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function (e) {

                    __Form.process('manage.package.write.create', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                $state.go('package');
                            });

                        });

                };


            }
        ])

        // Package Add Controller ends here

        /**
         * Package Edit Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object 'packageEditData'
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('PackageEditController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'packageEditData',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, packageEditData) {

                var scope = this,
                    packageIdOrUid = $state.params.packageIdOrUid;
                scope.showLoader = true;

                scope = __Form.setup(scope, 'package_form', 'packageData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.packageitems = packageEditData.packageitems;
                scope.currency_symbol = packageEditData.currency_symbol;
                scope.packageFrequencies = packageEditData.package_frequencies;
                scope.added_frequencies = packageEditData.added_frequencies;
                scope.packageFreq = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                scope = __Form.updateModel(scope, packageEditData.packageData);

                scope.calculatePackagePrice = function () {

                    var initPrice = 0, totalPrice = 0;
                    var selectedItems = [];

                    _.map(scope.packageitems.services, function (item) {
                        if (item.checked == true) {
                            selectedItems.push(item.item_id);
                        }
                    });

                    _.map(scope.packageitems.products, function (item) {
                        if (item.checked == true) {
                            selectedItems.push(item.item_id);
                        }
                    });

                    scope.packageData.selected_items = selectedItems;
                }

                scope.calculatePackagePrice();

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.package.write.update',
                        'packageIdOrUid': packageIdOrUid
                    }, scope).success(function (responseData) {

                        appServices.processResponse(responseData, null, function () {

                            $state.go('package');
                        });

                    });

                };

            }
        ])
        // Package Edit Controller ends here


        /**
         * Package Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('PackageDetailsController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            'packageDetail',
            function ($scope, __DataStore, __Form, $state, appServices, packageDetail) {

                var scope = this;
                scope.packageDetails = packageDetail;

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])
        // Package Controller ends here


        /**
         * New Customer Package Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('NewCustomerPackageController', [
            '$scope',
            '$rootScope',
            '__DataStore',
            'appServices',
            '__Utils',
            '__Form',
            '$state',
            'AssignPackageData',
            'PackageDataService',
            'ManageUserDataService',
            function ($scope, $rootScope, __DataStore, appServices, __Utils, __Form, $state, AssignPackageData, PackageDataService, ManageUserDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'assigne_package_form', 'assignData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.assign_package_data = AssignPackageData.assign_package_data;
                scope.currency_symbol = AssignPackageData.currency_symbol;
                // scope.userData = userData.userData;
                scope.assignedPackages = null;// userData.assignedPackages;
                scope.bill_generate_toggle = 2;
                scope.loader = false;

                scope.packageDetails = {};

                scope.dateTimePickerOptions = {
                    time: false,
                    okText: 'Select'
                };

                scope.assignData.serviceDetails = scope.service_data;

                scope.preparepackageDetails = function (selected_package) {
                    var package_selected = selected_package;
                    if (_.isString(selected_package)) {
                        package_selected = parseInt(selected_package);
                    }
                    _.map(scope.assign_package_data, function (packge) {

                        if (package_selected == packge.package_id) {
                            scope.assignData.packageDetails = {};
                            scope.assignData.selected_package = selected_package;
                            scope.assignData.packageDetails = packge;
                            scope.assignData.packageDetails.package_amount_paid = packge.price;
                        }

                    });
                }

                scope.loadPackagedata = function () {
                    return PackageDataService
                        .getAssignPackageSupportData().then(function (responseData) {
                            scope.assign_package_data = responseData.assign_package_data;
                            scope.preparepackageDetails(scope.assignData.selected_package);
                        });
                }

                $rootScope.$on('reloadItemsOptions', function (event, args) {
                    if (args.reactionCode == 1) {
                        scope.loadPackagedata();
                    }
                });

                if (!_.isEmpty(scope.assign_package_data)) {
                    scope.preparepackageDetails(scope.assign_package_data[0].package_id);
                }

                scope.addNewItemSlot = function (item_uid, item_title) {
                    appServices.showDialog({
                        'item_uid': item_uid,
                        'item_title': item_title
                    }, {
                        templateUrl: __globals.getTemplateURL('item.add-slot-dialog'),
                        controller: 'ItemSlotAddController as ItemSlotAddCtrl',
                        resolve: {
                            slotAddData: ['ItemDataService', function (ItemDataService) {
                                return ItemDataService.getAddSupportData();
                            }]
                        }
                    }, function (promiseObj) {

                    });

                }

                scope.addServiceToAssign = function (packageOrService, assign_type) {
                    if (assign_type == 1) {
                        scope.assignData.selected_package = packageOrService;
                        scope.assignData.assign_type = assign_type;
                        scope.preparepackageDetails(packageOrService);
                    } else if (assign_type == 2) {
                        scope.assignData.products_selected = packageOrService;
                        scope.assignData.assign_type = assign_type;
                    }
                }
                //bind frequency price
                scope.changePackagePrice = function (price) {
                    scope.assignData.packageDetails.package_amount_paid = price;
                }

                scope.changePhone = function (phone) {
                    if (phone.length == 11) {
                        scope.phone_error_msg = "";
                        scope.assignData.userData.member_id = null;
                        scope.assignData.userData.name = null;
                        scope.assignData.userData.email = null;
                        ManageUserDataService.getMemberInfo(phone).then(function (responseData) {
                            if (responseData.status != 200) {
                                scope.phone_error_msg = responseData.message;
                            } else {
                                scope.assignData.userData = responseData.data.member;
                            }
                        });
                    }
                }

                scope.getExpiryData = function (date, frequency) {
                    var format = 'YYYY-MM-DD ';
                    if (!_.isUndefined(frequency) && !_.isUndefined(date)) {

                        scope.expDate = '';
                        switch (frequency) {
                            case 1:
                                scope.expDate = moment(date).add(1, 'days').format(format);
                                break;

                            case 2:
                                scope.expDate = moment(date).add(1, 'weeks').format(format);
                                break;
                            case 3:
                                scope.expDate = moment(date).add(1, 'months').format(format);
                                break;
                            case 4:
                                scope.expDate = moment(date).add(3, 'months').format(format);
                                break;
                            case 5:
                                scope.expDate = moment(date).add(6, 'months').format(format);
                                break;
                            case 6:
                                scope.expDate = moment(date).add(1, 'years').format(format);
                                break;

                            default:
                                return '';
                                break;
                        }
                    }
                }

                scope.showAssignedPackageDialog = function (packageData) {
                    appServices.showDialog({
                        'packageData': packageData,
                    }, {
                        templateUrl: __globals.getTemplateURL('package.user-assigned-package-detail'),
                        controller: 'AssignPackageInfoController as AssignPackageInfoCtrl',
                    }, function (promiseObj) {

                    });
                }
                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */
                scope.assign = function () {
                    scope.loader = true;
                    if (!scope.assignData.userData.member_id) {
                        ManageUserDataService.createMember(scope.assignData.userData).then(function (responseData) {
                            // if (responseData.status != 200) {
                            //     scope.phone_error_msg = responseData.message;
                            //     scope.assigne_package_form.valid = false;
                            //     return;
                            // }
                            scope.assignData.userData = responseData.data.member
                            scope.sendForm();
                        });
                    } else {
                        scope.sendForm();
                    }
                };
                scope.sendForm = function () {
                    scope.loader = false;
                    if (scope.assignData.assign_type == 1) {
                        scope.assignData.serviceDetails = {};
                    } else if (scope.assignData.assign_type == 2) {
                        scope.assignData.packageDetails = {};
                    }
                    __Form.process({
                        'apiURL': 'manage.customer.package.write.package_assign_member'
                        // 'userId': scope.userData.id
                        // 'userId': 1
                    }, scope).success(function (responseData) {
                        appServices.processResponse(responseData, null, function () {
                            __globals.showConfirmation({
                                title: 'Package Assigned Sucessfully',
                                type: "success",
                                confirmButtonColor: "#337ab7",
                                html: __Utils.template('#packageAssignedSuccessAlert', {
                                    'assign_package_data': responseData.data.assign_package_data,
                                }),
                                confirmButtonText: 'Ok',
                                showCancelButton: false,
                                confirmButtonClass: 'btn btn-primary',
                                cancelButtonClass: 'btn btn-gray',
                                buttonsStyling: false,
                            }, function () {
                                if (scope.bill_generate_toggle == 1) {
                                    $state.go('generate_package_bill', {
                                        userPackageId: responseData.data.user_package_uid
                                    })
                                } else {
                                    $state.go('customers');
                                }
                            });
                        });
                    });

                };

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.assignAndGenerate = function () {

                    if (scope.assignData.assign_type == 1) {
                        scope.assignData.serviceDetails = {};
                    } else if (scope.assignData.assign_type == 2) {
                        scope.assignData.packageDetails = {};
                    }

                    __Form.process({
                        'apiURL': 'manage.customer.package.write.package_assign_member'
                        // 'userId': $state.params.customerUid
                    }, scope).success(function (responseData) {
                        appServices.processResponse(responseData, null, function () {
                            __globals.showConfirmation({
                                title: 'Package Assigned Sucessfully',
                                type: "success",
                                confirmButtonColor: "#337ab7",
                                html: __Utils.template('#packageAssignedSuccessAlert', {
                                    'assign_package_data': responseData.data.assign_package_data,
                                }),
                                confirmButtonText: 'Ok',
                                showCancelButton: false,
                                confirmButtonClass: 'btn btn-primary',
                                cancelButtonClass: 'btn btn-gray',
                                buttonsStyling: false,
                            }, function () {
                                $state.go('generate_package_bill', {
                                    userPackageId: responseData.data.user_package_uid
                                })
                            });
                        });

                    });

                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };


            }
        ])

        /**
         * Assign Package Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('AssignPackageController', [
            '$scope',
            '$rootScope',
            '__DataStore',
            'appServices',
            '__Utils',
            '__Form',
            '$state',
            'AssignPackageData',
            'userData',
            'PackageDataService',
            function ($scope, $rootScope, __DataStore, appServices, __Utils, __Form, $state, AssignPackageData, userData, PackageDataService) {

                var scope = this;

                scope = __Form.setup(scope, 'assigne_package_form', 'assignData', {
                    secured: true,
                    unsecuredFields: []
                });
                scope.assign_package_data = AssignPackageData.assign_package_data;
                scope.currency_symbol = AssignPackageData.currency_symbol;
                scope.userData = userData.userData;
                scope.assignedPackages = userData.assignedPackages;
                scope.bill_generate_toggle = 2;

                scope.packageDetails = {};

                scope.dateTimePickerOptions = {
                    time: false,
                    okText: 'Select'
                };

                scope.assignData.serviceDetails = scope.service_data;

                scope.preparepackageDetails = function (selected_package) {

                    var package_selected = selected_package;
                    if (_.isString(selected_package)) {
                        package_selected = parseInt(selected_package);
                    }
                    _.map(scope.assign_package_data, function (packge) {

                        if (package_selected == packge.package_id) {
                            scope.assignData.packageDetails = {};
                            scope.assignData.selected_package = selected_package;
                            scope.assignData.packageDetails = packge;
                            scope.assignData.packageDetails.package_amount_paid = packge.price;
                        }

                    });
                }

                scope.loadPackagedata = function () {
                    return PackageDataService
                        .getAssignPackageSupportData()
                        .then(function (responseData) {
                            scope.assign_package_data = responseData.assign_package_data;
                            scope.preparepackageDetails(scope.assignData.selected_package);
                        });
                }

                $rootScope.$on('reloadItemsOptions', function (event, args) {
                    if (args.reactionCode == 1) {
                        scope.loadPackagedata();
                    }
                });

                if (!_.isEmpty(scope.assign_package_data)) {
                    scope.preparepackageDetails(scope.assign_package_data[0].package_id);
                }

                scope.addNewItemSlot = function (item_uid, item_title) {

                    appServices.showDialog({
                        'item_uid': item_uid,
                        'item_title': item_title
                    }, {
                        templateUrl: __globals.getTemplateURL('item.add-slot-dialog'),
                        controller: 'ItemSlotAddController as ItemSlotAddCtrl',
                        resolve: {
                            slotAddData: ['ItemDataService', function (ItemDataService) {
                                return ItemDataService.getAddSupportData();
                            }]
                        }
                    }, function (promiseObj) {

                    });

                }

                scope.addServiceToAssign = function (packageOrService, assign_type) {

                    if (assign_type == 1) {
                        scope.assignData.selected_package = packageOrService;
                        scope.assignData.assign_type = assign_type;
                        scope.preparepackageDetails(packageOrService);
                    } else if (assign_type == 2) {
                        scope.assignData.products_selected = packageOrService;
                        scope.assignData.assign_type = assign_type;
                    }
                }

                //bind frequency price
                scope.changePackagePrice = function (price) {

                    scope.assignData.packageDetails.package_amount_paid = price;
                }

                scope.getExpiryData = function (date, frequency) {
                    var format = 'YYYY-MM-DD ';
                    if (!_.isUndefined(frequency) && !_.isUndefined(date)) {

                        scope.expDate = '';
                        switch (frequency) {
                            case 1:
                                scope.expDate = moment(date).add(1, 'days').format(format);
                                break;

                            case 2:
                                scope.expDate = moment(date).add(1, 'weeks').format(format);
                                break;
                            case 3:
                                scope.expDate = moment(date).add(1, 'months').format(format);
                                break;
                            case 4:
                                scope.expDate = moment(date).add(3, 'months').format(format);
                                break;
                            case 5:
                                scope.expDate = moment(date).add(6, 'months').format(format);
                                break;
                            case 6:
                                scope.expDate = moment(date).add(1, 'years').format(format);
                                break;

                            default:
                                return '';
                                break;
                        }
                    }
                }

                scope.showAssignedPackageDialog = function (packageData) {
                    appServices.showDialog({
                        'packageData': packageData,
                    }, {
                        templateUrl: __globals.getTemplateURL('package.user-assigned-package-detail'),
                        controller: 'AssignPackageInfoController as AssignPackageInfoCtrl',
                    }, function (promiseObj) {

                    });
                }

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */
                scope.assign = function () {
                    scope.assignData.userData = scope.userData;
                    if (scope.assignData.assign_type == 1) {
                        scope.assignData.serviceDetails = {};
                    } else if (scope.assignData.assign_type == 2) {
                        scope.assignData.packageDetails = {};
                    }
                    __Form.process({
                        'apiURL': 'manage.customer.package.write.package_assign_member'
                    }, scope).success(function (responseData) {
                        appServices.processResponse(responseData, null, function () {
                            __globals.showConfirmation({
                                title: 'Package Assigned Sucessfully',
                                type: "success",
                                confirmButtonColor: "#337ab7",
                                html: __Utils.template('#packageAssignedSuccessAlert', {
                                    'assign_package_data': responseData.data.assign_package_data,
                                }),
                                confirmButtonText: 'Ok',
                                showCancelButton: false,
                                confirmButtonClass: 'btn btn-primary',
                                cancelButtonClass: 'btn btn-gray',
                                buttonsStyling: false,
                            }, function () {
                                if (scope.bill_generate_toggle == 1) {
                                    $state.go('generate_package_bill', {
                                        userPackageId: responseData.data.user_package_uid
                                    })
                                } else {
                                    $state.go('customers');
                                }
                            });
                        });
                    });
                };

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.assignAndGenerate = function () {

                    if (scope.assignData.assign_type == 1) {
                        scope.assignData.serviceDetails = {};
                    } else if (scope.assignData.assign_type == 2) {
                        scope.assignData.packageDetails = {};
                    }

                    __Form.process({
                        'apiURL': 'manage.customer.package.write.assign_to_user',
                        'userId': $state.params.customerUid
                    }, scope).success(function (responseData) {

                        appServices.processResponse(responseData, null, function () {
                            __globals.showConfirmation({
                                title: 'Package Assigned Sucessfully',
                                type: "success",
                                confirmButtonColor: "#337ab7",
                                html: __Utils.template('#packageAssignedSuccessAlert', {
                                    'assign_package_data': responseData.data.assign_package_data,
                                }),
                                confirmButtonText: 'Ok',
                                showCancelButton: false,
                                confirmButtonClass: 'btn btn-primary',
                                cancelButtonClass: 'btn btn-gray',
                                buttonsStyling: false,
                            }, function () {
                                $state.go('generate_package_bill', {
                                    userPackageId: responseData.data.user_package_uid
                                })
                            });
                        });

                    });

                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };


            }
        ])

        /**
         * Assign Package Info Controllers
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('AssignPackageInfoController', [
            '$scope',
            function ($scope) {

                var scope = this,
                    ngDialogData = $scope.ngDialogData;
                scope.packageData = ngDialogData.packageData;

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
    ;

})(window, window.angular);
;
/*!
*  Component  : Item
*  File       : ItemDataServices.js
*  Engine     : ItemServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.ItemDataServices', [])
        .service('ItemDataService', [
            '$q',
            '__DataStore',
            'appServices',
            ItemDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function ItemDataService($q, __DataStore, appServices) {


        /*
            Get Add Support Data
            -------------------------------------------------------------- */
        this.getAddSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.item.read.support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);
                    });
                });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Edit Support Data
            -------------------------------------------------------------- */
        this.getEditSupportData = function (itemIdOrUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.item.read.update.data',
                'itemIdOrUid': itemIdOrUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Details
            -------------------------------------------------------------- */
        this.getDetails = function (itemIdOrUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.item.read.deatils',
                'itemIdOrUid': itemIdOrUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };


    };

})(window, window.angular);
;
/*!
*  Component  : Item
*  File       : Item.js
*  Engine     : Item
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.ItemEngine', [])


        /**
         * Item List Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object ItemDataService
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ItemListController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'ItemDataService',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, ItemDataService) {
                var dtColumnsData = [
                        {
                            "name": "title",
                            "orderable": true,
                            "template": '#dTitleTemplate'
                        },
                        {
                            "name": "type",
                            "orderable": true,
                        },
                        {
                            "name": "status",
                            "orderable": true,
                        },
                        {
                            "name": "created_at",
                            "orderable": true,
                            'template': '#dtCreatedAtTemplate'
                        },
                        {
                            "name": "updated_at",
                            "orderable": true,
                            'template': '#dtUpdatedAtTemplate'
                        },
                        {
                            "name": null,
                            "template": "#itemActionColumnTemplate"
                        }
                    ],
                    scope = this;

                /**
                 * Get general user test as a datatable source object
                 *
                 * @return  void
                 *---------------------------------------------------------- */

                scope.itemDataTable = __DataStore.dataTable('#lwitemList', {
                    url: 'manage.item.read.list',
                    dtOptions: {
                        "searching": true
                    },
                    columnsData: dtColumnsData,
                    scope: $scope
                });

                /*
                Reload current datatable
                ------------------------------------------------------------ */
                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.itemDataTable);
                };

                // when add new record
                $scope.$on('item_added_or_updated', function (data) {

                    if (data) {
                        scope.reloadDT();
                    }

                });

                /**
                 * item delete
                 *
                 * inject itemIdUid
                 *
                 * @return    void
                 *---------------------------------------------------------------- */

                scope.delete = function (itemIdOrUid, itemTitle) {

                    scope.deletingItemName = _.unescape(itemTitle);

                    _.defer(function () {

                        var $lwItemDeleteConfirm = $('#lwItemDeleteConfirm');
                        scope.deleteText = $lwItemDeleteConfirm.attr('data-message');
                        scope.deleteConfirmBtnText = $lwItemDeleteConfirm.attr('data-delete-button-text');
                        scope.successMsgText = $lwItemDeleteConfirm.attr('success-msg');

                        __globals.showConfirmation({
                                html: scope.deleteText,
                                confirmButtonText: scope.deleteConfirmBtnText
                            },
                            function () {

                                __DataStore.post({
                                    'apiURL': 'manage.item.write.delete',
                                    'itemIdOrUid': itemIdOrUid
                                }).success(function (responseData) {

                                    var message = responseData.data.message;

                                    appServices.processResponse(responseData, {

                                            error: function () {

                                                __globals.showConfirmation({
                                                    title: 'Deleted!',
                                                    text: message,
                                                    type: 'error'
                                                });
                                            }
                                        },
                                        function () {

                                            __globals.showConfirmation({
                                                title: 'Deleted!',
                                                text: scope.successMsgText,
                                                type: 'success'
                                            });

                                            scope.reloadDT();   // reload datatable

                                        });

                                });
                            })

                    });


                };


                /*
            add dialog
            ------------------------------------------------------------ */
                scope.openAddDialog = function () {

                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("item.add-dialog"),
                        controller: 'ItemAddController as itemAddCtrl',
                        resolve: {
                            itemAddData: function () {
                                return ItemDataService.getAddSupportData();
                            }
                        }
                    }, function (promiseObj) {

                        if (_.has(promiseObj.value, 'item_added_or_updated')
                            && promiseObj.value.item_added_or_updated) {

                            scope.reloadDT();
                        }

                    });

                };

                /*
            edit dialog
            ------------------------------------------------------------ */
                scope.openEditDialog = function (itemIdOrUid) {

                    appServices.showDialog({
                        'itemIdOrUid': itemIdOrUid
                    }, {
                        templateUrl: __globals.getTemplateURL("item.edit-dialog"),
                        controller: 'ItemEditController as itemEditCtrl',
                        resolve: {
                            itemEditData: function () {
                                return ItemDataService
                                    .getEditSupportData(itemIdOrUid);
                            }
                        }
                    }, function (promiseObj) {

                        if (_.has(promiseObj.value, 'item_added_or_updated') && promiseObj.value.item_added_or_updated) {

                            scope.reloadDT();
                        }


                    });
                };

                /*
            Get dialog
            ------------------------------------------------------------ */
                scope.showDetails = function (itemIdOrUid) {

                    appServices.showDialog({
                        'itemIdOrUid': itemIdOrUid
                    }, {
                        templateUrl: __globals.getTemplateURL("item.details-dialog"),
                        controller: 'ItemDetailsController as itemDetailsCtrl',
                        resolve: {
                            GetDetails: function () {
                                return ItemDataService.getDetails(itemIdOrUid);
                            }
                        }
                    }, function (promiseObj) {
                    });
                };


            }
        ])
        // Item List Controller ends here


        /**
         * Item Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ItemAddController', [
            '$scope',
            '__DataStore',
            '__Form',
            'appServices',
            'itemAddData',
            function ($scope, __DataStore, __Form, appServices, itemAddData) {

                var scope = this;

                scope.showLoader = true;

                scope = __Form.setup(scope, 'item_form', 'itemData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.itemData.status = 1;
                scope.itemConfig = itemAddData.item;

                scope.itemData.slots = [
                    {
                        rowId: 'row1',
                        title: '',
                        _uid: '',
                        slot_frequency: 1,
                        slot_duration_minutes: '',
                        slot_time: ''
                    }
                ];

                scope.slotFrequencyOp = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                /**
                 * Add New Row of slots
                 */
                scope.addNewRow = function () {

                    var newItemNo = scope.itemData.slots.length + 1;

                    scope.itemData.slots.push({
                        'rowId': 'row' + newItemNo,
                        'slot_frequency': 1,
                    });
                };

                /**
                 * Remove row
                 */
                scope.removeRow = function (index) {

                    // remove the row specified in index
                    scope.itemData.slots.splice(index, 1);

                    // if no rows left in the array create a blank array
                    if (scope.itemData.slots.length === 0 || scope.itemData.slots.length == null) {
                        // alert('no rec');
                        scope.itemData.slots.push = [{"row": "row1"}];
                    }

                };

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function (e) {

                    __Form.process('manage.item.write.create', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {
                                $scope.closeThisDialog({'item_added_or_updated': true});
                            });

                        });
                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
        // ItemAddController ends here


        /**
         * Item Slot Add Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ItemSlotAddController', [
            '$scope',
            '$rootScope',
            '__DataStore',
            '__Form',
            'appServices',
            'slotAddData',
            function ($scope, $rootScope, __DataStore, __Form, appServices, slotAddData) {

                var scope = this;
                var ngDialogData = $scope.ngDialogData;
                scope.item_title = ngDialogData.item_title;
                scope.item_uid = ngDialogData.item_uid;
                scope.showLoader = true;

                scope = __Form.setup(scope, 'item_slot_add_form', 'itemSlotData', {
                    secured: true,
                    unsecuredFields: []
                });

                scope.itemConfig = slotAddData.item;

                scope.itemSlotData.slots = [
                    {
                        rowId: 'row1',
                        title: '',
                        slot_frequency: 1,
                        slot_duration_minutes: '',
                        slot_time: ''
                    }
                ];

                scope.slotFrequencyOp = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                /**
                 * Add New Row of slots
                 */
                scope.addNewRow = function () {

                    var newItemNo = scope.itemSlotData.slots.length + 1;

                    scope.itemSlotData.slots.push({
                        'rowId': 'row' + newItemNo,
                        'slot_frequency': 1,
                    });
                };

                /**
                 * Remove row
                 */
                scope.removeRow = function (index) {

                    // remove the row specified in index
                    scope.itemSlotData.slots.splice(index, 1);

                    // if no rows left in the array create a blank array
                    if (scope.itemSlotData.slots.length === 0 || scope.itemSlotData.slots.length == null) {
                        // alert('no rec');
                        scope.itemSlotData.slots.push = [{"row": "row1"}];
                    }

                };

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function (e) {

                    __Form.process({
                        'apiURL': 'manage.item.add.slots',
                        'itemIdOrUid': scope.item_uid
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function (reactionCode) {

                                if (reactionCode == 1) {
                                    $scope.$emit('reloadItemsOptions', {reactionCode: reactionCode});
                                }

                                $scope.closeThisDialog({'item_added_or_updated': true});
                            });

                        });
                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
        // ItemAddController ends here

        /**
         * Item Edit Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object itemEditData
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ItemEditController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'itemEditData',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, itemEditData) {

                var scope = this;
                scope.showLoader = true;

                scope = __Form.setup(scope, 'item_form', 'itemData', {
                    secured: true,
                    unsecuredFields: []
                });

                var requestData = itemEditData;

                scope = __Form.updateModel(scope, requestData.item);
                scope.showLoader = false;

                var itemIdOrUid = $scope.ngDialogData.itemIdOrUid;

                scope.itemConfig = requestData.config;

                /**
                 * Add New Row of slots
                 */
                scope.addNewRow = function () {

                    var newItemNo = scope.itemData.slots.length + 1;

                    scope.itemData.slots.push({
                        'rowId': 'row' + newItemNo,
                        'slot_frequency': 1,
                        '_uid': ''
                    });

                };

                scope.slotFrequencyOp = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                /**
                 * Remove row
                 */
                scope.removeRow = function (index, show_delete_warning, title) {

                    scope.itemSlotToRemove = '';
                    scope.itemSlotToRemove = title;

                    if (show_delete_warning) {

                        _.defer(function () {

                            var lwItemSlotremoveConfirmTextMsg = $('#lwItemSlotremoveConfirmTextMsg');

                            scope.deleteText = lwItemSlotremoveConfirmTextMsg.attr('data-message');
                            scope.deleteConfirmBtnText = lwItemSlotremoveConfirmTextMsg.attr('data-delete-button-text');

                            __globals.showConfirmation({
                                    html: scope.deleteText,
                                    confirmButtonText: scope.deleteConfirmBtnText
                                },
                                function () {

                                    scope.itemData.slots.splice(index, 1);

                                    // if no rows left in the array create a blank array
                                    if (scope.itemData.slots.length === 0 || scope.itemData.slots.length == null) {
                                        // alert('no rec');
                                        scope.itemData.slots.push = [{"row": "row1"}];
                                    }

                                });

                        });
                    } else {
                        // remove the row specified in index
                        scope.itemData.slots.splice(index, 1);

                        // if no rows left in the array create a blank array
                        if (scope.itemData.slots.length === 0 || scope.itemData.slots.length == null) {
                            // alert('no rec');
                            scope.itemData.slots.push = [{"row": "row1"}];
                        }
                    }

                };

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.item.write.update',
                        'itemIdOrUid': itemIdOrUid
                    }, scope).success(function (responseData) {

                        appServices.processResponse(responseData, null, function () {
                            $scope.closeThisDialog({'item_added_or_updated': true});
                        });

                    });

                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }

        ])
        // Item Edit Controller ends here


        /**
         * Item Details Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object itemEditData
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ItemDetailsController', [
            '$scope',
            'GetDetails',
            function ($scope, GetDetails) {

                var scope = this;

                scope.details = GetDetails.item;


                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }])
    // Item Edit Controller ends here


    ;

})(window, window.angular);
;
/*!
*  Component  : Slots
*  File       : SlotsDataServices.js
*  Engine     : SlotsServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.SlotsDataServices', [])
        .service('SlotsDataService', [
            '$q',
            '__DataStore',
            'appServices',
            SlotsDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function SlotsDataService($q, __DataStore, appServices) {


    };

})(window, window.angular);
;
/*!
*  Component  : Slots
*  File       : Slots.js
*  Engine     : Slots
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.SlotsEngine', [])

        /**
         * Slots Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('SlotsController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$stateParams',
            function ($scope, __DataStore, __Form, $stateParams) {

                var scope = this;

            }
        ])


        /**
         * Change Slot Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $stateParams
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ChangeSlotController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$rootScope',
            'appServices',
            function ($scope, __DataStore, __Form, $rootScope, appServices) {

                var scope = this,
                    ngDialogData = $scope.ngDialogData;

                scope = __Form.setup(scope, 'change_slot_form', 'slotData', {secured: true});

                scope.requestType = ngDialogData.requestType;
                var selected_slot = ngDialogData.slotData.user_assigned_slot_id;
                scope.slotData = {
                    'package_name': ngDialogData.packageName,
                    'current_slot': ngDialogData.slotData.item_slot_time,
                    'item_slots': ngDialogData.slotData.item_slots,
                    'items__id': ngDialogData.slotData.items__id,
                    'item_title': ngDialogData.slotData.item_title,
                    'user_assigned_slot_uid': ngDialogData.slotData.user_assigned_slot_uid,
                    'selected_slot': selected_slot.toString()
                };

                /*
                 Submit form action
                -------------------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.customer.write.change_slot',
                        'userId': ngDialogData.userId
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                if (responseData.reaction == 1) {
                                    $rootScope.$broadcast('slot_changed', responseData);
                                }

                                // close dialog
                                $scope.closeThisDialog();

                            });

                        });

                };

                /**
                 * Close dialog
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])

    ;

})(window, window.angular);
;
/*!
*  Component  : Bill
*  File       : BillDataServices.js
*  Engine     : BillServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.BillDataServices', [])
        .service('BillDataService', [
            '$q',
            '__DataStore',
            'appServices',
            BillDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function BillDataService($q, __DataStore, appServices) {


        /*
            Get Details Support Data
            -------------------------------------------------------------- */
        this.getDetails = function (billUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.bill.read.details',
                'billUid': billUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get upcomming Support Data
            -------------------------------------------------------------- */
        this.GetUpcommingBills = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.bill.read.upcomming_bills')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);
                    });
                });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Details Support Data
            -------------------------------------------------------------- */
        this.prepareBillDetails = function (billUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.bill.read.bill_payment_details',
                'billUid': billUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };

        /*
            Get Details Support Data
            -------------------------------------------------------------- */
        this.prepareBillEditData = function (billUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.bill.read.edit_support_data',
                'billUid': billUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };

    };

})(window, window.angular);
;
/*!
*  Component  : Bill
*  File       : Bill.js
*  Engine     : Bill
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.BillEngine', [])


        /**
         * Bill List Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object BillDataService
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('BillListController', [
            '$scope',
            '__DataStore',
            '$state',
            'appServices',
            '$rootScope',
            'BillDataService',
            function ($scope, __DataStore, $state, appServices, $rootScope, BillDataService) {

                var dtColumnsData = [
                        {
                            "name": "bill_uid",
                            "orderable": true,
                            "template": '#billIdTemplate'
                        },
                        {
                            "name": "name",
                            "orderable": true,
                        },
                        {
                            "name": "formatted_status",
                            "orderable": false,
                        },
                        {
                            "name": "bill_period",
                            "orderable": true
                        },
                        {
                            "name": "formated_due_date",
                            "orderable": true
                        },
                        {
                            "name": "formated_amount",
                            "orderable": true
                        },
                        {
                            "name": null,
                            'template': '#billActionTemplate'
                        }
                    ],
                    upcommingDtColumnsData = [
                        {
                            "name": "bill_uid",
                            "orderable": true,
                            "template": '#billIdTemplate'
                        },
                        {
                            "name": "package_title",
                            "orderable": true
                        },
                        {
                            "name": "name",
                            "orderable": true,
                        },
                        {
                            "name": "bill_period",
                            "orderable": true
                        },
                        {
                            "name": "formated_due_date",
                            "orderable": true
                        },
                        {
                            "name": "formated_amount",
                            "orderable": true
                        },
                        {
                            "name": null,
                            'template': '#billActionTemplate'
                        }
                    ],
                    scope = this,
                    tabs = {
                        'general': {
                            id: 'generalBill',
                            route: 'bill.general',
                            status: 1
                        },
                        'upcoming': {
                            id: 'upcomingBill',
                            route: 'bill.upcoming',
                            status: 2
                        },
                        'paid': {
                            id: 'paidBill',
                            route: 'bill.paid',
                            status: 3
                        },
                        'cancelled': {
                            id: 'cancelledBill',
                            route: 'bill.cancelled',
                            status: 4
                        }
                    };

                $('#manageBillsList a').click(function (e) {

                    e.preventDefault();

                    var $this = $(this),
                        tabName = $this.attr('aria-controls'),
                        selectedTab = tabs[tabName];

                    if (!_.isEmpty(selectedTab)) {
                        $(this).tab('show');
                        scope.getBills(selectedTab.id, selectedTab.status);
                    }

                });

                /**
                 * Get general user test as a datatable source object
                 *
                 * @return  void
                 *---------------------------------------------------------- */
                scope.getBills = function (tableID, status) {

                    var visiableColumns,
                        status = parseInt(status);

                    if (status !== 1 && status !== 2) {

                        visiableColumns = [
                            {targets: [2], visible: false}
                        ];

                    } else {

                        visiableColumns = [
                            {targets: '_all', visible: true}
                        ];

                    }

                    if (scope.billDataTable) {
                        scope.billDataTable.destroy();
                    }

                    scope.billDataTable = __DataStore.dataTable('#' + tableID, {
                        url: {
                            apiURL: 'manage.bill.read.list',
                            status: status

                        },
                        dtOptions: {
                            "searching": true,
                            // "columnDefs": visiableColumns,
                            "paginate": (status === 2) ? false : true,
                            'columnDefs': [
                                visiableColumns,
                                {class: 'text-right', targets: 5}
                            ],
                            "rowCallback": function (settings, data) {

                                if (data.currentTab == 1 && data.isOverDue) {

                                    var tr = $(settings);
                                    tr.addClass('lw-overdue');
                                    tr.attr({'title': 'Overdue Bill'})
                                }
                            }
                        },
                        columnsData: (status == 2) ? upcommingDtColumnsData : dtColumnsData,
                        scope: $scope
                    });
                };

                _.defer(function (text) {

                    if ($state.current.name == 'bill.general') {

                        var selectedTab = $('.nav li a[href="#general"]');
                        selectedTab.triggerHandler('click', true);

                    } else if ($state.current.name == 'bill.paid') {

                        var selectedTab = $('.nav li a[href="#paid"]');
                        selectedTab.triggerHandler('click', true);

                    } else if ($state.current.name == 'bill.upcoming') {

                        var selectedTab = $('.nav li a[href="#upcoming"]');
                        selectedTab.triggerHandler('click', true);

                    } else if ($state.current.name == 'bill.cancelled') {

                        var selectedTab = $('.nav li a[href="#cancelled"]');
                        selectedTab.triggerHandler('click', true);
                    }

                }, 0);


                /**
                 * When click on tab so template open in url
                 *
                 * @param  $event
                 * @param  url
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.tabClick = function (event, url) {

                    event.preventDefault();

                    if ($state.current.name != url) {
                        $state.go(url);
                    }
                };

                /*
                Reload current datatable
                ------------------------------------------------------------ */
                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.billDataTable);
                };

                // when add new record
                $rootScope.$on('bill_added_or_updated', function (data) {
                    __pr('bill_added_or_updated');
                    if (data) {
                        scope.reloadDT();
                    }

                });

                /**
                 * Show Details dialog
                 */
                scope.showDetailsDialog = function (billUid) {

                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("bill.details-dialog"),
                        controller: 'BillDetailsController as BillDetailsCtrl',
                        resolve: {
                            GetDetails: function () {
                                return BillDataService.getDetails(billUid);
                            }
                        }
                    }, function (promiseObj) {
                    });

                };

                /**
                 * Show Details dialog
                 */
                scope.showUpcommingBills = function () {

                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("bill.upcomming-details-dialog"),
                        controller: 'ShowUpCommingBillsController as ShowUpCommingBillsCtrl',
                        resolve: {
                            GetUpCommingBills: ['BillDataService', function (BillDataService) {
                                return BillDataService.GetUpcommingBills();
                            }]
                        }
                    }, function (promiseObj) {
                    });

                };

                /**
                 * pay Bill Amount dialog
                 */
                scope.payBillAmount = function (billUid) {

                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("bill.pay-remaining-amount-dialog"),
                        controller: 'PayBillAmountController as PayBillAmountCtrl',
                        resolve: {
                            billPaymentDetails: function () {
                                return BillDataService.prepareBillDetails(billUid);
                            }
                        }
                    }, function (promiseObj) {

                    });

                };

                /**
                 * pay Bill Amount dialog
                 */
                scope.editBill = function (billUid) {

                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("bill.pay-remaining-amount-dialog"),
                        controller: 'PayBillAmountController as PayBillAmountCtrl',
                        resolve: {
                            billPaymentDetails: function () {
                                return BillDataService.prepareBillDetails(billUid);
                            }
                        }
                    }, function (promiseObj) {
                    });

                };

                /**
                 * Bill Cancel
                 *
                 * inject billUid
                 * inject billId
                 *
                 * @return void
                 *---------------------------------------------------------------- */

                scope.cancelBill = function (billUid, billId) {

                    scope.deletingBillId = billId;

                    _.defer(function () {

                        var $lwBillCancelConfirm = $('#lwBillCancelConfirm');
                        scope.deleteText = $lwBillCancelConfirm.attr('data-message');
                        scope.deleteConfirmBtnText = $lwBillCancelConfirm.attr('data-delete-button-text');
                        scope.successMsgText = $lwBillCancelConfirm.attr('success-msg');

                        __globals.showConfirmation({
                            html: scope.deleteText,
                            confirmButtonText: scope.deleteConfirmBtnText
                        }, function () {

                            __DataStore.post({
                                'apiURL': 'manage.bill.write.process_cancel',
                                'billUid': billUid
                            }).success(function (responseData) {

                                var message = responseData.data.message;

                                appServices.processResponse(responseData, {

                                        error: function () {

                                            __globals.showConfirmation({
                                                title: 'Cancelled!',
                                                text: message,
                                                type: 'error'
                                            });
                                        }
                                    },
                                    function () {

                                        __globals.showConfirmation({
                                            title: 'Cancelled!',
                                            text: scope.successMsgText,
                                            type: 'success'
                                        });

                                        scope.reloadDT();   // reload datatable

                                    });

                            });

                        });
                    });
                };


                /**
                 * Bill Delete
                 *
                 * inject billUid
                 * inject billId
                 *
                 * @return void
                 *---------------------------------------------------------------- */
                scope.deleteBill = function (billUid, billId) {

                    scope.deletingBillId = billId;

                    _.defer(function () {

                        var $lwBillCancelConfirm = $('#lwBillDeleteConfirm');
                        scope.deleteText = $lwBillCancelConfirm.attr('data-message');
                        scope.deleteConfirmBtnText = $lwBillCancelConfirm.attr('data-delete-button-text');
                        scope.successMsgText = $lwBillCancelConfirm.attr('success-msg');

                        __globals.showConfirmation({
                            html: scope.deleteText,
                            confirmButtonText: scope.deleteConfirmBtnText
                        }, function () {

                            __DataStore.post({
                                'apiURL': 'manage.bill.write.process_delete',
                                'billUid': billUid
                            }).success(function (responseData) {

                                var message = responseData.data.message;

                                appServices.processResponse(responseData, {

                                        error: function () {

                                            __globals.showConfirmation({
                                                title: 'Deleted!',
                                                text: message,
                                                type: 'error'
                                            });
                                        }
                                    },
                                    function () {

                                        __globals.showConfirmation({
                                            title: 'Deleted!',
                                            text: scope.successMsgText,
                                            type: 'success'
                                        });

                                        scope.reloadDT();   // reload datatable

                                    });

                            });

                        });

                    });
                };

            }
        ])
        // Bill List Controller ends here


        /**
         * Bill Add Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('BillAddController', [
            '$scope',
            '$state',
            '$stateParams',
            '__Form',
            'appServices',
            'GetUserPurchases',
            function ($scope, $state, $stateParams, __Form, appServices, GetUserPurchases) {
                var scope = this;

                scope.showLoader = true;
                scope.userPurchases = [];

                scope = __Form.setup(scope, 'bill_form', 'billData', {
                    modelUpdateWatcher: true,
                });

                scope.users = GetUserPurchases.users;
                scope.purchases = GetUserPurchases.purchases;

                scope.billData.items = [];
                scope.billData.total_bill_amount = 0;
                scope.billData.tax_amount = 0;
                scope.billData.discount_amount = 0;

                scope.usersSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: 'member_id',
                    labelField: 'memberName',
                    searchField: ['memberName']
                });

                // scope.packageSelectizeConfig = __globals.getSelectizeOptions({
                //     valueField: '_id',
                //     labelField: 'title',
                //     searchField: ['title']
                // });
                scope.packageSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: 'user_packages_id',
                    labelField: 'title',
                    searchField: ['title']
                });


                scope.billData.bill_period_from = moment().format('YYYY-MM-DD');
                scope.billData.bill_period_to = scope.billData.bill_period_from;
                scope.billData.due_date = moment().format('YYYY-MM-DD');

                scope.bill_period_from = {
                    minDate: scope.billData.bill_period_from
                };

                scope.bill_period_to = {
                    minDate: scope.billData.bill_period_from
                };

                scope.due_date = {
                    minDate: scope.billData.bill_period_from,
                    maxDate: scope.billData.bill_period_to
                };

                /**
                 * On select user
                 */
                scope.onSelectUser = function (selectedUser) {
                    scope.userPurchases = scope.purchases[selectedUser];
                };

                /**
                 * On selected purchase
                 */
                scope.onSelectedPurchase = function (selectedPurchase) {
                    _.each(scope.userPurchases, function (row, index) {

                        if (row.user_packages_id == selectedPurchase) {

                            scope.billData.items = [{
                                short_description: row.short_description,
                                amount: parseFloat(row.price),
                                user_packages__id: row.user_packages_id
                            }];

                            scope.billData.customer_user_authorities__id = row.user_authorities__id;

                            scope = __Form.updateModel(scope, scope.billData);
                        }
                    });
                };

                /**
                 * Add New Row of slots
                 */
                scope.addNewRow = function () {
                    var newItemNo = scope.billData.items.length + 1;

                    scope.billData.items.push({
                        rowId: 'row' + newItemNo,
                        amount: parseFloat(0)
                    });
                };

                /**
                 * Remove row
                 */
                scope.removeRow = function (index) {

                    // remove the row specified in index
                    scope.billData.items.splice(index, 1);

                    // if no rows left in the array create a blank array
                    if (scope.billData.items.length === 0 || scope.billData.items.length == null) {
                        // alert('no rec');
                        scope.billData.items.push = [];
                    }

                };

                /**
                 * Calculate item total amount
                 */
                scope.itemTotal = function () {

                    var total = 0;

                    angular.forEach(scope.billData.items, function (item) {
                        total += parseFloat(item.amount);
                    })

                    scope.billData.total_item_amount = total;

                    return isNaN(total) ? 0 : __globals.parseAmount(total);
                }

                /**
                 * Calculate Final Amount
                 */
                scope.calculateFinalAmount = function () {

                    var itemAmount = scope.billData.total_item_amount;

                    itemAmount = (itemAmount - scope.billData.discount_amount);

                    itemAmount += scope.billData.tax_amount;

                    parseFloat(itemAmount);

                    scope.billData.total_bill_amount = itemAmount;

                    return isNaN(itemAmount) ? 0 : __globals.parseAmount(itemAmount);
                }


                $scope.$watch('BillAddCtrl.billData.bill_period_from', function (currentValue, oldValue) {

                    var $element = angular.element('#bill_period_to');

                    if (scope.billData.bill_period_to < scope.billData.bill_period_from) {

                        scope.billData.bill_period_to = currentValue;
                    }

                    if ((scope.billData.due_date > scope.billData.bill_period_to) || (scope.billData.due_date < scope.billData.bill_period_from)) {
                        scope.billData.due_date = scope.billData.bill_period_from;
                    }


                    var $dueElement = angular.element('#due_date');

                    $dueElement.bootstrapMaterialDatePicker('setMinDate', scope.billData.bill_period_from);
                    $dueElement.bootstrapMaterialDatePicker('setMaxDate', scope.billData.bill_period_to);

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });


                $scope.$watch('BillAddCtrl.billData.bill_period_to', function (currentValue, oldValue) {

                    var $element = angular.element('#bill_period_from');

                    if (scope.billData.bill_period_to < scope.billData.bill_period_from) {

                        scope.billData.bill_period_from = currentValue;
                    }

                    if ((scope.billData.due_date > scope.billData.bill_period_to) || (scope.billData.due_date < scope.billData.bill_period_from)) {

                        scope.billData.due_date = scope.billData.bill_period_from;
                    }

                    var $dueElement = angular.element('#due_date');

                    $dueElement.bootstrapMaterialDatePicker('setMinDate', scope.billData.bill_period_from);
                    $dueElement.bootstrapMaterialDatePicker('setMaxDate', scope.billData.bill_period_to);

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process('manage.bill.write.generate', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {
                                $state.go('bill.general');
                            });

                        });
                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
        // ItemAddController ends here

        /**
         * Bill Edit Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('BillEditController', [
            '$scope',
            '$state',
            '$stateParams',
            '__Form',
            'appServices',
            'BillEditData',
            function ($scope, $state, $stateParams, __Form, appServices, BillEditData) {

                var scope = this;
                scope.showLoader = true;

                scope = __Form.setup(scope, 'bill_form', 'billData', {
                    modelUpdateWatcher: true,
                    secured: true
                });

                scope = __Form.updateModel(scope, BillEditData.data);

                scope.billData.total_bill_amount = 0;

                scope.due_date = {
                    minDate: scope.billData.bill_period_from,
                    maxDate: scope.billData.bill_period_to
                };

                /**
                 * Add New Row of slots
                 */
                scope.addNewRow = function () {

                    var newItemNo = scope.billData.items.length + 1;

                    scope.billData.items.push({
                        rowId: 'row' + newItemNo,
                        amount: parseFloat(0),
                        user_packages__id: BillEditData.data.user_packages__id
                    });
                };


                scope.billData.item_to_remove = [];

                /**
                 * Remove row
                 */
                scope.removeRow = function (index, bill_item_id) {

                    // remove the row specified in index
                    scope.billData.items.splice(index, 1);

                    // if no rows left in the array create a blank array
                    if (scope.billData.items.length === 0 || scope.billData.items.length == null) {
                        // alert('no rec');
                        scope.billData.items.push = [];
                    }

                    if (!_.isUndefined(bill_item_id)) {
                        scope.billData.item_to_remove.push(bill_item_id);
                    }
                };

                /**
                 * Calculate item total amount
                 */
                scope.itemTotal = function () {

                    var total = 0;

                    angular.forEach(scope.billData.items, function (item) {
                        total += parseFloat(item.amount);
                    })

                    scope.billData.total_item_amount = total;

                    return isNaN(total) ? 0 : __globals.parseAmount(total);
                }

                /**
                 * Calculate Final Amount
                 */
                scope.calculateFinalAmount = function () {

                    var itemAmount = scope.billData.total_item_amount;

                    itemAmount = (itemAmount - scope.billData.discount_amount) + scope.billData.tax_amount;

                    parseFloat(itemAmount);

                    scope.billData.total_bill_amount = itemAmount;

                    return isNaN(itemAmount) ? 0 : __globals.parseAmount(itemAmount);
                }

                $scope.$watch('BillEditCtrl.billData.bill_period_from', function (currentValue, oldValue) {

                    var $element = angular.element('#bill_period_to');

                    if (scope.billData.bill_period_to < scope.billData.bill_period_from) {

                        scope.billData.bill_period_to = currentValue;
                    }

                    if ((scope.billData.due_date > scope.billData.bill_period_to) || (scope.billData.due_date < scope.billData.bill_period_from)) {
                        scope.billData.due_date = scope.billData.bill_period_from;
                    }


                    var $dueElement = angular.element('#due_date');

                    $dueElement.bootstrapMaterialDatePicker('setMinDate', scope.billData.bill_period_from);
                    $dueElement.bootstrapMaterialDatePicker('setMaxDate', scope.billData.bill_period_to);

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });


                $scope.$watch('BillEditCtrl.billData.bill_period_to', function (currentValue, oldValue) {

                    var $element = angular.element('#bill_period_from');

                    if (scope.billData.bill_period_to < scope.billData.bill_period_from) {

                        scope.billData.bill_period_from = currentValue;
                    }

                    if ((scope.billData.due_date > scope.billData.bill_period_to) || (scope.billData.due_date < scope.billData.bill_period_from)) {

                        scope.billData.due_date = scope.billData.bill_period_from;
                    }

                    var $dueElement = angular.element('#due_date');

                    $dueElement.bootstrapMaterialDatePicker('setMinDate', scope.billData.bill_period_from);
                    $dueElement.bootstrapMaterialDatePicker('setMaxDate', scope.billData.bill_period_to);

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.bill.write.update',
                        'billUid': $stateParams.billUid
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {
                                //$state.go('bill.upcoming');
                            });

                        });
                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
        // BillEditController ends here


        /**
         * Bill Add Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('GenerateUserPackageBillController', [
            '$scope',
            '$state',
            '__Form',
            'appServices',
            '__Utils',
            'GetUserPurchases',
            function ($scope, $state, __Form, appServices, __Utils, GetUserPurchases) {

                var scope = this;

                scope.showLoader = true;
                scope.userPurchases = [];

                scope = __Form.setup(scope, 'bill_form', 'billData', {
                    modelUpdateWatcher: true,
                });

                scope.currentstate = $state.current.name;

                scope = __Form.updateModel(scope, GetUserPurchases.user_package);

                scope.recentBill = GetUserPurchases.recent_bill;

                scope.billData.items = GetUserPurchases.items;

                scope.bill_period_from = {
                    minDate: scope.billData.bill_period_from
                };

                scope.bill_period_to = {
                    minDate: scope.billData.bill_period_from
                };

                scope.due_date = {
                    minDate: scope.billData.bill_period_from,
                    maxDate: scope.billData.bill_period_to
                };

                /**
                 * Add New Row of slots
                 */
                scope.addNewRow = function () {

                    var newItemNo = scope.billData.items.length + 1;

                    scope.billData.items.push({
                        rowId: 'row' + newItemNo,
                        amount: parseFloat(0),
                        user_packages__id: GetUserPurchases.user_package.user_packages_id
                    });
                };

                /**
                 * Remove row
                 */
                scope.removeRow = function (index) {

                    // remove the row specified in index
                    scope.billData.items.splice(index, 1);

                    // if no rows left in the array create a blank array
                    if (scope.billData.items.length === 0 || scope.billData.items.length == null) {
                        // alert('no rec');
                        scope.billData.items.push = [];
                    }

                };

                /**
                 * Calculate item total amount
                 */
                scope.itemTotal = function () {

                    var total = 0;

                    angular.forEach(scope.billData.items, function (item) {
                        total += parseFloat(item.amount);
                    })

                    scope.billData.total_item_amount = total;

                    return isNaN(total) ? 0 : __globals.parseAmount(total);
                }

                /**
                 * Calculate Final Amount
                 */
                scope.calculateFinalAmount = function () {

                    var itemAmount = scope.billData.total_item_amount;

                    itemAmount = (itemAmount - scope.billData.discount_amount) + scope.billData.tax_amount;

                    parseFloat(itemAmount);

                    scope.billData.total_bill_amount = itemAmount;

                    return isNaN(itemAmount) ? 0 : __globals.parseAmount(itemAmount);
                }

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.payLater = function () {

                    __Form.process('manage.bill.write.generate', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function (reaction) {

                                __globals.showConfirmation({
                                    title: responseData.data.message,
                                    type: 'success',
                                    confirmButtonColor: "#337ab7",
                                    html: __Utils.template('#succesInformationAlert', {
                                        'bill_data': responseData.data.bill_data,
                                        'customer_name': GetUserPurchases.customer_info.name
                                    }),
                                    confirmButtonText: 'Ok',
                                    showCancelButton: false,
                                    confirmButtonClass: 'btn btn-primary',
                                    cancelButtonClass: 'btn btn-gray',
                                    buttonsStyling: false,
                                }, function () {
                                    $state.go('bill.general');
                                });

                            });

                        });
                };

                scope.isGenerateBill = false;
                scope.billResponse = '';

                scope.showPayDialog = function (responseData) {

                    appServices.showDialog({
                        'message': responseData.message,
                        'bill_data': responseData.bill_data,
                        'customer_name': GetUserPurchases.customer_info.name,
                        'requestFrom': 'bill'
                    }, {
                        templateUrl: __globals.getTemplateURL("bill.pay-remaining-amount-dialog"),
                        controller: 'PayBillAmountController as PayBillAmountCtrl',
                        resolve: {
                            billPaymentDetails: ['BillDataService', function (BillDataService) {
                                return BillDataService.prepareBillDetails(responseData.bill_data.bill_uid);
                            }]
                        }
                    }, function (promiseObj) {
                    });
                };

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.pay = function () {

                    if (scope.billResponse) {
                        scope.showPayDialog(scope.billResponse);
                    } else {

                        __Form.process('manage.bill.write.generate', scope)
                            .success(function (responseData) {

                                appServices.processResponse(responseData, null, function () {

                                    scope.isGenerateBill = true;
                                    scope.billResponse = responseData.data;

                                    scope.showPayDialog(scope.billResponse);

                                });

                            });
                    }
                };


                $scope.$watch('BillAddCtrl.billData.bill_period_from', function (currentValue, oldValue) {

                    var $element = angular.element('#bill_period_to');

                    if (scope.billData.bill_period_to < scope.billData.bill_period_from) {

                        scope.billData.bill_period_to = currentValue;
                    }

                    if ((scope.billData.due_date > scope.billData.bill_period_to) || (scope.billData.due_date < scope.billData.bill_period_from)) {
                        scope.billData.due_date = scope.billData.bill_period_from;
                    }


                    var $dueElement = angular.element('#due_date');

                    $dueElement.bootstrapMaterialDatePicker('setMinDate', scope.billData.bill_period_from);
                    $dueElement.bootstrapMaterialDatePicker('setMaxDate', scope.billData.bill_period_to);

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });


                $scope.$watch('BillAddCtrl.billData.bill_period_to', function (currentValue, oldValue) {

                    var $element = angular.element('#bill_period_from');

                    if (scope.billData.bill_period_to < scope.billData.bill_period_from) {

                        scope.billData.bill_period_from = currentValue;
                    }

                    if ((scope.billData.due_date > scope.billData.bill_period_to) || (scope.billData.due_date < scope.billData.bill_period_from)) {

                        scope.billData.due_date = scope.billData.bill_period_from;
                    }

                    var $dueElement = angular.element('#due_date');

                    $dueElement.bootstrapMaterialDatePicker('setMinDate', scope.billData.bill_period_from);
                    $dueElement.bootstrapMaterialDatePicker('setMaxDate', scope.billData.bill_period_to);

                    // Check if currentValue exist
                    if (_.isEmpty(currentValue)) {
                        $element.bootstrapMaterialDatePicker('setMinDate', '');
                    } else {
                        $element.bootstrapMaterialDatePicker('setMinDate', currentValue);
                    }

                });
            }
        ])
        // BillAddController ends here


        /**
         * Bill Details Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('BillDetailsController', [
            '$scope',
            'GetDetails',
            function ($scope, GetDetails) {

                var scope = this;

                scope.details = GetDetails.data;

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])
        // BillDetailsController ends here

        /**
         * Pay Bill Amount Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('PayBillAmountController', [
            '$scope',
            '$state',
            '__Form',
            'appServices',
            '__Utils',
            '$rootScope',
            'billPaymentDetails',
            function ($scope, $state, __Form, appServices, __Utils, $rootScope, billPaymentDetails) {

                var scope = this;
                scope.data = $scope.ngDialogData;

                scope.paymentData = billPaymentDetails.paymentData;
                scope.payment_methods = billPaymentDetails.payment_methods;

                scope = __Form.setup(scope, 'pay_form', 'payData', {
                    secured: true,
                    modelUpdateWatcher: true,
                });

                scope.paymentMethodSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                scope = __Form.updateModel(scope, billPaymentDetails.paymentData);

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process({
                        'apiURL': 'manage.transaction.write.process_pay_bill',
                        'billUid': scope.paymentData.bill_uid
                    }, scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {

                                var requestData = responseData.data;

                                $rootScope.$broadcast('bill_added_or_updated', true);

                                scope.closeDialog();

                                __globals.showConfirmation({
                                    title: responseData.data.message,
                                    type: 'success',
                                    confirmButtonColor: "#337ab7",
                                    html: __Utils.template('#succesInformationOfPaymentAlert', responseData.data),
                                    confirmButtonText: 'Ok',
                                    showCancelButton: false,
                                    confirmButtonClass: 'btn btn-primary',
                                    cancelButtonClass: 'btn btn-gray',
                                    buttonsStyling: false,
                                }, function () {
                                    $state.go('bill.general');

                                });
                            });

                        });
                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])


        /**
         * ShowUpCommingBillsController
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('ShowUpCommingBillsController', [
            '$scope',
            'GetUpCommingBills',
            function ($scope, GetUpCommingBills) {

                var scope = this;

                scope.details = GetUpCommingBills.data;

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */
                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };

            }
        ])
    // BillDetailsController ends here

    ;

})(window, window.angular);
;
/*!
*  Component  : Transaction
*  File       : TransactionDataServices.js
*  Engine     : TransactionServices
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.TransactionDataServices', [])
        .service('TransactionDataService', [
            '$q',
            '__DataStore',
            'appServices',
            TransactionDataService
        ])

    /*!
         This service use for to get the promise on data
        ----------------------------------------------------------------------------- */

    function TransactionDataService($q, __DataStore, appServices) {


        /*
            Get Add Support Data
            -------------------------------------------------------------- */
        this.getAddSupportData = function () {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch('manage.transaction.read.support_data')
                .success(function (responseData) {

                    appServices.processResponse(responseData, null, function (reactionCode) {

                        //this method calls when the require
                        //work has completed successfully
                        //and results are returned to client
                        defferedObject.resolve(responseData.data);
                    });
                });

            //return promise to caller
            return defferedObject.promise;
        };
        /*
            Get Edit Support Data
            -------------------------------------------------------------- */
        this.getEditSupportData = function (transactionIdOrUid) {

            //create a differed object
            var defferedObject = $q.defer();

            __DataStore.fetch({
                'apiURL': 'manage.transaction.read.update.data',
                'transactionIdOrUid': transactionIdOrUid
            }).success(function (responseData) {

                appServices.processResponse(responseData, null, function (reactionCode) {

                    //this method calls when the require
                    //work has completed successfully
                    //and results are returned to client
                    defferedObject.resolve(responseData.data);
                });
            });

            //return promise to caller
            return defferedObject.promise;
        };


    };

})(window, window.angular);
;
/*!
*  Component  : Transaction
*  File       : Transaction.js
*  Engine     : Transaction
----------------------------------------------------------------------------- */

(function (window, angular, undefined) {

    'use strict';

    angular
        .module('app.TransactionEngine', [])


        /**
         * Transaction List Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         * inject object TransactionDataService
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('TransactionListController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'TransactionDataService',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, TransactionDataService) {

                var dtColumnsData = [
                        {
                            "name": "txn_id",
                            "orderable": true,
                        },
                        {
                            "name": "bill_uid",
                            "orderable": true,
                        },
                        {
                            "name": "amount",
                            "orderable": true,
                        },
                        {
                            "name": "created_at",
                            "orderable": true,
                        },
                        {
                            "name": "updated_at",
                            "orderable": true,
                        },
                        {
                            "name": "formatted_type",
                            "orderable": false,
                        },
                        {
                            "name": "formatted_status",
                            "orderable": false,
                        },
                        {
                            "name": "formatted_subtype",
                            "orderable": false,
                        }
                    ],
                    scope = this;

                /**
                 * Get general user test as a datatable source object
                 *
                 * @return  void
                 *---------------------------------------------------------- */

                scope.transactionDataTable = __DataStore.dataTable('#lwTransactionList', {
                    url: 'manage.transaction.read.list',
                    dtOptions: {
                        "searching": true,
                        'columnDefs': [
                            {class: 'text-right', targets: 2}
                        ],
                    },
                    columnsData: dtColumnsData,
                    scope: $scope
                });

                /*
            Reload current datatable
            ------------------------------------------------------------ */
                scope.reloadDT = function () {
                    __DataStore.reloadDT(scope.transactionDataTable);
                };

                // when add new record
                $scope.$on('package_added_or_updated', function (data) {

                    if (data) {
                        scope.reloadDT();
                    }

                });

                /*
            add dialog
            ------------------------------------------------------------ */
                scope.openAddDialog = function () {

                    appServices.showDialog(scope, {
                        templateUrl: __globals.getTemplateURL("transaction.add-dialog"),
                        controller: 'TransactionAddController as transactionAddCtrl',
                        resolve: {
                            transactionAddData: function () {
                                return TransactionDataService
                                    .getAddSupportData();
                            }
                        }
                    }, function (promiseObj) {

                        if (_.has(promiseObj.value, 'transaction_added_or_updated')
                            && promiseObj.value.transaction_added_or_updated) {

                            scope.reloadDT();
                        }

                    });

                };
            }
        ])
        // Transaction List Controller ends here


        /**
         * Transaction Controller
         *
         * inject object $scope
         * inject object __DataStore
         * inject object __Form
         * inject object $state
         * inject object appServices
         * inject object $rootScope
         *
         * @return  void
         *---------------------------------------------------------------- */

        .controller('TransactionAddController', [
            '$scope',
            '__DataStore',
            '__Form',
            '$state',
            'appServices',
            '$rootScope',
            'transactionAddData',
            function ($scope, __DataStore, __Form, $state, appServices, $rootScope, transactionAddData) {

                var scope = this;

                scope.showLoader = true;

                scope = __Form.setup(scope, 'transaction_form', 'transactionData', {
                    secured: true,
                    unsecuredFields: []
                });

                var requestData = transactionAddData;
                scope.types = requestData.transactionsConfig.types;
                scope.subtypes = requestData.transactionsConfig.subtypes;
                scope.payment_methods = requestData.transactionsConfig.payment_methods;
                // scope.bills = requestData.bills;
                scope.users = requestData.users;

                scope.billSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: '_id',
                    labelField: 'bill_no',
                    searchField: ['bill_no']
                });

                scope.usersSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: 'member_id',
                    labelField: 'memberName',
                    searchField: ['memberName']
                });

                scope.subtypeSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                scope.paymentMethodSelectizeConfig = __globals.getSelectizeOptions({
                    valueField: 'id',
                    labelField: 'title',
                    searchField: ['title']
                });

                /**
                 * Get Slected user bills whech is unpaid or partial
                 */
                scope.getUserBills = function (memberId) {

                    if (!memberId || _.isUndefined(memberId)) {
                        scope.transactionData.amount = 0;
                        scope.selectedBillData = [];
                    }

                    if (!_.isUndefined(memberId)) {
                        __DataStore.fetch({
                            'apiURL': 'manage.transaction.read.user_bills',
                            'memberId': memberId
                        }).success(function (responseData) {

                            appServices.processResponse(responseData, null, function (reactionCode) {
                                var requestData = responseData.data;
                                scope.bills = requestData.bills;
                            });

                        });

                    } else {
                        scope.bills = [];
                    }

                }

                scope.selectedBillData = [];

                /**
                 *
                 */
                scope.onSelectBill = function (bilId) {

                    if (!bilId || _.isUndefined(bilId)) {
                        scope.transactionData.amount = 0;
                        scope.selectedBillData = [];
                    }

                    _.each(scope.bills, function (item) {

                        if (item._id == bilId) {
                            scope.selectedBillData = item;
                        }

                    });

                }

                scope.onSelectType = function (type) {
                    if (scope.selectedBillData) {

                        if (type == 1) {
                            scope.transactionData.amount = scope.selectedBillData.balance_amount;
                        } else if (type == 2) {
                            scope.transactionData.amount = scope.selectedBillData.paid_amount;
                        }

                    } else {

                        scope.transactionData.amount = 0;
                    }

                }

                /**
                 * Submit form
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.submit = function () {

                    __Form.process('manage.transaction.write.create', scope)
                        .success(function (responseData) {

                            appServices.processResponse(responseData, null, function () {
                                $scope.closeThisDialog({'transaction_added_or_updated': true});
                            });

                        });
                };

                /**
                 * Close dialog
                 *
                 * @return  void
                 *---------------------------------------------------------------- */

                scope.closeDialog = function () {
                    $scope.closeThisDialog();
                };
            }
        ])
    // TransactionAddController ends here
    ;

})(window, window.angular);
//# sourceMappingURL=../source-maps/manage-app.src.js.map
